/*!
 * Fuse JavaScript Framework
 *
 * Copyright (c) 2008-2009 John-David Dalton
 * Redistributable under the MIT License,
 * see: http://www.fusejs.com/license
 *
 * Version: 0.1
 * Release: 20090804
 */
(function(global){
  global.Fuse = (function() {
    function Fuse() { }
    return Fuse;
  })();

  Fuse._body =
  Fuse._root =
  Fuse._scrollEl = null;

  Fuse._doc =   document;
  Fuse._div =   Fuse._doc.createElement('div');
  Fuse._docEl = Fuse._doc.documentElement;

  Fuse.JSONFilter =     /^\/\*-secure-([\s\S]*)\*\/\s*$/;
  Fuse.ScriptFragment = '<script[^>]*>([^\\x00]*?)<\/script>';
  Fuse.Version =        '@FUSEJS_VERSION';

  Fuse.emptyFunction = (function() {
    function emptyFunction() { }
    return emptyFunction;
  })();

  Fuse.K = (function() {
    function K(x) { return x; }
    return K;
  })();

  Fuse._info = {
    'body':  { 'nodeName': 'BODY', 'property': 'body' },
    'docEl': { 'nodeName': 'HTML', 'property': 'documentElement' }
  };

  Fuse._info.root = Fuse._info.docEl;
  Fuse._info.scrollEl = Fuse._info.body;

  /*----------------------- PRIVATE VARIABLES/METHODS ------------------------*/

  var $, Bug, Feature,
   expando = '_fuse' + String(1 * new Date).slice(0, 10),
   slice = Array.prototype.slice,
   userAgent = global.navigator.userAgent;

  // Host objects have a range of typeof values. For example:
  // document.createElement('div').offsetParent -> unknown
  // document.createElement -> object
  function isHostObject(object, property) {
    var type = typeof object[property];
    return type === 'function' || type === 'object' || type === 'unknown';
  }

  function getDocument(element) { // assume element is not null
    // Check for `ownerDocument` first because an element of a document fragment
    // will have a `document` property that is NOT the pages document object.
    return element.ownerDocument || element.document ||
      (element.nodeType === 9 ? element : Fuse._doc);
  }

  function concatList(list, otherList) {
    var pad = list.length, length = otherList.length;
    while (length--) list[pad + length] = otherList[length];
    return list;
  }

  function prependList(list, value, results) {
    // allow a pre-sugared array to be passed
    (results = results || [])[0] = value;
    var length = list.length;
    while (length--) results[1 + length] = list[length];
    return results;
  }

  var getNodeName = (function() {
    var getNodeName =
      function getNodeName(element) { return element.nodeName.toUpperCase(); };
    if (Fuse._docEl.nodeName === 'HTML')
      getNodeName = function getNodeName(element) { return element.nodeName; };
    return getNodeName;
  })();

  /* Based on work by Diego Perini */
  var getWindow = (function() {
    var getWindow = function getWindow(element) {
      // Safari 2.0.x returns `Abstract View` instead of `global`
      var frame, i = 0, doc = getDocument(element);
      if (Fuse._doc !== doc) {
        while ((frame = global.frames[i++])) {
          if (frame.document === doc) return frame;
        }
      }
      return global;
    };

    if (isHostObject(Fuse._doc, 'parentWindow')) {
      getWindow = function getWindow(element) {
        return getDocument(element).parentWindow || element;
      };
    } else if (isHostObject(document, 'defaultView') && Fuse._doc.defaultView === global) {
      getWindow = function getWindow(element) {
        return getDocument(element).defaultView || element;
      };
    }
    return getWindow;
  })();

  /*--------------------------- NAMESPACE UTILITY ----------------------------*/

  Fuse.addNS = function(path) {
    var part, object = this, propIndex = 0, parts = path.split('.'),
     length = parts.length, i = 0, properties = slice.call(arguments, 1);

    // if parent is passed then incriment the propIndex by 1
    if (typeof properties[0] === 'function') propIndex++;
    properties[propIndex] = properties[propIndex] || { };

    while ((part = parts[i++])) {
      if (object[part]) {
        object = object[part];
      } else {
        if (i === length) {
          // if no parent pass prepend object as parent
          if (!propIndex) properties = prependList(properties, object);
          object = object[part] = Fuse.Class.apply(global,
            Fuse.Object.hasKey(properties[1], 'constructor') ? properties :
              (properties[1].constructor = part) && properties);
        }
        else object = object[part] = Fuse.Class(object, { 'constructor': part });
      }
    }
    return object;
  };

  /*----------------------------- BROWSER OBJECT -----------------------------*/

  Fuse.Browser = {
    Agent: {
      'IE': isHostObject(global, 'attachEvent') && userAgent.indexOf('Opera') < 0,
      'Opera': userAgent.indexOf('Opera') > -1,
      'WebKit': userAgent.indexOf('AppleWebKit/') > -1,
      'Gecko': userAgent.indexOf('Gecko') > -1 && userAgent.indexOf('KHTML') < 0,
      'MobileSafari': !!userAgent.match(/AppleWebKit.*Mobile/)
    }
  };

  /*--------------------------- FEATURE/BUG TESTER ---------------------------*/

  (function() {
    function createTester(name) {
      var Tester = new Function('', [
        'function ' + name + '() {',
        'var title, o = ' + name + '._object, i = 0;',
      	'while (title = arguments[i++]) {',
        'if (typeof o[title] === "function") o[title] = o[title]();',
        'if (o[title] !== true) return false;',
        '}', 'return true;',
        '}', 'return ' + name].join('\n'))();

      Tester.set = function(name, value) {
        var o = this._object;
        if (typeof name === 'object'){
          for (var i in name) o[i] = name[i];
        } else o[name] = value;
      };

      Tester.unset = function(name) {
        var o = this._object;
        name = name.valueOf();
        if (typeof name === 'string') delete o[name];
        else {
          for (var i in name) delete o[i];
        }
      };

      Tester._object = { };
      return Tester;
    }

    Bug = Fuse.Browser.Bug = createTester('Bug');
    Feature = Fuse.Browser.Feature = createTester('Feature');
  })();

  /*---------------------------- BROWSER FEATURES ----------------------------*/

  Feature.set({
    'ACTIVE_X_OBJECT': function() {
      // true for IE
      return isHostObject(global, 'ActiveXObject');
    },

    'CREATE_ELEMENT_WITH_HTML': function() {
      try { // true for IE
        var div = Fuse._doc.createElement('<div id="x">');
        return div.id === 'x';
      } catch(e) {
        return false;
      }
    },

    'DOCUMENT_ALL_COLLECTION': function() {
      // true for all but Firefox
      isHostObject(Fuse._doc, 'all');
    },

    'DOCUMENT_CREATE_EVENT': function() {
      // true for all but IE
      return isHostObject(Fuse._doc, 'createEvent');
    },

    'DOCUMENT_CREATE_EVENT_OBJECT': function() {
      // true for IE
      return isHostObject(Fuse._doc, 'createEventObject')
    },

    'DOCUMENT_RANGE': function(){
      // true for all but IE
      return isHostObject(Fuse._doc, 'createRange');
    },

    'DOCUMENT_RANGE_CREATE_CONTEXTUAL_FRAGMENT': function() {
      if (Feature('DOCUMENT_RANGE'))
        return isHostObject(Fuse._doc.createRange(), 'createContextualFragment');
    },

    'DOCUMENT_STYLE_SHEETS_COLLECTION': function() {
      // true for all so far
      return isHostObject(Fuse._doc, 'styleSheets');
    },

    'ELEMENT_ADD_EVENT_LISTENER': function() {
      // true for all but IE
      return isHostObject(Fuse._doc, 'addEventListener');
    },

    'ELEMENT_ATTACH_EVENT': function() {
      // true for IE
      return isHostObject(Fuse._doc, 'attachEvent') &&
        !Feature('ELEMENT_ADD_EVENT_LISTENER');
    },

    'ELEMENT_BOUNDING_CLIENT_RECT': function() {
      // true for IE, Firefox 3
      return isHostObject(Fuse._docEl, 'getBoundingClientRect');
    },

    'ELEMENT_CLASS': function() {
      // true for all but Safari 2 and IE7-
      return isHostObject(global, 'Element') &&
        isHostObject(global.Element, 'prototype');
    },

    'ELEMENT_COMPUTED_STYLE': function() {
      // true for all but IE
      return isHostObject(Fuse._doc, 'defaultView') &&
        isHostObject(Fuse._doc.defaultView, 'getComputedStyle');
    },

    'ELEMENT_COMPARE_DOCUMENT_POSITION': function() {
      // true for Firefox and Opera 9.5+
      return isHostObject(Fuse._docEl, 'compareDocumentPosition');
    },

    'ELEMENT_CURRENT_STYLE': function() {
      // true for IE
      return isHostObject(Fuse._docEl, 'currentStyle') &&
        !Feature('ELEMENT_COMPUTED_STYLE');
    },

    'ELEMENT_DISPATCH_EVENT': function() {
      // true for all but IE
      return isHostObject(Fuse._docEl, 'dispatchEvent');
    },

    'ELEMENT_DO_SCROLL': function() {
      // true for IE
      return isHostObject(Fuse._docEl, 'doScroll');
    },

    'ELEMENT_EXTENSIONS': function() {
      // true for all but Safari 2 and IE7-
      return Feature('HTML_ELEMENT_CLASS') || Feature('ELEMENT_CLASS');
    },

    'ELEMENT_FIRE_EVENT': function() {
      // true for IE
      return isHostObject(Fuse._docEl, 'fireEvent');
    },

    'ELEMENT_GET_ATTRIBUTE_IFLAG': function() {
      // true for IE
      var div = Fuse._div, result = false;
      try {
        div.setAttribute('align', 'center'); div.setAttribute('aLiGn', 'left');
        result = (div.getAttribute('aLiGn') === 'center' &&
          div.getAttribute('aLiGn', 1) === 'left');
        div.removeAttribute('align'); div.removeAttribute('aLiGn');
      } catch(e) { }
      return result;
    },

    'ELEMENT_INNER_TEXT': function() {
      // true for IE
      return !Feature('ELEMENT_TEXT_CONTENT') &&
        typeof Fuse._div.innerText === 'string';
    },

    'ELEMENT_MS_CSS_FILTERS': function() {
      // true for IE
      return isHostObject(Fuse._docEl, 'filters') &&
        typeof Fuse._docEl.style.filter === 'string' &&
        typeof Fuse._docEl.style.opacity !== 'string';
    },

    'ELEMENT_REMOVE_NODE': function() {
      // true for IE
      return isHostObject(Fuse._docEl, 'removeNode');
    },

    'ELEMENT_SOURCE_INDEX': function() {
      // true for IE and Opera
      return typeof Fuse._docEl.sourceIndex === 'number';
    },

    'ELEMENT_SPECIFIC_EXTENSIONS': function() {
      var docEl = Fuse._docEl, result = false;
      if (isHostObject(global, 'HTMLHtmlElement') &&
          isHostObject(global.HTMLHtmlElement, 'prototype') &&
         (docEl.constructor === HTMLHtmlElement || 
          docEl instanceof HTMLHtmlElement || Feature('OBJECT__PROTO__') &&
          docEl['__proto__'] === HTMLHtmlElement.prototype)) {
        result = true;
      } else result = Feature('EMULATE_ELEMENT_CLASSES_WITH_PROTO');

      // TODO: Remove this browser sniff
      return Fuse.Browser.Agent.MobileSafari ? false : result;
    },

    'ELEMENT_TEXT_CONTENT': function() {
      // true for all but IE and Safari 2
      return typeof Fuse._div.textContent === 'string';
    },

    'EMULATE_ELEMENT_CLASSES_WITH_PROTO': function() {
      return Feature('OBJECT__PROTO__') &&
        Fuse._div['__proto__'] !== Fuse._docEl['__proto__'];
    },

    'FUNCTION_TO_STRING_RETURNS_SOURCE': function() {
      // true for all but some mobile browsers
      function toStringTest(param1, param2) { var x = 1 }
      var source = toStringTest.toString();
      return source.indexOf('param1') > -1 && source.indexOf('x = 1') > -1;
    },

    'HTML_ELEMENT_CLASS': function() {
      // true for all but IE
      // (Safari 2 support is emulated in element.js)
      return isHostObject(global,'HTMLElement') &&
        isHostObject(global.HTMLElement, 'prototype');
    },

    'OBJECT__PROTO__': function() {
      // true for Gecko and Webkit
      if (isHostObject(Fuse._docEl, '__proto__') &&
          [ ]['__proto__'] === Array.prototype  &&
          { }['__proto__'] === Object.prototype) {
        // test if it's writable and restorable
        var result, list = [], backup = list['__proto__'];
        list['__proto__'] = { };
        result = typeof list.push === 'undefined';
        list['__proto__'] = backup;
        return result && typeof list.push === 'function';
      }
    },

    'OBJECT__COUNT__': function() {
      // true for Gecko
      if (Feature('OBJECT__PROTO__')) {
        var o = { 'x':0 };
        delete o['__count__'];
        return typeof o['__count__'] === 'number' && o['__count__'] === 1;
      }
    },

    'SELECTORS_API': function() {
      // true for IE8, WebKit (Safari 3, Chrome)
      var doc = Fuse._doc, docEl = Fuse._docEl;
      return isHostObject(doc, 'querySelector') &&
        isHostObject(doc,   'querySelectorAll') &&
        isHostObject(docEl, 'querySelector')    &&
        isHostObject(docEl, 'querySelectorAll');
    },

    'XPATH': function() {
      // true for all but IE
      return isHostObject(Fuse._doc, 'evaluate') &&
        isHostObject(global, 'XPathResult') &&
        typeof XPathResult.ORDERED_NODE_SNAPSHOT_TYPE === 'number';
    }
  });

  (function() {
    var div = Fuse._div, docEl = Fuse._docEl,
     // true for IE, Safari 3, Opera, Firefox 3+
     ELEMENT_CHILDREN_NODELIST = isHostObject(docEl, 'children'),
     // true for all but IE and Safari 2
     ELEMENT_CONTAINS = isHostObject(docEl, 'contains');

    div.innerHTML = '<div><\/div><div><div><\/div><\/div>';

    // ensure children collection only contains direct descendants
    if (ELEMENT_CHILDREN_NODELIST)
      ELEMENT_CHILDREN_NODELIST = div.children.length === div.childNodes.length;

    // ensure element.contains() returns the correct results;
    if (ELEMENT_CONTAINS)
      ELEMENT_CONTAINS = !div.firstChild.contains(div.childNodes[1].firstChild);

    div.innerHTML = '';

    Feature.set({
      'ELEMENT_CHILDREN_NODELIST': ELEMENT_CHILDREN_NODELIST,
      'ELEMENT_CONTAINS':          ELEMENT_CONTAINS
    });
  })();

  /*------------------------------ BROWSER BUGS ------------------------------*/

  Bug.set({
    'ARRAY_CONCAT_ARGUMENTS_BUGGY': function() {
      // true for Opera
      return (function() { return Array.prototype.concat &&
        [].concat(arguments).length === 2 })(1, 2);
    },

    'ATTRIBUTE_NODES_PERSIST_ON_CLONED_ELEMENTS': function() {
      // true for IE
      var node, clone, div = Fuse._div;
      (node = document.createAttribute('name')).value = 'x';
      div.setAttributeNode(node);
      (clone = div.cloneNode(false)).setAttribute('name', 'y');
      div.removeAttribute('name');
      return (node = clone.getAttributeNode('name')) && node.value === 'x';
    },

    'BODY_ACTING_AS_ROOT': function() {
      // true for IE Quirks, Opera 9.25
      var body = Fuse._body, div = Fuse._div, docEl = Fuse._docEl;
      if (docEl.clientWidth === 0) return true;

      var ds = div.style, bs = body.style, des = docEl.style,
       bsBackup = bs.cssText, desBackup = des.cssText;

      bs.margin  = des.margin = '0';
      bs.height  = des.height = 'auto';
      ds.cssText = 'display:block;height:8500px;';

      body.insertBefore(div, body.firstChild);
      var result = docEl.clientHeight >= 8500;

      // check scroll coords
      var scrollTop = docEl.scrollTop;
      Bug.set('BODY_SCROLL_COORDS_ON_DOCUMENT_ELEMENT',
        ++docEl.scrollTop && docEl.scrollTop === scrollTop + 1);
      docEl.scrollTop = scrollTop;

      // cleanup
      body.removeChild(div);
      bs.cssText  = bsBackup;
      des.cssText = desBackup;
      ds.cssText  = '';

      return result;
    },

    'BODY_OFFSETS_INHERIT_ITS_MARGINS': function() {
      // true for Safari
      var body = Fuse._body, bs = body.style, backup = bs.cssText;
      bs.cssText += ';position:absolute;top:0;margin:1px 0 0 0;';
      var result = body.offsetTop === 1;
      bs.cssText = backup;
      return result;
    },

    'ELEMENT_COMPUTED_STYLE_DEFAULTS_TO_ZERO': function() {
      if (Feature('ELEMENT_COMPUTED_STYLE')) {
        // true for Opera
        var result, des = Fuse._docEl.style, backup = des.cssText;
        des.position = 'static';
        des.top = des.left = '';

        var style = Fuse._doc.defaultView.getComputedStyle(Fuse._docEl, null);
        result = (style && style.top === '0px' && style.left === '0px');
        des.cssText = backup;
        return result;
      }
    },

    'ELEMENT_COMPUTED_STYLE_DIMENSIONS_EQUAL_BORDER_BOX': function() {
      if (Feature('ELEMENT_COMPUTED_STYLE')) {
        // true for Opera 9.2x
        var docEl = Fuse._docEl, des = docEl.style, backup = des.paddingBottom;
        des.paddingBottom = '1px';
        var style = Fuse._doc.defaultView.getComputedStyle(docEl, null),
         result = style && (parseInt(style.height) || 0) ===  docEl.offsetHeight;
        des.paddingBottom = backup;
        return result;
      }
    },

    'ELEMENT_COMPUTED_STYLE_HEIGHT_IS_ZERO_WHEN_HIDDEN': function() {
      if (Feature('ELEMENT_COMPUTED_STYLE')) {
        // true for Opera
        var des = Fuse._docEl.style, backup = des.display;
        des.display = 'none';

        // In Safari 2 getComputedStyle() will return null for elements with style display:none
        var style = Fuse._doc.defaultView.getComputedStyle(Fuse._docEl, null),
         result = style && style.height === '0px';

        des.display = backup;
        return result;
      }
    },

    'ELEMENT_COORD_OFFSETS_DONT_INHERIT_ANCESTOR_BORDER_WIDTH': function() {
      // true for all but IE8
      var body = Fuse._body, div = Fuse._div, bs = Fuse._body.style, backup = bs.cssText;
      body.appendChild(div);
      var value = div.offsetLeft;
      bs.cssText += ';border: 1px solid transparent;';
      var result = (value === div.offsetLeft);
      bs.cssText = backup;
      body.removeChild(div);
      return result;
    },

    'ELEMENT_OBJECT_AND_RELATIVES_FAILS_TO_INHERIT_FROM_PROTOTYPE': function() {
      // IE8 bugs:
      // Must reference Element as a property of global when assigning
      // properties to its prototype or it will create a seperate instance
      // for Element and global.Element.

      // HTMLObjectElement, HTMLAppletElement and HTMLEmbedElement objects
      // don't inherit from their prototypes. Creating an APPLET element
      // will alert a warning message if Java is not installed.
      if (Feature('ELEMENT_SPECIFIC_EXTENSIONS')) {
        var element = Fuse._doc.createElement('object'),
         prototype = global.Element.prototype;
        prototype[expando] = true;
        var result = !element[expando];
        delete prototype[expando];
        return result;
      }
    },

    'ELEMENT_PROPERTIES_ARE_ATTRIBUTES': function() {
      // true for IE
      var div = Fuse._div;
      div[expando] = 'x';
      var result = div.getAttribute(expando) === 'x';
      div.removeAttribute(expando);
      if (typeof div[expando] !== 'undefined')
        delete div[expando];
      return result;
    },

    'ELEMENT_SCRIPT_FAILS_TO_EVAL_TEXT_PROPERTY_ON_INSERT': function() {
      var docEl = Fuse._docEl, element = Fuse._doc.createElement('script');
      element.text = 'Fuse.' + expando +' = true;';
      docEl.insertBefore(element, docEl.firstChild);
      var result = !Fuse[expando];
      docEl.removeChild(element);
      delete Fuse[expando];
      return result;
    },

    'ELEMENT_TABLE_INNERHTML_INSERTS_TBODY': function() {
      // true for IE and Firefox 3
      var div = Fuse._div;
      div.innerHTML = '<table><tr><td><\/td><\/tr><\/table>';
      var result = getNodeName(div.firstChild.firstChild) === 'TBODY';
      div.innerHTML = '';
      return result;
    },

    'SELECTORS_API_CASE_INSENSITIVE_CLASSNAME': function() {
      // Safari 3 before 3.1.2 treat class names
      // case-insensitively in quirks mode.
      var div = Fuse._div, result = false;
      if (Feature('SELECTORS_API')) {
        div.id = expando;
        div.innerHTML = '<span class="X"><\/span>';
        result = div.querySelector('#'+ expando +' .x') !== null;
        div.id = div.innerHTML = '';
      }
      return result;
    },

    'STRING_LAST_INDEX_OF_BUGGY_WITH_NEGATIVE_POSITION': function() {
       // true for Chrome 1 and 2
       return 'x'.lastIndexOf('x', -1) !== 0;
    },

    'STRING_METHODS_WRONGLY_SETS_REGEXP_LAST_INDEX': function() {
      // true for IE
      var string = 'oxo', data = [], pattern = /x/;
      string.replace(pattern, '');
      data[0] = !!pattern.lastIndex;
      string.match(pattern);
      data[1] = !!pattern.lastIndex;
      return data[0] || data[1];
    },

    'STRING_REPLACE_COHERSE_FUNCTION_TO_STRING': function() {
      // true for Safari 2
      var func = function() { return '' };
      return 'a'.replace(/a/, func) === String(func);
    },

    'STRING_REPLACE_BUGGY_WITH_GLOBAL_FLAG_AND_EMPTY_PATTERN': function() {
      // true for Chrome 1
      var string = 'xy', replacement = function() { return 'o' };
      return !(string.replace(/()/g, 'o') === 'oxoyo' &&
        string.replace(new RegExp('', 'g'), replacement) === 'oxoyo' &&
        string.replace(/(y|)/g, replacement) === 'oxoo');
    },

    'TABLE_ELEMENTS_RETAIN_OFFSET_DIMENSIONS_WHEN_HIDDEN': function() {
      // true for IE7 and lower
      Fuse._div.innerHTML = '<table><tbody style="display:none"><tr style="width:1px"><td><\/td><\/tr><\/tbody><\/table>';
      Fuse._body.appendChild(Fuse._div);
      var result = !!Fuse._div.firstChild.firstChild.offsetWidth;
      Fuse._body.removeChild(Fuse._div);
      return result;
    }
  });

  Bug.set((function() {
    function createInnerHTMLTest(source, innerHTML, targetNode) {
      return function() {
        var div = Fuse._div, result = true;
        div.innerHTML = source;
        var element = div.firstChild;
        if (targetNode) element = element.getElementsByTagName(targetNode)[0];
        try {
          result = (element.innerHTML = innerHTML) &&
            element.innerHTML.toLowerCase() !== innerHTML;
        } catch(e) { }
        div.innerHTML = '';
        return result;
      };
    }

    return {
      'ELEMENT_COLGROUP_INNERHTML_BUGGY': createInnerHTMLTest(
        '<table><colgroup><\/colgroup><tbody><\/tbody><\/table>',
        '<col><col>', 'colgroup'
      ),

      'ELEMENT_OPTGROUP_INNERHTML_BUGGY': createInnerHTMLTest(
        '<select><optgroup><\/optgroup><\/select>',
        '<option>x<\/option>', 'optgroup'
      ),

      'ELEMENT_SELECT_INNERHTML_BUGGY': createInnerHTMLTest(
        '<select><option><\/option><\/select>', '<option>x<\/option>'
      ),

      'ELEMENT_TABLE_INNERHTML_BUGGY': createInnerHTMLTest(
        // left out tbody to test if it's auto inserted
        '<table><tr><td><\/td><\/tr><\/table>', '<tr><td><p>x<\/p><\/td><\/tr>'
      )
    };
  })());

  Bug.set((function() {
    function createCommentTest(conditional) {
      return function() {
        var div = Fuse._div;
        div.innerHTML = '<p>x<\/p><!--y-->';
        var result = conditional(div);
        div.innerHTML = '';
        return result;
      };
    }

    return {
      'COMMENT_NODES_IN_CHILDREN_NODELIST': createCommentTest(function(element) {
        // true for IE
        return Feature('ELEMENT_CHILDREN_NODELIST') && element.children.length === 2;
      }),

      'GET_ELEMENTS_BY_TAG_NAME_RETURNS_COMMENT_NODES': createCommentTest(function(element) {
        // true for IE
        return element.getElementsByTagName('*').length === 2;
      })
    };
  })());

  /*----------------------------- FUSE: FUSEBOX ------------------------------*/

  Fuse.Fusebox = (function() {
    function Fusebox() {
      if (this === Fuse) return new Fuse.Fusebox();

      function _cleanup(object) {
        if (Fuse.Fusebox.mode === 'IFRAME') {
          var cache = Fuse.Fusebox._createSandbox.cache,
           iframeEl = object || cache[cache.length -1];
          iframeEl.parentNode.removeChild(iframeEl);
        }
      }

      var _postProcess = function() { };

      function Fusebox(sandbox) {
        if (this === Fuse) return new Fuse.Fusebox();
        sandbox = sandbox || Fuse.Fusebox._createSandbox();

        // use call to avoid polluting the scope of the called
        // methods with another varaiable
        Fuse.Fusebox._createNatives.call(this, sandbox);
        _postProcess(this);

        Fuse.Fusebox._createStatics.call(this, sandbox);
        Fuse.Fusebox._wrapAccessorMethods.call(this, sandbox);

        // assign prototype properties, Plugin alias, and static updateGenerics method
        var n, i = 0;
        while (n = natives[i++]) {
          this[n].constructor = this;
          this[n].prototype.constructor = this[n];
          this[n].Plugin = this[n].prototype;

          if (n !== 'Object') this[n].updateGenerics = new Function('', [
            'function updateGenerics() {',
            'this.constructor.updateGenerics("' + n + '");',
            '}', 'return updateGenerics;'].join('\n'))();
        }

        _cleanup();
        this.updateGenerics();
      }

      var sandbox,
       natives = ['Array', 'Date', 'Function', 'Number', 'Object', 'RegExp', 'String'];

      // Safari 2.0.2 and lower will not populate the window.frames collection
      // until the dom has loaded. This will cause early attempts to create an
      // iframe sandbox to fail.
      try {
        sandbox = Fuse.Fusebox._createSandbox();
      } catch(e) {
        if (Fuse.Fusebox.mode !== 'IFRAME') throw e;
      }

      if (Fuse.Fusebox.mode === 'IFRAME')
        (function(Array) {
          // remove iframe from the document and nullify sandbox variable in
          // case it was corrupted by Opera
          sandbox && _cleanup(Fuse.Fusebox._createSandbox.cache.pop());
          sandbox = null;

          // Safari 3+ will fail to extend iframe sandboxed natives before the
          // dom has loaded. Catches Safari 2.0.2 and lower as well because sandbox
          // will be undefined
          try {
            Array.prototype.x = 1;
            if (!Array().x) throw new Error;
            delete Array.prototype.x;
          }
          catch(e) {
            // switch to "__proto__" powered sandboxes if available
            if (Feature('OBJECT__PROTO__')) {
              Fuse.Fusebox.mode = 'OBJECT__PROTO__';
              sandbox = Fuse.Fusebox._createSandbox();
              Array = sandbox.Array;
            } else throw e;
          }

          // Opera 9.5 - 10a throws a security error when calling Array#map or String#lastIndexOf
          // Opera 9.5 - 9.64 will error by simply calling the methods.
          // Opera 10 will error when first accessing the contentDocument of
          // another iframe and then accessing the methods.
          if (Array.prototype.map) {
            _cleanup(Fuse.Fusebox._createIframeObject().frameElement);
            try { new Array().map(Fuse.K) } catch (e) {
              _postProcess = function(fusebox) {
                fusebox.Array.prototype.map =
                fusebox.String.prototype.lastIndexOf = null;
              };
            }
          }
        })(sandbox && sandbox.Array);

      // Chrome, IE, and Opera's Array accessors return sugared arrays so we skip wrapping them
      sandbox = sandbox || Fuse.Fusebox._createSandbox();
      if (!(new sandbox.Array().slice(0) instanceof global.Array))
        Fuse.Fusebox._wrapAccessorMethods.skip.Array = { 'concat': 1, 'filter': 1, 'slice': 1 };

      // copy original properties to lazy loaded Fusebox and then replace
      (function(i) { for (i in this) Fusebox[i] = this[i]; }).call(Fuse.Fusebox);
      Fusebox.prototype = Fuse.Fusebox.prototype;
      Fuse.Fusebox = Fusebox;

      return new Fusebox(sandbox);
    }
    return Fusebox;
  })();

  /*--------------------------------------------------------------------------*/

  // create lists of spec'ed methods belonging to natives
  (function() {
    this.Array =
      'concat every filter join indexOf lastIndexOf map slice some'.split(' ');

    this.Date =
      ('getDate getDay getFullYear getHours getMilliseconds getMinutes getMonth ' +
       'getSeconds getTime getTimezoneOffset getUTCDate getUTCDay ' +
       'getUTCFullYear getUTCHours getUTCMilliseconds getUTCMinutes ' +
       'getUTCMonth getUTCSeconds getYear toJSON').split(' ');

    this.Number =
      'toExponential toFixed toJSON toPrecision'.split(' ');

    this.String =
      ('charAt charCodeAt concat indexOf lastIndexOf localeCompare match ' +
       'replace search slice split substr substring toJSON toLowerCase ' +
       'toUpperCase toLocaleLowerCase toLocaleUpperCase trim').split(' ');

    this.Function = ['bind'];
    this.RegExp   = ['exec'];

  }).call(Fuse.Fusebox.AccessorNames = { });

  (function(AccessorNames) {
    for (var n in AccessorNames)
      this[n] = slice.call(AccessorNames[n], 0);

    this.Array = this.Array.concat((
      'pop push forEach reduce reduceRight reverse shift sort splice unshift ' +
      'toLocaleString toString valueOf').split(' '));

    this.Date = this.Date.concat((
      'setDate setDay setFullYear setHours setMilliseconds setMinutes setMonth '  +
      'setSeconds setTime setTimezoneOffset setUTCDate setUTCDay setUTCFullYear ' +
      'setUTCHours setUTCMilliseconds setUTCMinutes setUTCMonth setUTCSeconds '   +
      'setYear toDateString toGMTString toISOString toLocaleDateString ' +
      'toLocaleString toLocaleTimeString toString toTimeString toUTCString ' +
      'valueOf').split(' '));

    this.Function = this.Function.concat('toString', 'valueOf');
    this.Number   = this.Number.concat('toLocaleString', 'toString', 'valueOf');
    this.RegExp   = this.RegExp.concat('test', 'toString', 'valueOf');
    this.String   = this.String.concat('toString', 'valueOf');

  }).call(Fuse.Fusebox.MethodNames = { }, Fuse.Fusebox.AccessorNames);

  /*--------------------------------------------------------------------------*/

  Fuse.Fusebox._createIframeObject = (function() {
    function _createIframeObject(content) {
      var iframeDoc, i = 0,
       frame = false,
       frames = global.frames,
       iframeEl = Fuse._doc.createElement('iframe'),
       parentNode = Fuse._body || Fuse._docEl,
       id = 'iframe_' + expando + counter++;

      iframeEl.id = id;
      iframeEl.style.cssText = 'position:absolute;left:-1000px;width:0;height:0;overflow:hidden';
      parentNode.insertBefore(iframeEl, parentNode.firstChild);

      if (frames[0]) {
        while (frame = frames[i++]) {
          if (frame.frameElement.id === id) {
            iframeDoc = frame.document; break;
          }
        }
        if (content) {
          iframeDoc.open();
          iframeDoc.write(content);
          iframeDoc.close();
        }
      }
      return frame;
    }

    var counter = 0;
    return _createIframeObject;
  })();

  /*--------------------------------------------------------------------------*/

  Fuse.Fusebox._createSandbox = (function() {
    function _createSandbox(mode) {
      var cache = Fuse.Fusebox._createSandbox.cache;
      switch(mode || Fuse.Fusebox.mode) {
        case 'ACTIVE_X_OBJECT':
          var htmlfile = new ActiveXObject('htmlfile');
          htmlfile.open();
          htmlfile.write('<script>document.domain="' + Fuse._doc.domain + '";document.global = this;<\/script>');
          htmlfile.close();
          cache.push(htmlfile);
          return htmlfile.global;

        case 'IFRAME':
          var frame = Fuse.Fusebox._createIframeObject('<script>parent.Fuse.' +
            expando + ' = this;<\/script>');
          if (frame) {
            var result = Fuse[expando];
            delete Fuse[expando];
            cache.push(frame.frameElement);
            return result;
          }
          break;

         case 'OBJECT__PROTO__': return global;
      }
      throw new Error('Fuse failed to create a sandbox.');
    }

    // IE requires the iframe/htmlfile remain in the cache or it will be
    // marked for garbage collection
    _createSandbox.cache = [];
    return _createSandbox;
  })();

  /*--------------------------------------------------------------------------*/

  // determine default mode for creating a sandbox
  Fuse.Fusebox.mode = (function()  {
    var isFileProtocol = global.location &&
      (global.location.href || '').indexOf('file:') === 0;

    // avoids the htmlfile activeX warning when served from the file protocol
    if (Feature('ACTIVE_X_OBJECT') && !isFileProtocol)
      return 'ACTIVE_X_OBJECT';

    if (Feature('OBJECT__PROTO__')) return 'OBJECT__PROTO__';

    // check "OBJECT__PROTO__" first because Firefox will permanently screw up
    // other iframes on the page if an iframe is inserted before the dom has loaded
    if (Fuse._doc && isHostObject(global, 'frames') &&
        isHostObject(Fuse._doc, 'createElement')) return 'IFRAME';
  })();

  /*--------------------------------------------------------------------------*/

  Fuse.Fusebox._createStatics = (function() {
    function _createStatics(sandbox) {

      /* Array statics */

      this.Array.create = (function(Array) {
        function create() { return Array.fromArray(arguments) }
        return create;
      })(this.Array);

      // ECMA-5 15.4.3.2
      this.Array.isArray = sandbox.Array.isArray || (function(toString) {
        function isArray(value) { return toString.call(value) === '[object Array]' }
        return isArray;
      })(sandbox.Object.prototype.toString);

      this.Array.fromArray = (function(fn) {
        var fromArray = function fromArray(array) {
          var result = new fn;
          result.push.apply(result, array);
          return result;
        };

        if (Fuse.Fusebox.mode === 'OBJECT__PROTO__') {
          var Array = this, slice = fn.prototype.slice;
          fromArray = function fromArray(array) {
            var result = slice.call(array, 0);
            result['__proto__'] = Array.prototype;
            return result;
          };
        }
        else if ((Fuse.Fusebox._wrapAccessorMethods.skip.Array || { }).slice) {
          var slice = fn.prototype.slice;
          fromArray = function fromArray(array) {
            return slice.call(array, 0);
          };
        }
        return fromArray;
      }).call(this.Array, sandbox.Array);

      /* Number statics */

      (function(fn, Number) {
        // ECMA-5 15.9.4.4
        var now = function now() { return Number(1 * new Date()) };
        if (fn.now) now = function now() { return Number(fn.now()) };
        this.now = now;

        // ECMA-5 15.9.4.2
        this.parse = function parse(dateString) {
          return Number(fn.parse(dateString));
        };

        // ECMA-5 15.9.4.3
        this.UTC = function UTC() {
          return Number(fn.UTC.apply(fn, arguments));
        };

        // prevent JScript bug with named function expressions
        var parse = null, UTC = null;
      }).call(this.Date, sandbox.Date, this.Number);

      (function(fn) {
        this.MAX_VALUE         = fn.MAX_VALUE;
        this.MIN_VALUE         = fn.MIN_VALUE;
        this.NaN               = fn.NaN;
        this.NEGATIVE_INFINITY = fn.NEGATIVE_INFINITY;
        this.POSITIVE_INFINITY = fn.POSITIVE_INFINITY;
      }).call(this.Number, sandbox.Number);

      /* String statics */

      this.String.fromCharCode = (function(fn, String) {
        function fromCharCode() { return String(fn.fromCharCode.apply(fn, arguments)) }
        return fromCharCode;
      })(sandbox.String, this.String);

      sandbox = null;
    }
    return _createStatics;
  })();

  /*--------------------------------------------------------------------------*/

  Fuse.Fusebox._createNatives = (function() {
    function _createNatives(sandbox) {
      this.Array = (function(fn, self) {
        function Array(length) {
          return arguments.length === 1 ? new fn(length) : self.Array.fromArray(arguments);
        }
        Array.prototype = fn.prototype;
        return Array;
      })(sandbox.Array, this);

      this.Function = (function(fn) {
        function Function(argN, body) {
          var result, args = slice.call(arguments, 0), originalBody = args.pop();
          body = originalBody;
          argN = args.join(',');

          // ensure we aren't in strict mode
          if (body && body.search(/^\s*(['"])use strict(( |,)?[^\1]+?)?\1/i) < 0)
            body = 'arguments.callee = arguments.callee.' + expando + '; ' + body;

          result = global.Function(argN, body);
          result[expando] = fn('global, result',
            'var sandbox = this; return function() { return result.apply(this == sandbox ? global : this, arguments) }'
          )(global, result);

          result[expando].toString = function toString() { return originalBody };

          var toString = null;
          return result[expando];
        }
        Function.prototype = fn.prototype;
        return Function;
      })(sandbox.Function);

      this.Number = (function(fn) {
        function Number(value) { return new fn(value) }
        Number.prototype = fn.prototype;
        return Number;
      })(sandbox.Number);

      this.RegExp = (function(fn) {
        var RegExp = function RegExp(pattern, flags) { return new fn(pattern, flags) };

        // versions of WebKit and IE have non-spec-conforming /\s/
        // so we emulate it (see: ECMA-5 15.10.2.12)
        // http://www.unicode.org/Public/UNIDATA/PropList.txt
        var specialCharMap = {
          's': {
            /* whitespace */
            '\x09': 1, '\x0B': 1, '\x0C': 1, '\x20': 1, '\xA0': 1,

            /* line terminators */
            '\x0A': 1, '\x0D': 1, '\u2028': 1, '\u2029': 1,

            /* unicode category "Zs" space separators */
            '\u1680': 1, '\u180e': 1, '\u2000': 1, '\u2001': 1, '\u2002': 1, '\u2003': 1,
            '\u2004': 1, '\u2005': 1, '\u2006': 1, '\u2007': 1, '\u2008': 1, '\u2009': 1,
            '\u200a': 1, '\u202f': 1, '\u205f': 1, '\u3000': 1
          }
        };

        var s = (function() {
          var result = ['\\s'];
          for (character in specialCharMap.s)
            if (character.replace(/\s/, '').length)
              result.push('\\u' +('0000' + character.charCodeAt(0).toString(16)).slice(-4));
          return result.length > 1 ? '(' + result.join('|') + ')' : result[0];
        })();

        if (s !== '\\s') {
          var toString = sandbox.Object.prototype.toString;
          RegExp = function RegExp(pattern, flags) {
            if (toString.call(pattern) === '[object RegExp]') {
              if (pattern.global || pattern.ignoreCase || pattern.multiline)
                throw new TypeError;
              if (pattern.source.indexOf('\\s') > -1)
                pattern = pattern.source.replace(/\\s/g, s);
            }
            else pattern = String(pattern).replace(/\\s/g, s);

            return new fn(pattern, flags);
          };
        } else s = null;

        RegExp.prototype = fn.prototype;
        RegExp.specialCharMap = specialCharMap;

        specialCharMap = null;
        return RegExp;
      })(sandbox.RegExp);

      this.String = (function(fn) {
        function String(value) { return new fn(arguments.length ? value : '') }
        String.prototype = fn.prototype;
        return String;
      })(sandbox.String);

      this.Date = (function(fn, self) {
        function Date(year, month, date, hours, minutes, seconds, ms) {
          if (this instanceof self.Date) {
            return arguments.length === 1
              ? new fn(year)
              : new fn(year, month, date || 1, hours || 0, minutes || 0, seconds || 0, ms || 0);
          }
          return self.String(new fn);
        }
        Date.prototype = fn.prototype;
        return Date;
      })(sandbox.Date, this);

      this.Object = (function(fn, self) {
        function Object(value) {
          if (value != null) {
            switch (toString.call(value)) {
              case '[object Boolean]': return new Boolean(value);
              case '[object Number]':  return self.Number(value);
              case '[object String]':  return self.String(value);
              case '[object Array]':
                if (value.constructor !== self.Array)
                  return self.Array.fromArray(value);
            }
            return value;
          }
          return new fn;
        }
        Object.prototype = fn.prototype;
        var toString = fn.prototype.toString;
        return Object;
      })(sandbox.Object, this);

      sandbox = null;
    }

    // clean scope
    var $ = null, Bug = null, Feature = null, concatList = null, document = null,
     getDocument = null, getNodeName = null, getWindow = null, isHostObject = null,
     prependList = null, userAgent = null, window = null;

    return _createNatives;
  })();

  /*--------------------------------------------------------------------------*/

  // wrap original _createNatives to make __proto__ support optional
  Fuse.Fusebox._createNatives = (function(fn) {
    function _createNatives(sandbox) {
      if (Fuse.Fusebox.mode !== 'OBJECT__PROTO__')
        return fn.call(this, sandbox);

      // call original _createNatives method
      fn.call(this, sandbox);

      // wrap native constructors to add __proto__ support
      var n, i = 0, natives = ['Array', 'Function', 'Number', 'String'];
      while (n = natives[i++])
        this[n] = new Function('global,fn', [
          'function ' + n + '() {',
          'var result = arguments.length ?',
          'fn.apply(this, arguments) : fn.call(this);',
          'result["__proto__"] = ' + n + '.prototype;',
          'return result; }',
          n + '.prototype["__proto__"] = fn.prototype;',
          'return ' + n].join('\n'))(global, this[n]);

      i = n = natives = null;

      this.Date = (function(fn, self) {
        function Date(year, month, date, hours, minutes, seconds, ms) {
          var result = fn.apply(this, arguments);
          if (this instanceof self.Date)
            result['__proto__'] = Date.prototype;
          return result;
        }
        Date.prototype = fn.prototype;
        return Date;
      })(this.Date, this);

      this.Object = (function(fn) {
        function Object(value) {
          var result = fn(value);
          if (value == null)
            result['__proto__'] = Object.prototype;
          return result;
        }
        Object.prototype['__proto__'] = fn.prototype;
        return Object;
      })(this.Object);

      this.RegExp = (function(fn) {
        function RegExp(pattern, flags) {
          var result = fn(pattern, flags);
          result['__proto__'] = RegExp.prototype;
          return result;
        }
        RegExp.prototype['__proto__'] = fn.prototype;
        RegExp.specialCharMap = fn.specialCharMap;
        return RegExp;
      })(this.RegExp);
    }

    return _createNatives;
  })(Fuse.Fusebox._createNatives);

  /*--------------------------------------------------------------------------*/

  Fuse.Fusebox._wrapAccessorMethods = (function() {
    function _wrapAccessorMethods() {
      var code, i, name, names, type, natives = Fuse.Fusebox.AccessorNames,
       skip = Fuse.Fusebox._wrapAccessorMethods.skip;

      // Opera and Chrome still need a convienence wrapper for filter
      // so that it supports an undefined callback
      if (this.Array.prototype.filter && skip.Array && skip.Array.filter) (function() {
        this.filter = new Function('fn', [
          'function filter(callback, thisArg) {',
          'return fn.call(this, callback || function(value) { return value != null }, thisArg);',
          '} return filter'].join('\n'))(this.filter);
      }).call(this.Array.prototype);

      // iterate over native constructors
      for (n in natives) {
        i = 0; names = natives[n];

        // iterate over each method on the native object
        while (name = names[i++]) {
          if ((skip[n] && skip[n][name]) || !this[n].prototype[name]) continue;

          // determine the data type of the returned value
          type = n;
          if (/ndexOf|Date|Day|Hour|Minutes|Month|econds|Time|Year|^(charCodeAt|search|push|unshift)$/.test(name))
            type = 'Number';
          else if (/String|^(join|charAt)$/.test(name))
            type = 'String';
          else if (/^(exec|match|split)$/.test(name))
            type = 'Array';

          // compile wrapper code
          code = [
            'var ' + type + ' = this.' + type + ';',
            'var fn = this.' + n + '.prototype.' + name + ';',
            'function ' + name + '() {',
            'var args = arguments;'];

          // ensure a sugared array is returned when needed
          if (type === 'Array') {
            if (/^(RegExp|String)$/.test(n)) {
              code.unshift('var String = this.String;');

              if (/^(exec|match)$/.test(name)) code.push(
                'var results = args.length ? fn.apply(this, args) : fn.call(this);',
                'if (!results) return null;',
                'results = Array.fromArray(results);'
              );
              else code.push(
                'var results = Array.fromArray(args.length ?',
                'fn.apply(this, args) : fn.call(this));'
              );

              code.push(
                'var length = results.length, i = 0;',
                'while (length--) results[length] = String(results[length]);',
                'return results;'
              );
            }
            else {
              // ensure a default callback argument is provided
              code.push('return ' + (/^(every|some)$/.test(name) ? ' (' : 'Array.fromArray('));
              if (name === 'filter')
                code.push('fn.call(this, args[0] || function(value) { return value != null }, args[1]));');
              else if (/^(every|map|some)$/.test(name))
                code.push('fn.call(this, args[0] || Fuse.K, args[1]));');
            }
          }
          else code.push('return new ' + type + '(');

          if (code.length === 5)
            code.push('args.length ? fn.apply(this, args) : fn.call(this));');
          code.push('} return ' + name);

          this[n].prototype[name] = new Function('', code.join('\n')).call(this);
        }
      }
    }

    _wrapAccessorMethods.skip = { };
    return _wrapAccessorMethods;
  })();

  /*--------------------------------------------------------------------------*/

  Fuse.Fusebox.prototype.updateGenerics = (function() {
    function _createGeneric(methodName) {
      return new Function('', [
        'function ' + methodName + '(thisArg) {',
        'return this.prototype.' + methodName + '.apply(thisArg,',
        'Array.prototype.slice.call(arguments, 1));',
        '}', 'return ' + methodName].join('\n'))();
    }

    function _forIn(object, callback) {
      for (var key in object) callback(object[key], key, object);
    }

    function updateGenerics() {
      var c, j, n, names, object, i = 0,
       cache = Fuse.Fusebox.prototype.updateGenerics.cache,
       forIn = Fuse.Object && Fuse.Object._each || _forIn,
       natives = arguments.length ? arguments : _natives;

      // iterate over native constructors
      while (n = natives[i++]) {
        j = 0; c = cache[n] || (cache[n] = { });
        object = this[n];
        names  = Fuse.Fusebox.MethodNames[n] || [];

        // convert common methods on the native object prototype to
        // generics on the native constructor
        while (name = names[j++])
          if (!c[name] && object.prototype[name])
            object[name] = c[name] = _createGeneric(name);

        // convert additional methods: _forIn is safe the first time because
        // we have avoided problem properties
        forIn(object.prototype, function(value, key) {
          if (!c[key] && !/^(constructor|prototype|Plugin)$/.test(key))
            object[key] = c[key] = _createGeneric(key);
        });
      }
    }

    var _natives = ['Array', 'Date', 'Function', 'Number', 'Object', 'RegExp', 'String'];
    updateGenerics.cache = { };
    return updateGenerics;
  })();

  /*--------------------------------------------------------------------------*/

  // assign Fusebox natives to Fuse object
  (function() {
    var n, i = 0;
    // add generics updater
    Fuse.updateGenerics = this.updateGenerics;
    // assign natives to Fuse
    while (n = arguments[i++]) (Fuse[n] = this[n]).constructor = Fuse;
    // alias
    Fuse.List = Fuse.Array;
  }).call(Fuse.Fusebox(), 'Array', 'Date', 'Function', 'Number', 'Object', 'RegExp', 'String');

  /*------------------------------ LANG: OBJECT ------------------------------*/

  Fuse.Object._each = (function() {
    // use switch statement to avoid creating a temp variable
    var _each;
    switch (function() {
      var key, count = 0, klass = function() { this.toString = 1 };
      klass.prototype.toString = 1;
      for (key in new klass()) count++;
      return count;
    }()) {
      case 0: // IE
        var dontEnumProperties = ['constructor', 'hasOwnkey', 'isPrototypeOf',
          'propertyIsEnumerable', 'prototype', 'toLocaleString', 'toString', 'valueOf'];
        return _each = function _each(object, callback) {
          if (object) {
            var key, i = 0;
            for (key in object) callback(object[key], key, object);
            while(key = dontEnumProperties[i++])
              if (Fuse.Object.hasKey(object, key))
                callback(object[key], key, object);
          }
          return object;
        };

      case 2:
        // Tobie Langel: Safari 2 broken for-in loop
        // http://tobielangel.com/2007/1/29/for-in-loop-broken-in-safari/
        return _each = function _each(object, callback) {
          var key, keys = { };
          for (key in object)
            if (!Fuse.Object.hasKey(keys, key) && (keys[key] = 1))
              callback(object[key], key, object);
          return object;
        };

      default: // Others
        return _each = function _each(object, callback) {
          for (var key in object) callback(object[key], key, object);
          return object;
        };
    }
  })();

  Fuse.Object.hasKey = (function() {
    var hasKey, hasOwnProperty = Object.prototype.hasOwnProperty;
    if (typeof hasOwnProperty !== 'function') {
      if (Feature('OBJECT__PROTO__')) {
        // Safari 2
        hasKey = function hasKey(object, property) {
          if (object == null) throw new TypeError;
          // convert primatives to objects so IN operator will work
          object = Object(object);

          var result, proto = object['__proto__'];
          object['__proto__'] = null;
          result = property in object;
          object['__proto__'] = proto;
          return result;
        };
      } else {
        // Other
        hasKey = function hasKey(object, property) {
          if (object == null) throw new TypeError;
          object = Object(object);
          return object[property] !== object.constructor.prototype[property];
        };
      }
    }
    else hasKey = function(object, property) {
      // ECMA-5 15.2.4.5
      if (object == null) throw new TypeError;
      return hasOwnProperty.call(object, property);
    };

    // Opera (bug occurs with the window object and not the global)
    if (typeof window !== 'undefined' && window.Object && !hasKey(window, 'Object')) {
      var _hasKey = hasKey;
      hasKey = function hasKey(object, property) {
        if (object == null) throw new TypeError;
        if(object == global) {
          return property in object &&
            object[property] !== Object.prototype[property];
        }
        return _hasKey(object, property);
      };
    }
    return hasKey;
  })();

  /*--------------------------------------------------------------------------*/

  (function() {
    this._extend = function _extend(destination, source) {
      for (var key in source)
         destination[key] = source[key];
      return destination;
    };

    this.clone = function clone(object) {
      if (object && typeof object.clone === 'function')
        return object.clone();
      return Fuse.Object.extend(Fuse.Object(), object);
    };

    this.each = function each(object, callback, thisArg) {
      try {
        Fuse.Object._each(object, function(value, key, object) {
          callback.call(thisArg, value, key, object);
        });
      } catch (e) {
        if (e !== Fuse.$break) throw e;
      }
      return object;
    };

    this.extend = function extend(destination, source) {
      if (source) {
        Fuse.Object._each(source, function(value, key) {
          destination[key] = value;
        });
      }
      return destination;
    };

    // ECMA-5 15.2.3.14
    if (!this.keys) this.keys = function keys(object) {
      if (Fuse.Object.isPrimitive(object))
        throw new TypeError;
      var results = Fuse.List();
      Fuse.Object._each(object, function(value, key) {
        if (Fuse.Object.hasKey(object, key))
          results.push(key);
      });
      return results;
    };

    this.values = function values(object) {
      if (Fuse.Object.isPrimitive(object))
        throw new TypeError;
      var results = Fuse.List();
      Fuse.Object._each(object, function(value, key) {
        if (Fuse.Object.hasKey(object, key))
          results.push(value);
      });
      return results;
    };

    // prevent JScript bug with named function expressions
    var _extend = null,
     clone =      null,
     each =       null,
     extend =     null,
     keys =       null,
     values =     null;
  }).call(Fuse.Object);

  /*--------------------------------------------------------------------------*/

  (function() {
    // used to access the an object's internal [[Class]] property
    var toString = Object.prototype.toString;

    this.inspect = function inspect(value) {
      if (value != null) {
        var object = Fuse.Object(value);
        if (typeof object.inspect === 'function')
          return object.inspect();

        // Attempt to avoid inspecting DOM nodes.
        // IE treats nodes like objects:
        // IE7 and below are missing the node's constructor property 
        // IE8 node constructors are typeof "object"
        try {
          var string = toString.call(object), constructor = object.constructor;
          if (string === '[object Object]' && constructor &&
              typeof constructor !== 'object') {
            var results = [];
            Fuse.Object._each(object, function(value, key) {
              if (Fuse.Object.hasKey(object, key))
                results.push(Fuse.String(key).inspect() + ': ' +
                  Fuse.Object.inspect(object[key]));
            });
            return Fuse.String('{' + results.join(', ') + '}');
          }
        } catch (e) { }
      }

      // try coercing to string
      try {
        return Fuse.String(value);
      } catch (e) {
        if (e.constructor === global.RangeError) return Fuse.String('...');
        throw e;
      }
    };

    this.isArray = function isArray(value) {
      return Fuse.List.isArray(value);
    };

    this.isElement = function isElement(value) {
      return !!value && value.nodeType === 1;
    };

    this.isEmpty = (function() {
      var isEmpty = function isEmpty(object) {
        for (var key in object)
          if (Fuse.Object.hasKey(object, key))
            return false;
        return true;
      };

      if (Feature('OBJECT__COUNT__')) {
        // __count__ is buggy on arrays so we check for push because it's fast.
        var _isEmpty = isEmpty;
        isEmpty = function isEmpty(object) {
          return !object || object.push ? _isEmpty(object) : !object['__count__'];
        };
      }
      return isEmpty;
    })();

    this.isFunction = function isFunction(value) {
      return toString.call(value) === '[object Function]';
    };

    this.isHash = function isHash(value) {
      return value instanceof Fuse.Hash;
    };

    this.isNumber = function isNumber(value) {
      return toString.call(value) === '[object Number]' && isFinite(value);
    };

    this.isPrimitive = function isPrimitive(value) {
      // ECMA-5 4.3.2
      var type = typeof value;
      return value == null || type === 'boolean' || type === 'number' || type === 'string';
    };

    this.isRegExp = function isRegExp(value) {
      return toString.call(value) === '[object RegExp]';
    };

    this.isSameOrigin = function isSameOrigin(url) {
      // https://developer.mozilla.org/En/Same_origin_policy_for_JavaScript
      // http://www.iana.org/assignments/port-numbers
      var domain = Fuse._doc.domain, protocol = global.location.protocol,
       defaultPort = protocol === 'ftp:' ? 21 : protocol === 'https:' ? 443 : 80,
       parts = String(url).match(/([^:]+:)\/\/(?:[^:]+(?:\:[^@]+)?@)?([^\/:$]+)(?:\:(\d+))?/) || [];

      return !parts[0] || (parts[1] === protocol &&
        Fuse.String.Plugin.endsWith.call(parts[2], domain) &&
          (parts[3] || defaultPort) === (global.location.port || defaultPort));
    };

    this.isString = function isString(value) {
      return toString.call(value) === '[object String]';
    };

    this.isUndefined = function isUndefined(value) {
      return typeof value === 'undefined';
    };

    // prevent JScript bug with named function expressions
    var inspect =   null,
     isArray =      null,
     isElement =    null,
     isFunction =   null,
     isHash =       null,
     isNumber =     null,
     isPrimitive =  null,
     isRegExp =     null,
     isSameOrigin = null,
     isString =     null,
     isUndefined =  null;
  }).call(Fuse.Object);

  /*--------------------------------------------------------------------------*/

  (function() {
    this.toHTML = function toHTML(object) {
      return object && typeof object.toHTML === 'function'
        ? Fuse.String(object.toHTML())
        : Fuse.String.interpret(object);
    };

    this.toQueryString = (function() {
      function toQueryPair(key, value) {
        return Fuse.String(typeof value === 'undefined' ? key :
          key + '=' + encodeURIComponent(Fuse.String.interpret(value)));
      }

      function toQueryString(object) {
        var results = Fuse.List();
        Fuse.Object._each(object, function(value, key) {
          key = encodeURIComponent(key);
          if (value && typeof value === 'object') {
            if (Fuse.List.isArray(value)) {
              var i = results.length, j = 0, length = i + value.length;
              while (i < length) results[i++] = toQueryPair(key, value[j++]);
            }
          } else results.push(toQueryPair(key, value));
        });
        return results.join('&');
      }
      return toQueryString;
    })();

    // prevent JScript bug with named function expressions
    var toHTML = null;
  }).call(Fuse.Object);

  /*------------------------------ LANG: CLASS -------------------------------*/
  /* Based on work by Alex Arnell, Joey Hurst, John Resig, and Prototype core */

  Fuse.Class = (function() {
    var _subclass = function() { };

    function _createNamedClass(name) {
      return new Function('', [
        'function ' + name + '() {',
        'return this.initialize && this.initialize.apply(this, arguments);',
        '}', 'return ' + name].join('\n'))();
    }

    function Class() {
      var klass, parent, props, i = 0, properties = slice.call(arguments, 0);
      if (typeof properties[0] === 'function')
        parent = properties.shift();

      // search properties for a custom `constructor` method
      while (props = properties[i++]) {
        if (Fuse.Object.hasKey(props, 'constructor')) {
          if (typeof props.constructor === 'function')
            klass = props.constructor;
          else if (Fuse.Object.isString(props.constructor))
            klass = _createNamedClass(props.constructor);
          delete props.constructor;
        }
      }

      klass = klass || _createNamedClass('UnnamedClass');
      Fuse.Object.extend(klass, Fuse.Class.Methods);

      if (parent) {
        // note: Safari 2, inheritance won't work with subclass = new Function;
        _subclass.prototype = parent.prototype;
        klass.prototype = new _subclass;
        parent.subclasses.push(klass);
      }

      klass.superclass = parent; i = 0;
      while (props = properties[i++]) klass.addMethods(props);

      klass.superclass = parent;
      klass.subclasses = Fuse.List();
      klass.Plugin = klass.prototype;
      klass.Plugin.constructor = klass;
      return klass;
    }

    return Class;
  })();

  Fuse.Class.Methods = (function() {
    var matchSuper = Feature('FUNCTION_TO_STRING_RETURNS_SOURCE')
      ? /\bthis\._super\b/
      : { 'test': function() { return true } };

    function addMethods(source) {
      var prototype = this.prototype,
       ancestor = this.superclass && this.superclass.prototype;
      Fuse.Object._each(source, function(method, key) {
        // avoid typeof === 'function' because Safari 3.1+ mistakes
        // regexp instances as typeof 'function'
        if (ancestor && Fuse.Object.isFunction(ancestor[key]) && 
            Fuse.Object.isFunction(method) && matchSuper.test(method)) {
          var __method = method;
          method = function() {
            // backup this._super and assign the ancestors method to it
            var result, backup = this._super;
            this._super = ancestor[key];

            // execute and capture the result
            result = arguments.length
              ? __method.apply(this, arguments)
              : __method.call(this);

            // restore backup and return result
            this._super = backup;
            return result;
          };

          method.valueOf  = Fuse.Function.bind(__method.valueOf, __method);
          method.toString = Fuse.Function.bind(__method.toString, __method);
        }
        prototype[key] = method;
      });
      return this;
    }

    return {
      'addMethods': addMethods
    };
  })();

  // replace placeholder objects with inheritable classes
  global.Fuse = Fuse.Class({ 'constructor': Fuse });
  Fuse.Plugin = Fuse.prototype = Fuse.Object.prototype;

  Fuse.Browser = Fuse.Object._extend(Fuse.Class(Fuse,
    { 'constructor': 'Browser' }), Fuse.Browser);

  Fuse.Browser.Agent = Fuse.Object._extend(Fuse.Class(Fuse.Browser,
    { 'constructor': 'Agent' }), Fuse.Browser.Agent);

  Fuse.Browser.Bug = Fuse.Class(Fuse.Browser, { 'constructor': Bug });

  Fuse.Browser.Feature = Fuse.Class(Fuse.Browser, { 'constructor': Feature });

  Fuse.Class = Fuse.Class(Fuse, { 'constructor': Fuse.Class });

  Fuse.Fusebox = Fuse.Class(Fuse,
    { 'constructor': Fuse.Fusebox }, Fuse.Fusebox.prototype);

  /*----------------------------- LANG: FUNCTIONS ----------------------------*/

  (function() {
    this.argumentNames = function argumentNames(fn) {
      var names = Fuse.String(fn).match(/^[\s\(]*function[^(]*\(([^)]*)\)/)[1]
       .replace(/\/\/.*?[\r\n]|\/\*(?:.|[\r\n])*?\*\//g, '')
        .replace(/\s+/g, '').split(',');
      return names.length === 1 && !names[0].length ? Fuse.List() : names;
    };

    // ECMA-5 15.3.4.5
    this.bind = (function() {
      var bind = function bind(fn, thisArg) {
        // allows lazy loading the target method
        var f, context, name;
        if (Fuse.Object.isArray(fn)) {
          name = fn[0]; context = fn[1];
        } else f = fn;

        // simple bind
        if (arguments.length < 3 ) {
          if (typeof thisArg === 'undefined') return fn;
          return function() {
            var fn = f || context[name];
            return arguments.length
              ? fn.apply(thisArg, arguments)
              : fn.call(thisArg);
          };
        }
        // bind with curry
        var args = slice.call(arguments, 2); reset = args.length;
        return function() {
          args.length = reset; // reset arg length
          var fn = f || context[name];
          return fn.apply(thisArg, arguments.length ?
            concatList(args, arguments) : args);
        };
      };
      // native support
      if (typeof this.prototype.bind === 'function') {
        bind = function bind(fn, thisArg) {
          return Fuse.Function.prototype.bind.call(f || thisArg[name], thisArg);
        };
      }
      return bind;
    }).call(this);

    this.bindAsEventListener = function bindAsEventListener(fn, thisArg) {
      // allows lazy loading the target method
      var f, context, name;
      if (Fuse.Object.isArray(fn)) {
        name = fn[0]; context = fn[1];
      } else f = fn;

      // simple bind
      if (arguments.length < 3 ) {
        return function(event) {
          return (f || context[name]).call(thisArg, event || getWindow(this).event);
        };
      }
      // bind with curry
      var args = slice.call(arguments, 2);
      return function(event) {
        return (f || context[name]).apply(thisArg,
          prependList(args, event || getWindow(this).event));
      };
    };

    this.curry = function curry(fn) {
      // allows lazy loading the target method
      var f, context, name;
      if (Fuse.Object.isArray(fn)) {
        name = fn[0]; context = fn[1]; fn = context[name];
      } else f = fn;

      if (arguments.length === 1) return fn;
      var args = slice.call(arguments, 1), reset = args.length;
      return function() {
        args.length = reset; // reset arg length
        var fn = f || context[name];
        return arguments.length
          ? fn.apply(this, concatList(args, arguments))
          : fn.apply(this, args);
      }
    };

    this.delay = function delay(fn, timeout) {
      timeout *= 1000;
      var f, context, name, args = slice.call(arguments, 2);

      // allows lazy loading the target method
      if (Fuse.Object.isArray(fn)) {
        name = fn[0]; context = fn[1];
      } else f = fn;

      return global.setTimeout(function() {
        var fn = f || context[name];
        return fn.apply(fn, args);
      }, timeout);
    };

    this.defer = function defer(fn) {
      return Fuse.Function.delay.apply(global,
        concatList([fn, 0.01], slice.call(arguments, 1)));
    };

    this.methodize = function methodize(fn) {
      // allows lazy loading the target method
      var f, context, name;
      if (Fuse.Object.isArray(fn)) {
        name = fn[0]; context = fn[1]; fn = context[name];
      } else f = fn;

      return fn._methodized || (fn._methodized = function() {
        var fn = f || context[name];
        return arguments.length
          ? fn.apply(global, prependList(arguments, this))
          : fn.call(global, this);
      });
    };

    this.wrap = function wrap(fn, wrapper) {
      // allows lazy loading the target method
      var f, context, name, bind = Fuse.Function.bind;
      if (Fuse.Object.isArray(fn)) {
        name = fn[0]; context = fn[1];
      } else f = fn;

      return function() {
        var fn = f || context[name];
        return arguments.length
          ? wrapper.apply(this, prependList(arguments, bind(fn, this)))
          : wrapper.call(this, bind(fn, this));
      }
    };

    // prevent JScript bug with named function expressions
    var argumentNames =    null,
     bindAsEventListener = null,
     curry =               null,
     delay =               null,
     defer =               null,
     methodize =           null,
     wrap =                null;
  }).call(Fuse.Function);

  /*--------------------------------------------------------------------------*/

  (function() {
    var name, i = 0, cache = Fuse.updateGenerics.cache,
     names = ['argumentNames', 'bind', 'bindAdEventListener', 'curry', 'delay', 'defer', 'methodize', 'wrap'];
    cache.Function = { };
    while (name = names[i++]) {
      cache.Function[name] = this[name];
      if (this.prototype[name] !== 'function') {
        this.prototype[name] = new Function('global', [
          'var object = Fuse.Function, slice = Array.prototype.slice;',
          'function ' + name + '() {',
          'return arguments.length',
          '? object.' + name + '.apply(object, [this].concat(slice.call(arguments, 0)))',
          ': object.' + name + '.call(object, this);',
          '}', 'return ' + name].join('\n'))(global);
      }
    }
  }).call(Fuse.Function);

  /*---------------------------- LANG: ENUMERABLE ----------------------------*/

  Fuse.$break = (function() {
    function $break() { }
    return $break;
  })();

  Fuse.addNS('Enumerable');

  (function() {
    this.contains = function contains(object, strict) {
      var result = 0;
      if (strict)
        this.each(function(value) { if (value === object && result++) throw Fuse.$break; });
      else
        this.each(function(value) { if (value == object && result++) throw Fuse.$break; });
      return !!result;
    };

    this.each = function each(callback, thisArg) {
      try {
        this._each(function(value, index, iterable) {
          callback.call(thisArg, value, index, iterable);
        });
      } catch (e) {
        if (e !== Fuse.$break) throw e;
      }
      return this;
    };

    this.eachSlice = function eachSlice(size, callback, thisArg) {
      var index = -size, slices = Fuse.List(), list = this.toList();
      if (size < 1) return list;
      while ((index += size) < list.length)
        slices[slices.length] = list.slice(index, index + size);
      return slices.map(callback, thisArg);
    };

    this.every = function every(callback, thisArg) {
      callback = callback || Fuse.K;
      var result = true;
      this.each(function(value, index, iterable) {
        if (!callback.call(thisArg, value, index, iterable)) {
          result = false; throw Fuse.$break;
        }
      });
      return result;
    };

    this.filter = function filter(callback, thisArg) {
      var results = Fuse.List();
      callback = callback || function(value) { return value != null };
      this._each(function(value, index, iterable) {
        if (callback.call(thisArg, value, index, iterable))
          results.push(value);
      });
      return results;
    };

    this.first = function first(callback, thisArg) {
      if (callback == null) {
        var result;
        this.each(function(value) { result = value; throw Fuse.$break; });
        return result;
      }
      return this.toList().first(callback, thisArg);
    };

    this.grep = function grep(pattern, callback, thisArg) {
      if (!pattern || pattern == '' || Fuse.Object.isRegExp(pattern) &&
         !pattern.source) return this.toList();

      callback = callback || Fuse.K;
      var results = Fuse.List();
      if (Fuse.Object.isString(pattern))
        pattern = new RegExp(Fuse.RegExp.escape(pattern));

      this._each(function(value, index, iterable) {
        if (pattern.test(value))
          results.push(callback.call(thisArg, value, index, iterable));
      });
      return results;
    };

    this.inGroupsOf = function inGroupsOf(size, filler) {
      filler = (typeof filler === 'undefined') ? null : filler;
      return this.eachSlice(size, function(slice) {
        while (slice.length < size) slice.push(filler);
        return slice;
      });
    };

    this.inject = function inject(accumulator, callback, thisArg) {
      this._each(function(value, index, iterable) {
        accumulator = callback.call(thisArg, accumulator, value, index, iterable);
      });
      return accumulator;
    };

    this.inspect = function inspect() {
      return '#<Enumerable:' + this.toList().inspect() + '>';
    };

    this.invoke = function invoke(method) {
      var args = slice.call(arguments, 1);
      return this.map(function(value) {
        return Function.prototype.apply.call(value[method], value, args);
      });
    };

    this.last = function last(callback, thisArg) {
      return this.toList().last(callback, thisArg);
    };

    this.map = function map(callback, thisArg) {
      if (!callback) return this.toList();
      var results = Fuse.List();
      if (thisArg) {
        this._each(function(value, index, iterable) {
          results.push(callback.call(thisArg, value, index, iterable));
        });
      } else {
        this._each(function(value, index, iterable) {
          results.push(callback(value, index, iterable));
        });
      }
      return results;
    };

    this.max = function max(callback, thisArg) {
      callback = callback || Fuse.K;
      var comparable, max, result;
      this._each(function(value, index, iterable) {
        comparable = callback.call(thisArg, value, index, iterable);
        if (max == null || comparable > max) {
          max = comparable; result = value;
        }
      });
      return result;
    };

    this.min = function min(callback, thisArg) {
      callback = callback || Fuse.K;
      var comparable, min, result;
      this._each(function(value, index, iterable) {
        comparable = callback.call(thisArg, value, index, iterable);
        if (min == null || comparable < min) {
          min = comparable; result = value;
        }
      });
      return result;
    };

    this.partition = function partition(callback, thisArg) {
      callback = callback || Fuse.K;
      var trues = Fuse.List(), falses = Fuse.List();
      this._each(function(value, index, iterable) {
        (callback.call(thisArg, value, index, iterable) ?
          trues : falses).push(value);
      });
      return Fuse.List(trues, falses);
    };

    this.pluck = function pluck(property) {
      return this.map(function(value) {
        return value[property];
      });
    };

    this.size = function size() {
      return Fuse.Number(this.toList().length);
    };

    this.some = function some(callback, thisArg) {
      callback = callback || Fuse.K;
      var result = false;
      this.each(function(value, index, iterable) {
        if (callback.call(thisArg, value, index, iterable)) {
          result = true; throw Fuse.$break;
        }
      });
      return result;
    };

    this.sortBy = function sortBy(callback, thisArg) {
      return this.map(function(value, index, iterable) {
        return {
          'value': value,
          'criteria': callback.call(thisArg, value, index, iterable)
        };
      }).sort(function(left, right) {
        var a = left.criteria, b = right.criteria;
        return a < b ? -1 : a > b ? 1 : 0;
      }).pluck('value');
    };

    this.toArray = function toArray() {
      var results = Fuse.List();
      this._each(function(value, index) { results[index] = value });
      return results;
    };

    this.zip = function zip() {
      var callback = Fuse.K, args = slice.call(arguments, 0);
      if (typeof Fuse.List.Plugin.last.call(args) === 'function')
        callback = args.pop();

      var collection = prependList(Fuse.List.Plugin.map.call(args, Fuse.Util.$A),
        this.toArray(), Fuse.List());

      return this.map(function(value, index, iterable) {
        return callback(collection.pluck(index), index, iterable);
      });
    };

    // alias
    this.toList = this.toArray;

    // prevent JScript bug with named function expressions
    var contains = null,
     each =        null,
     eachSlice =   null,
     every =       null,
     filter =      null,
     first =       null,
     grep =        null,
     inject =      null,
     inGroupsOf =  null,
     inspect =     null,
     invoke =      null,
     last =        null,
     map =         null,
     max =         null,
     min =         null,
     partition =   null,
     pluck =       null,
     size =        null,
     some =        null,
     sortBy =      null,
     toArray =     null,
     zip =         null;
  }).call(Fuse.Enumerable.Plugin);

  /*------------------------------ LANG: ARRAY -------------------------------*/

  (function() {
    this.from = function from(iterable) {
      if (!iterable || iterable == '') return Fuse.Array();

      // Safari 2.x will crash when accessing a non-existent property of a
      // node list, not in the document, that contains a text node unless we
      // use the `in` operator
      var object = Fuse.Object(iterable);
      if ('toArray' in object) return object.toArray();
      if ('item' in iterable)  return Fuse.Array.fromNodeList(iterable);

      var length = iterable.length >>> 0, results = Fuse.Array(length);
      while (length--) if (length in object) results[length] = iterable[length];
      return results;
    };

    this.fromNodeList = function fromNodeList(nodeList) {
      var i = 0, results = Fuse.Array();
      while (results[i] = nodeList[i++]) { }
      return results.length-- && results;
    };

    // prevent JScript bug with named function expressions
    var from = null, fromNodeList = null;
  }).call(Fuse.Array);

  /*--------------------------------------------------------------------------*/

  Fuse.addNS('Util');

  Fuse.Util.$A = Fuse.Array.from;

  Fuse.Util.$w = (function() {
    function $w(string) {
      if (!Fuse.Object.isString(string)) return Fuse.Array();
      string = proto.trim.call(string);
      return string != '' ? string.split(/\s+/) : Fuse.Array();
    }
    var proto = Fuse.String.prototype;
    return $w;
  })();

  /*--------------------------------------------------------------------------*/

  (function() {
    var proto = this;

    this._each = function _each(callback) {
      this.forEach(callback);
      return this;
    };

    this.clear = function clear() {
      if (this == null) throw new TypeError;
      var object = Object(this);

      if (!Fuse.Object.isArray(object)) {
        var length = object.length >>> 0;
        while (length--) if (length in object) delete object[length];
      }
      object.length = 0;
      return object;
    };

    this.clone = function clone() {
      var object = Object(this);
      if (this == null) throw new TypeError;

      if (Fuse.Object.isArray(object)) {
        return object.constructor !== Fuse.Array
          ? Fuse.Array.fromArray(object)
          : object.slice(0);
      }
      return Fuse.Array.from(object);
    };

    this.compact = function compact(falsy) {
      if (this == null) throw new TypeError;
      var i = 0, results = Fuse.Array(), object = Object(this),
       length = object.length >>> 0;

      if (falsy) {
        for ( ; i < length; i++)
          if (object[i] && object[i] != '') results.push(object[i]);
      } else {
        for ( ; i < length; i++)
          if (object[i] != null) results.push(object[i]);
      }
      return results;
    };

    this.each = function each(callback, thisArg) {
      try {
        proto.forEach.call(this, callback, thisArg);
      } catch (e) {
        if (e !== Fuse.$break) throw e;
      }
      return this;
    };

    this.first = function first(callback, thisArg) {
      if (this == null) throw new TypeError;
      var i = 0, object = Object(this),
       length = object.length >>> 0;

      if (callback == null) {
        for ( ; i < length; i++)
          if (i in object) return object[i];
      }
      else if (typeof callback === 'function') {
        for ( ; i < length; i++)
          if (callback.call(thisArg, object[i], i))
            return object[i];
      }
      else {
        var count = 1 * callback; // fast coerce to number
        if (isNaN(count)) return Fuse.Array();
        count = count < 1 ? 1 : count > length ? length : count;
        return proto.slice.call(object, 0, count);
      }
    };

    this.flatten = function flatten() {
      if (this == null) throw new TypeError;
      var i = 0, isArray = Fuse.Array.isArray, results = Fuse.Array(),
       object = Object(this), length = object.length >>> 0;

      for ( ; i < length; i++) {
        if (isArray(object[i]))
          concatList(results, proto.flatten.call(object[i]));
        else results.push(object[i]);
      }
      return results;
    };

    this.insert = function insert(index, value) {
      if (this == null) throw new TypeError;
      var object = Object(this),
       length = object.length >>> 0;

      if (length < index) object.length = index;
      if (index < 0) index = length;
      if (arguments.length > 2)
        proto.splice.apply(object, concatList([index, 0], slice.call(arguments, 1)));
      else proto.splice.call(object, index, 0, value);
      return object;
    };

    this.inspect = function inspect() {
      if (this == null) throw new TypeError;
      var i = 0, results = result = [], object = Object(this),
       length = object.length >>> 0;

      while (length--) results[length] = Fuse.Object.inspect(object[length]);
      return '[' + results.join(', ') + ']';
    };

    this.intersect = function intersect(array) {
      if (this == null) throw new TypeError;
      var item, i = 0, indexOf = proto.indexOf, results = Fuse.Array(),
       object = Object(this), length = object.length >>> 0;

      for ( ; i < length; i++) {
        item = array[i];
        if (i in object && indexOf.call(object, item) != -1 &&
            results.indexOf(item) == -1)
          results.push(item);
      }
      return results;
    };

    this.last = function last(callback, thisArg) {
      if (this == null) throw new TypeError;
      var object = Object(this), length = object.length >>> 0;

      if (callback == null)
        return object[length && length - 1];
      if (typeof callback === 'function') {
        while (length--)
          if (callback.call(thisArg, object[length], length, object))
            return object[length];
      }
      else {
        var results = Fuse.Array(), count = 1 * callback;
        if (isNaN(count)) return results;

        count = count < 1 ? 1 : count > length ? length : count;
        return proto.slice.call(object, length - count);
      }
    };

    this.size = function size() {
      if (this == null) throw new TypeError;
      return Fuse.Number(Object(this).length >>> 0);
    };

    this.unique = function unique() {
      var i = 0, results = Fuse.Array(), object = Object(this),
       length = object.length >>> 0;

      for ( ; i < length; i++)
        if (i in object && results.indexOf(object[i]) == -1)
          results.push(object[i]);
      return results;
    };

    this.without = function without() {
      if (this == null) throw new TypeError;
      var i = 0, args = slice.call(arguments, 0), indexOf = proto.indexOf,
       results = Fuse.Array(), object = Object(this),
       length = object.length >>> 0;

      for ( ; i < length; i++)
        if (i in object && indexOf.call(args, object[i]) == -1)
          results.push(object[i]);
      return results;
    };

    /* Create optimized Enumerable equivalents */

    this.contains = (function() {
      var contains = function contains(value, strict) {
        if (this == null) throw new TypeError;
        var object = Object(this), length = object.length >>> 0;

        if (strict) {
          while (length--)
            if (length in object && object[length] === value) return true;
        } else {
          while (length--)
            if (length in object && object[length] == value) return true;
        }
        return false;
      };

      if (typeof proto.indexOf === 'function') {
        var _contains = contains;
        contains = function contains(value, strict) {
          // attempt a fast strict search first
          if (this == null) throw new TypeError;
          var object = Object(this),
           result = proto.indexOf.call(object, value) != -1;

          if (strict || result) return result;
          return _contains.call(object, value);
        };
      }
      return contains;
    })();

    this.inject = (function() {
      var inject = function inject(accumulator, callback, thisArg) {
        if (this == null) throw new TypeError;
        var i = 0, object = Object(this), length = object.length >>> 0;

        if (thisArg) {
          for ( ; i < length; i++) if (i in object)
            accumulator = callback.call(thisArg, accumulator, object[i], i, object);
        }
        else {
          for ( ; i < length; i++) if (i in object)
            accumulator = callback(accumulator, object[i], i, object);
        }
        return accumulator;
      };

      // use Array#reduce if available
      if (typeof proto.reduce === 'function') {
        var _inject = inject;
        inject = function inject(accumulator, callback, thisArg) {
          return thisArg
            ? _inject.call(this, accumulator, callback, thisArg)
            : proto.reduce.call(this, callback, accumulator);
        };
      }
      return inject;
    })();

    this.invoke = function invoke(method) {
      if (this == null) throw new TypeError;
      var args, i = 0, results = Fuse.Array(), object = Object(this),
       length = object.length >>> 0;

      if (arguments.length < 2) {
        while (length--) if (length in object)
          results[length] = Function.prototype.call.call(object[length][method], object[length]);
      } else {
        args = slice.call(arguments, 1);
        while (length--) if (length in object)
          results[length] = Function.prototype.apply.call(object[length][method], object[length], args);
      }
      return results;
    };

    this.grep = function grep(pattern, callback, thisArg) {
      if (this == null) throw new TypeError;
      if (!pattern || pattern == '' || Fuse.Object.isRegExp(pattern) &&
         !pattern.source) return proto.toArray.call(this);

      callback = callback || Fuse.K;
      var item, i = 0, results = Fuse.Array(), object = Object(this),
       length = object.length >>> 0;

      if (Fuse.Object.isString(pattern))
        pattern = new RegExp(Fuse.RegExp.escape(pattern));

      for ( ; i < length; i++)
        if (i in object && pattern.test(object[i]))
          results.push(callback.call(thisArg, object[i], i, object));
      return results;
    };

    this.max = function max(callback, thisArg) {
      if (this == null) throw new TypeError;

      var result, undef;
      if (!callback && (callback = Fuse.K) && Fuse.Object.isArray(this)) {
        // John Resig's fast Array max|min:
        // http://ejohn.org/blog/fast-javascript-maxmin
        result = Math.max.apply(Math, this);
        if (!isNaN(result)) return result;
        result = undef;
      }

      var comparable, max, value, i = 0,
       object = Object(this), length = object.length >>> 0;

      for ( ; i < length; i++) {
        if (i in object) {
          comparable = callback.call(thisArg, value = object[i], i, object);
          if (max == null || comparable > max) {
            max = comparable; result = value;
          }
        }
      }
      return result;
    };

    this.min = function min(callback, thisArg) {
      if (this == null) throw new TypeError;

      var result, undef;
      if (!callback && (callback = Fuse.K) && Fuse.Object.isArray(this)) {
        result = Math.min.apply(Math, this);
        if (!isNaN(result)) return result;
        result = undef;
      }

      var comparable, min, value, i = 0,
       object = Object(this), length = object.length >>> 0;

      for ( ; i < length; i++) {
        if (i in object) {
          comparable = callback.call(thisArg, value = object[i], i, object);
          if (min == null || comparable < min) {
            min = comparable; result = value;
          }
        }
      }
      return result;
    };

    this.partition = function partition(callback, thisArg) {
      if (this == null) throw new TypeError;

      callback = callback || Fuse.K;
      var i = 0, trues = Fuse.Array(), falses = Fuse.Array(),
       object = Object(this), length = object.length >>> 0;

      for ( ; i < length; i++) if (i in object)
        (callback.call(thisArg, object[i], i, object) ?
          trues : falses).push(object[i]);
      return Fuse.Array(trues, falses);
    };

    this.pluck = function pluck(property) {
      if (this == null) throw new TypeError;
      var i = 0, results = Fuse.Array(), object = Object(this),
       length = object.length >>> 0;

      for ( ; i < length; i++) if (i in object)
        results[i] = object[i][property];
      return results;
    };

    this.sortBy = function sortBy(callback, thisArg) {
      if (this == null) throw new TypeError;

      callback = callback || Fuse.K;
      var value, results = Fuse.Array(), object = Object(this),
       length = object.length >>> 0;

      while (length--) {
        value = object[length];
        results[length] = { 'value': value, 'criteria': callback.call(thisArg, value, length, object) };
      }

      return results.sort(function(left, right) {
        var a = left.criteria, b = right.criteria;
        return a < b ? -1 : a > b ? 1 : 0;
      }).pluck('value');
    };

    this.zip = function zip() {
      if (this == null) throw new TypeError;

      var i = 0, results = Fuse.Array(), callback = Fuse.K,
       args = slice.call(arguments, 0), object = Object(this),
       length = object.length >>> 0;

      if (typeof proto.last.call(args) === 'function')
        callback = args.pop();

      var collection = prependList(proto.map.call(args, Fuse.Util.$A), object, Fuse.Array());
      for ( ; i < length; i++)
        results.push(callback(collection.pluck(i), i, object));
      return results;
    };

    /* Use native browser JS 1.6 implementations if available */

    // Opera's implementation of Array.prototype.concat treats a functions arguments
    // object as an array so we overwrite concat to fix it.
    // ECMA-5 15.4.4.4
    if (!this.concat || Bug('ARRAY_CONCAT_ARGUMENTS_BUGGY'))
      this.concat = function concat() {
        if (this == null) throw new TypeError;

        var i = 0, args = arguments, length = args.length, object = Object(this),
         results = Fuse.Array.isArray(object) ? Fuse.Array.fromArray(object) : Fuse.Array(object);

        for ( ; i < length; i++) {
          if (Fuse.Array.isArray(args[i])) {
            for (var j = 0, sub = args[i], subLen = sub.length; j < subLen; j++)
              results.push(sub[j]);
          } else results.push(args[i]);
        }
        return results;
      };

    // ECMA-5 15.4.4.16
    if (!this.every) this.every = function every(callback, thisArg) {
      callback = callback || Fuse.K;
      if (this == null || !Fuse.Object.isFunction(callback)) throw new TypeError;

      var i = 0, object = Object(this), length = object.length >>> 0;
      for ( ; i < length; i++)
        if (i in object && !callback.call(thisArg, object[i], i, object))
          return false;
      return true;
    };

    // ECMA-5 15.4.4.20
    if (!this.filter) this.filter = function filter(callback, thisArg) {
      callback = callback || function(value) { return value != null };
      if (this == null || !Fuse.Object.isFunction(callback)) throw new TypeError;

      var i = 0, results = Fuse.Array(), object = Object(this),
       length = object.length >>> 0;

      for ( ; i < length; i++)
        if (i in object && callback.call(thisArg, object[i], i, object))
          results.push(object[i]);
      return results;
    };

    // ECMA-5 15.4.4.18
    if (!this.forEach) this.forEach = function forEach(callback, thisArg) {
      if (this == null || !Fuse.Object.isFunction(callback)) throw new TypeError;

      var i = 0, object = Object(this), length = object.length >>> 0;
      if (thisArg) {
        for ( ; i < length; i++)
          i in object && callback.call(thisArg, object[i], i, object);
      } else {
        for ( ; i < length; i++)
          i in object && callback(object[i], i, object);
      }
    };

    // ECMA-5 15.4.4.14
    if (!this.indexOf) this.indexOf = function indexOf(item, fromIndex) {
      if (this == null) throw new TypeError;

      fromIndex = fromIndex >> 0;
      var object = Object(this), length = object.length >>> 0;
      if (fromIndex < 0) fromIndex = length + fromIndex;

      // ECMA-5 draft oversight, should use [[HasProperty]] instead of [[Get]]
      for ( ; fromIndex < length; fromIndex++)
        if (fromIndex in object && object[fromIndex] === item)
          return Fuse.Number(fromIndex);
      return Fuse.Number(-1);
    };

    // ECMA-5 15.4.4.15
    if (!this.lastIndexOf) this.lastIndexOf = function lastIndexOf(item, fromIndex) {
      if (this == null) throw new TypeError;

      var object = Object(this), length = object.length >>> 0;
      fromIndex = arguments.length === 2 ? fromIndex >> 0 : length;

      if (!length) return Fuse.Number(-1);
      if (fromIndex > length) fromIndex = length - 1;
      if (fromIndex < 0) fromIndex = length + fromIndex;

      // ECMA-5 draft oversight, should use [[HasProperty]] instead of [[Get]]
      for ( ; fromIndex > -1; fromIndex--)
        if (fromIndex in object && object[fromIndex] === item) break;
      return Fuse.Number(fromIndex);
    };

    // ECMA-5 15.4.4.19
    if (!this.map) this.map = function map(callback, thisArg) {
      if (!callback) return proto.clone.call(this);
      if (this == null || !Fuse.Object.isFunction(callback)) throw new TypeError;

      var i = 0, results = Fuse.Array(), object = Object(this),
       length = object.length >>> 0;

      if (thisArg) {
        for ( ; i < length; i++)
          if (i in object) results[i] = callback.call(thisArg, object[i], i, object);
      } else {
        for ( ; i < length; i++)
          if (i in object) results[i] = callback(object[i], i, object);
      }
      return results;
    };

    // ECMA-5 15.4.4.17
    if (!this.some) this.some = function some(callback, thisArg) {
      callback = callback || Fuse.K;
      if (this == null || !Fuse.Object.isFunction(callback)) throw new TypeError;

      var i = 0, object = Object(this), length = object.length >>> 0;
      for ( ; i < length; i++)
        if (i in object && callback.call(thisArg, object[i], i, object))
          return true;
      return false;
    };

    // aliases
    this.toArray =
    this.toList  = this.clone;

    // assign any missing Enumerable methods
    if (Fuse.Enumerable) {
      Fuse.Object.each(Fuse.Enumerable.Plugin, function(value, key) {
        if (typeof proto[key] !== 'function') proto[key] = value;
      });
    }

    // prevent JScript bug with named function expressions
    var _each =    null,
     clear =       null,
     clone =       null,
     compact =     null,
     concat =      null,
     each =        null,
     every =       null,
     filter =      null,
     first =       null,
     flatten =     null,
     forEach =     null,
     grep =        null,
     indexOf =     null,
     insert =      null,
     inspect =     null,
     intersect =   null,
     invoke =      null,
     last =        null,
     lastIndexOf = null,
     map =         null,
     max =         null,
     min =         null,
     partition =   null,
     pluck =       null,
     size =        null,
     some =        null,
     sortBy =      null,
     unique =      null,
     without =     null,
     zip =         null;
  }).call(Fuse.Array.Plugin);

  /*------------------------------ LANG: NUMBER ------------------------------*/

  (function() {
    this.abs = (function() {
      function abs() { return Fuse.Number(_abs(this)) }
      var _abs = Math.abs;
      return abs;
    })();

    this.ceil = (function() {
      function ceil() { return Fuse.Number(_ceil(this)) }
      var _ceil = Math.ceil;
      return ceil;
    })();

    this.floor = (function() {
      function floor() { return Fuse.Number(_floor(this)) }
      var _floor = Math.floor;
      return floor;
    })();

    this.round = (function() {
      function round() { return Fuse.Number(_round(this)) }
      var _round = Math.round;
      return round;
    })();

  }).call(Fuse.Number.Plugin);

  /*--------------------------------------------------------------------------*/

  (function() {
    var proto = this;

    var _toInteger = (function() {
      var _abs = Math.abs, _floor = Math.floor;
      return function(object) {
        var number = 1 * object; // fast coerce to number
        if (number == 0 || !isFinite(number)) return number || 0;

        // avoid issues with large numbers against bitwise operators
        return number < 2147483648
          ? number | 0
          : (number < 0 ? -1 : 1) * _floor(_abs(number));
      };
    })();

    this.succ = function succ() {
      return Fuse.Number(_toInteger(this) + 1);
    };

    this.times = function times(callback, thisArg) {
      var i = 0, length = _toInteger(this);
      if (arguments.length === 1) {
        while (i < length) callback(i, i++);
      } else {
        while (i < length) callback.call(thisArg, i, i++);
      }
      return this;
    };

    this.toColorPart = function toColorPart() {
      return proto.toPaddedString.call(_toInteger(this), 2, 16);
    };

    this.toPaddedString = (function() {
      function toPaddedString(length, radix) {
        var string = _toInteger(this).toString(radix || 10);
        if (length <= string.length) return Fuse.String(string);
        if (length > pad.length) pad = new Array(length + 1).join('0');
        return Fuse.String((pad + string).slice(-length));
      }

      var pad = '000000';
      return toPaddedString;
    })();

    // prevent JScript bug with named function expressions
    var succ = null, times = null, toColorPart = null;
  }).call(Fuse.Number.Plugin);

  /*------------------------------ LANG: REGEXP ------------------------------*/

  Fuse.RegExp.escape = (function() {
    function escape(string) {
      return Fuse.String(string).replace(/([.*+?^=!:${}()|[\]\/\\])/g, '\\$1');
    }
    return escape;
  })();

  (function() {
    this.clone = function clone(options) {
      options = Fuse.Object._extend({
        'global':     this.global,
        'ignoreCase': this.ignoreCase,
        'multiline':  this.multiline
      }, options);

      return Fuse.RegExp(this.source,
        (options.global     ? 'g' : '') +
        (options.ignoreCase ? 'i' : '') +
        (options.multiline  ? 'm' : ''));
    };

    // alias
    this.match = this.test;

    // prevent JScript bug with named function expressions
    var clone = null;
  }).call(Fuse.RegExp.Plugin);

  /*------------------------------ LANG: STRING ------------------------------*/

  Fuse.Object._extend(Fuse.String, (function() {
    function interpret(value) {
      return Fuse.String(value == null ? '' : value);
    }

    return {
      'specialChar': {
        '\b': '\\b',
        '\f': '\\f',
        '\n': '\\n',
        '\r': '\\r',
        '\t': '\\t',
        '\\': '\\\\'
      },
      'interpret': interpret
    };
  })());

  /*--------------------------------------------------------------------------*/

  (function() {
    var __replace = this.replace;

    function replace(pattern, replacement) {
      if (typeof replacement === 'function') {
        var _replacement = replacement;
        replacement = function() {
          // ensure `null` and `undefined` are returned
          var result = _replacement.apply(global, arguments);
          return result || String(result);
        };
      }
      var result = __replace.call(this, pattern, replacement);
      if (Fuse.Object.isRegExp(pattern)) pattern.lastIndex = 0;
      return result;
    }

    // For IE
    if (Bug('STRING_METHODS_WRONGLY_SETS_REGEXP_LAST_INDEX'))
      this.replace = replace;
  }).call(Fuse.String.Plugin);

  // ECMA-5 15.5.4.11
  (function() {
    var __replace = this.replace,
     exec = RegExp.prototype.exec;

    function replace(pattern, replacement) {
      if (typeof replacement !== 'function')
        return __replace.call(this, pattern, replacement);

      if (this == null) throw new TypeError;

      if (!Fuse.Object.isRegExp(pattern))
        pattern = new RegExp(Fuse.RegExp.escape(pattern));

      // set pattern.lastIndex to 0 before we perform string operations
      var match, index = 0, nonGlobal = !pattern.global,
       lastIndex = pattern.lastIndex = 0,
       result = '', source = String(this),
       srcLength = source.length;

      while (match = exec.call(pattern, source)) {
        index = match.index;
        result += source.slice(lastIndex, index);

        // set lastIndex before replacement call to avoid potential
        // pattern.lastIndex tampering
        lastIndex = pattern.lastIndex;
        result += replacement.apply(global, concatList(match, [index, source]));

        if (nonGlobal) {
          pattern.lastIndex = lastIndex = index + match[0].length;
          break;
        }
        // handle empty pattern matches like /()/g
        if (lastIndex === index) {
          if (lastIndex === srcLength) break;
          result += source.charAt(lastIndex++);
        }
        pattern.lastIndex = lastIndex;
      }

      // append the remaining source to the result
      if (lastIndex < srcLength)
        result += source.slice(lastIndex, srcLength);

      return Fuse.String(result);
    }

    // primarily for Safari 2.0.2 and lower, based on work by Dean Edwards
    // http://code.google.com/p/base2/source/browse/trunk/lib/src/base2-legacy.js?r=239#174
    if (Bug('STRING_REPLACE_COHERSE_FUNCTION_TO_STRING') ||
        Bug('STRING_REPLACE_BUGGY_WITH_GLOBAL_FLAG_AND_EMPTY_PATTERN'))
      this.replace = replace;
  }).call(Fuse.String.Plugin);

  /*--------------------------------------------------------------------------*/

  // ECMA-5 15.5.4.8
  (function() {
    function lastIndexOf(searchString) {
      if (this == null) throw new TypeError;
      searchString = String(searchString);

      var string = String(this),
       pos = 1 * arguments[1], // fast coerce to number
       len = string.length,
       searchLen = searchString.length;

      if (searchLen > len) return Fuse.Number(-1);
      if (pos < 0) pos = 0;
      else if (isNaN(pos) || pos > len - searchLen) pos = len - searchLen;
      if (!searchLen) return Fuse.Number(pos);

      pos++;
      while (pos--)
        if (string.slice(pos, pos + searchLen) === searchString)
          return Fuse.Number(pos);
      return Fuse.Number(-1);
    }

    if (!this.lastIndexOf)
      this.lastIndexOf = lastIndexOf;
  }).call(Fuse.String.Plugin);

  (function() {
    var __lastIndexOf = this.lastIndexOf;

    function lastIndexOf(searchString) {
      var pos = 1 * arguments[1];
      return __lastIndexOf.call(this, searchString, pos < 0 ? 0 : pos);
    }

    // For Chome 1 and 2
    if (Bug('STRING_LAST_INDEX_OF_BUGGY_WITH_NEGATIVE_POSITION'))
      this.lastIndexOf = lastIndexOf;
  }).call(Fuse.String.Plugin);

  /*--------------------------------------------------------------------------*/

  // ECMA-5 15.5.4.10
  (function() {
    var __match = this.match;

    function match(pattern) {
      var result = __match.call(this, pattern);
      if (Fuse.Object.isRegExp(pattern)) pattern.lastIndex = 0;
      return result;
    }

    // For IE
    if (Bug('STRING_METHODS_WRONGLY_SETS_REGEXP_LAST_INDEX'))
      this.match = match;
  }).call(Fuse.String.Plugin);

  /*--------------------------------------------------------------------------*/

  (function() {
    function _prepareReplacement(replacement) {
      if (typeof replacement === 'function')
        return function() { return replacement(slice.call(arguments, 0, -2)) };
      var template = new Fuse.Template(replacement);
      return function() { return template.evaluate(slice.call(arguments, 0, -2)) };
    }

    this.gsub = function gsub(pattern, replacement) {
      if (this == null) throw new TypeError;

      if (!Fuse.Object.isRegExp(pattern))
        pattern = Fuse.RegExp(Fuse.RegExp.escape(pattern), 'g');
      if (!pattern.global)
        pattern = Fuse.RegExp.clone(pattern, { 'global': true });
      return this.replace(pattern, _prepareReplacement(replacement));
    };

    this.sub = function sub(pattern, replacement, count) {
      if (this == null) throw new TypeError;

      count = (typeof count === 'undefined') ? 1 : count;
      if (count === 1) {
        if (!Fuse.Object.isRegExp(pattern))
          pattern = Fuse.RegExp(Fuse.RegExp.escape(pattern));
        if (pattern.global)
          pattern = Fuse.RegExp.clone(pattern, { 'global': false });
        return this.replace(pattern, _prepareReplacement(replacement));
      }

      if (typeof replacement !== 'function') {
        var template = new Fuse.Template(replacement);
        replacement = function(match) { return template.evaluate(match) };
      }
      return this.gsub(pattern, function(match) {
        if (--count < 0) return match[0];
        return replacement(match);
      });
    };

    // prevent JScript bug with named function expressions
    var gsub = null, sub = null;
  }).call(Fuse.String.Plugin);

  /*--------------------------------------------------------------------------*/

  (function() {
    this.interpolate = function interpolate(object, pattern) {
      if (this == null) throw new TypeError;
      return new Fuse.Template(this, pattern).evaluate(object);
    };

    this.succ = function succ() {
      if (this == null) throw new TypeError;
      return Fuse.String(this.slice(0, this.length - 1) +
        String.fromCharCode(this.charCodeAt(this.length - 1) + 1));
    };

    this.times = function times(count) {
      if (this == null) throw new TypeError;
      return Fuse.String(count < 1 ? '' : new Array(count + 1).join(this));
    };

    this.toArray = function toArray() {
      if (this == null) throw new TypeError;
      return Fuse.String(this).split('');
    };

    this.toQueryParams = function toQueryParams(separator) {
      if (this == null) throw new TypeError;
      var match = String(this).split('?'), object = Fuse.Object();
      if (match.length > 1 && !match[1]) return object;

      (match = (match = match[1] || match[0]).split('#')) &&
        (match = match[0].split(' ')[0]);
      if (!match) return object;

      var pair, key, value, undef, index, i = 0,
       pairs = match.split(separator || '&'), length = pairs.length;

      for ( ; i < length; i++) {
        value = undef;
        index = (pair = pairs[i]).indexOf('=');
        if (!pair || index == 0) continue;

        if (index != -1) {
          key = decodeURIComponent(pair.slice(0, index));
          value = pair.slice(index + 1);
          if (value) value = decodeURIComponent(value);
        } else key = pair;

        if (Fuse.Object.hasKey(object, key)) {
          if (!Fuse.List.isArray(object[key])) object[key] = [object[key]];
          object[key].push(value);
        }
        else object[key] = value;
      }
      return object;
    };

    // aliases
    this.toList = this.toArray;
    this.parseQuery = this.toQueryParams;

    // prevent JScript bug with named function expressions
    var interpolate = null,
     succ =           null,
     times =          null,
     toArray =        null,
     toQueryParams =  null;
  }).call(Fuse.String.Plugin);

  /*--------------------------------------------------------------------------*/

  (function() {
    this.blank = function blank() {
      if (this == null) throw new TypeError;
      return /^\s*$/.test(this);
    };

    this.contains = function contains(pattern) {
      if (this == null) throw new TypeError;
      return String(this).indexOf(pattern) > -1;
    };

    this.empty = function empty() {
      if (this == null) throw new TypeError;
      return !String(this).length;
    };

    this.endsWith = function endsWith(pattern) {
      if (this == null) throw new TypeError;
      var string = String(this), d = string.length - pattern.length;
      return d >= 0 && string.lastIndexOf(pattern) == d;
    };

    this.inspect = function inspect(useDoubleQuotes) {
      if (this == null) throw new TypeError;

      var escapedString = Fuse.String(this).replace(/[\x00-\x1f\\]/g, function(match) {
        var character = Fuse.String.specialChar[match];
        return character ?
          character :
          '\\u00' + Fuse.Number(match.charCodeAt(0)).toPaddedString(2, 16);
      });
      return Fuse.String(useDoubleQuotes
        ? '"' + escapedString.replace(/"/g, '\\"')  + '"'
        : "'" + escapedString.replace(/'/g, '\\\'') + "'");
    };

    this.scan = function scan(pattern, callback) {
      if (this == null) throw new TypeError;
      Fuse.String(this).gsub(pattern, callback); // gsub for backcompat
      return Fuse.String(this);
    };

    this.startsWith = function startsWith(pattern) {
      if (this == null) throw new TypeError;
      return String(this).indexOf(pattern) == 0;
    };

    // prevent JScript bug with named function expressions
    var blank =     null,
      contains =    null,
      empty =       null,
      endsWith =    null,
      inspect =     null,
      scan =        null,
      startsWith =  null;
  }).call(Fuse.String.Plugin);

  /*--------------------------------------------------------------------------*/

  (function() {
    this.camelize = (function() {
      function _toUpperCase(match, character) {
        return character ? character.toUpperCase() : '';
      }

      function camelize() {
        if (this == null) throw new TypeError;
        var string = String(this), expandoKey = expando + string;
        return cache[expandoKey] ||
          (cache[expandoKey] = _replace.call(string, matchHyphenated, _toUpperCase));
      }

      var _replace = Fuse.String.Plugin.replace,
       cache = { },
       matchHyphenated = /\-+(.)?/g;

      return camelize;
    })();

    this.capitalize = (function() {
      function capitalize() {
        if (this == null) throw new TypeError;
        var string = String(this), expandoKey = expando + string;
        return cache[expandoKey] ||
          (cache[expandoKey] = Fuse.String(string.charAt(0).toUpperCase() +
            string.slice(1).toLowerCase()));
      }
      var cache = { };
      return capitalize;
    })();

    this.hyphenate = function hyphenate() {
      if (this == null) throw new TypeError;
      return Fuse.String(String(this).replace(/_/g,'-'));
    };

    this.truncate = function truncate(length, truncation) {
      if (this == null) throw new TypeError;
      var endIndex, string = String(this);

      length = 1 * length;
      if (isNaN(length)) length = 30;

      if (length < string.length) {
        truncation = truncation == null ? '...' : String(truncation);
        endIndex = length - truncation.length;
        string = endIndex > 0 ? string.slice(0, endIndex) + truncation : truncation;
      }
      return Fuse.String(string);
    };

    this.underscore = function underscore() {
      if (this == null) throw new TypeError;
      return Fuse.String(String(this).replace(/::/g, '/')
        .replace(/([A-Z]+)([A-Z][a-z])/g, '$1_$2')
        .replace(/([a-z\d])([A-Z])/g, '$1_$2')
        .replace(/-/g,'_').toLowerCase());
    };

    // prevent JScript bug with named function expressions
    var hyphenate = null, truncate = null, underscore = null;
  }).call(Fuse.String.Plugin);

  /*--------------------------------------------------------------------------*/

  (function() {
    var matchScripts   = RegExp(Fuse.ScriptFragment, 'gi'),
     matchHTMLComments = Fuse.RegExp('<!--\\s*' + Fuse.ScriptFragment + '\\s*-->', 'gi'),
     matchOpenTag      = /<script/i;

    this.evalScripts = function evalScripts() {
      if (this == null) throw new TypeError;
      return Fuse.String(this).extractScripts().map(function(script) {
        return global.eval(String(script));
      });
    };

    this.extractScripts = function extractScripts() {
      if (this == null) throw new TypeError;
      var string = Fuse.String(this), results = Fuse.List();

      if (!matchOpenTag.test(string)) return results;
      var match, scriptTags = string.replace(matchHTMLComments, '');
      while (match = matchScripts.exec(scriptTags))
        match[1] && results.push(match[1]);
      return results;
    };

    this.stripScripts = function stripScripts() {
      if (this == null) throw new TypeError;
      return Fuse.String(this).replace(matchScripts, '');
    };

    // prevent JScript bug with named function expressions
    var evalScripts = null, extractScripts = null, stripScripts = null;
  }).call(Fuse.String.Plugin);

  /*--------------------------------------------------------------------------*/

  (function() {
    var sMap = Fuse.RegExp.specialCharMap.s;

    // ECMA-5 15.5.4.20
    if (!this.trim)
      this.trim = function trim() {
        if (this == null) throw new TypeError;
        var string = String(this), start = -1, end = string.length;

        if (!end) return Fuse.String(string);
        while (sMap[string.charAt(++start)]);
        if (start === end) return Fuse.String('');

        while (sMap[string.charAt(--end)]);
        return Fuse.String(string.slice(start, end + 1));
      };

    // non-standard
    if (!this.trimLeft)
      this.trimLeft = function trimLeft() {
        if (this == null) throw new TypeError;
        var string = String(this), start = -1;

        if (!string) return Fuse.String(string);
        while (sMap[string.charAt(++start)]);
        return Fuse.String(string.slice(start));
      };

    if (!this.trimRight)
      this.trimRight = function trimRight() {
        if (this == null) throw new TypeError;
        var string = String(this), end = string.length;

        if (!end) return Fuse.String(string);
        while (sMap[string.charAt(--end)]);
        return Fuse.String(string.slice(0, end + 1));
      };

    // prevent JScript bug with named function expressions
    var trim = null, trimLeft = null, trimRight = null;
  }).call(Fuse.String.Plugin);

  /*--------------------------------------------------------------------------*/

  (function() {
    // Information on parsing tags can be found at
    // http://www.w3.org/TR/REC-xml-names/#ns-using
    var matchTags = (function() {
      var name   = '\\w+',
       space     = '[\\x20\\x09\\x0D\\x0A]',
       eq        = space + '?=' + space + '?',
       charRef   = '&#[0-9]+;',
       entityRef = '&' + name + ';',
       reference = entityRef + '|' + charRef,
       attValue  = '"(?:[^<&"]|' + reference + ')*"|\'(?:[^<&\']|' + reference + ')*\'',
       attribute = '(?:' + name + eq + attValue + '|' + name + ')';

      return new RegExp('<'+ name + '(?:' + space + attribute + ')*' + space + '?/?>|' +
        '</' + name + space + '?>', 'g');
    })();

    this.escapeHTML = (function() {
      var escapeHTML = function escapeHTML() {
        if (this == null) throw new TypeError;
        textNode.data = String(this);
        return Fuse.String(container.innerHTML);
      };

      var container = Fuse._doc.createElement('pre'),
       textNode = container.appendChild(Fuse._doc.createTextNode(''));

      // Safari 2.x has issues with escaping html inside a "pre"
      // element so we use the deprecated "xmp" element instead.
      if ((textNode.data = '&') && container.innerHTML !== '&amp;') {
        container = Fuse._doc.createElement('xmp');
        textNode = container.appendChild(Fuse._doc.createTextNode(''));
      }

      // Safari 3.x has issues with escaping the ">" character
      if ((textNode.data = '>') && container.innerHTML !== '&gt;') {
        escapeHTML = function escapeHTML() {
          if (this == null) throw new TypeError;
          textNode.data = String(this);
          return Fuse.String(container.innerHTML.replace(/>/g, '&gt;'));
        };
      }
      return escapeHTML;
    })();

    this.unescapeHTML = (function() {
      var div = Fuse._div, tags = [],
       matchToken = new RegExp(expando + '\\d+' + expando, 'g');

      function _swapTagsToTokens(tag) {
        var length = tags.length;
        tags.push(tag);
        return expando + length + expando;
      }

      function _swapTokensToTags(token) {
        return tags[token.slice(15).slice(0, -15)];
      }

      function unescapeHTML() {
        if (this == null) throw new TypeError;
        var result, string = String(this);

        // tokenize tags before setting innerHTML then swap them after
        tags.length = 0;
        if (string.indexOf('<') > -1) string = string.replace(matchTags, _swapTagsToTokens);
        div.innerHTML = '<pre>' + string + '<\/pre>';

        result = Fuse.String(_unescape());
        return tags.length
          ? result.replace(matchToken, _swapTokensToTags)
          : result;
      }

      var _unescape = function() { return div.textContent };
      if (!Feature('ELEMENT_TEXT_CONTENT')) {
        div.innerHTML = '<pre>&lt;p&gt;x&lt;/p&gt;<\/pre>';
        if (Feature('ELEMENT_INNER_TEXT') && div.firstChild.innerText === '<p>x<\/p>')
          _unescape = function() { return div.firstChild.innerText.replace(/\r/g, '') };
        else if (div.firstChild.innerHTML === '<p>x<\/p>')
          _unescape = function() { return div.firstChild.innerHTML };
        else _unescape = function() {
          var node, nodes = div.firstChild.childNodes, parts = [], i = 0;
          while (node = nodes[i++]) parts.push(node.nodeValue);
          return parts.join('');
        };
      }
      return unescapeHTML;
    })();

    this.stripTags = (function() {
      function stripTags() {
        if (this == null) throw new TypeError;
        return Fuse.String(String(this).replace(matchTags, ''));
      }
      return stripTags;
    })();

    // cleanup
    Fuse._div.innerHTML = '';
  }).call(Fuse.String.Plugin);

  /*------------------------------- LANG: HASH -------------------------------*/

  Fuse.addNS('Hash', Fuse.Enumerable, (function() {
    function _indexOfKey(hash, key) {
      key = String(key);
      var index = 0, keys = hash._keys, length = keys.length;
      for ( ; index < length; index++)
        if (keys[index] == key) return index;
    }

    function _set(hash, key, value) {
      if (!key.length) return hash;
      var data = hash._data, expandoKey = expando + key, keys = hash._keys;

      // avoid a method call to Hash#hasKey
      if (expandoKey in data)
        _unsetByIndex(hash, _indexOfKey(hash, key));

      keys.push(key = Fuse.String(key));

      hash._pairs.push(Fuse.List(key, value));
      hash._values.push(value);

      hash._data[expandoKey] =
      hash._object[key] = value;
      return hash;
    }

    function _setWithObject(hash, object) {
      if (object instanceof Fuse.Hash) {
        var pair, i = 0, pairs = object._pairs;
        while (pair = pairs[i++]) _set(hash, pair[0], pair[1]);
      }
      else {
        Fuse.Object._each(object, function(value, key) {
          if (Fuse.Object.hasKey(object, key)) _set(hash, key, value);
        });
      }
      return hash;
    }

    function _unsetByIndex(hash, index) {
      var keys = hash._keys;
      delete hash._data[expando + keys[index]];
      delete hash._object[keys[index]];

      keys.splice(index, 1);
      hash._pairs.splice(index, 1);
      hash._values.splice(index, 1);
    }

    return {
      'constructor': (function() {
        function Hash(object) {
          if (!(this instanceof Hash))
            return new Hash(object);
          return _setWithObject(this.clear(), object);
        }
        return Hash;
      })(),

      'merge': (function() {
        function merge(object) {
          return _setWithObject(this.clone(), object);
        }
        return merge;
      })(),

      'set': (function() {
        function set(key, value) {
          return Fuse.Object.isString(key)
            ? _set(this, key, value)
            : _setWithObject(this, key);
        }
        return set;
      })(),

      'unset': (function() {
        function unset(key) {
          var data = this._data, i = 0,
           keys = Fuse.List.isArray(key) ? key : arguments;

          while (key = keys[i++])  {
            if ((expando + key) in data)
              _unsetByIndex(this, _indexOfKey(this, key));
          }
          return this;
        }
        return unset;
      })()
    };
  })());

  /*--------------------------------------------------------------------------*/

  (function() {
    function _returnPair(pair) {
      var key, value;
      pair = Fuse.List(key = pair[0], value = pair[1]);
      pair.key = key;
      pair.value = value;
      return pair;
    }

    this._each = function _each(callback) {
      var pair, i = 0, pairs = this._pairs;
      while (pair = pairs[i]) callback(_returnPair(pair), i++, this);
      return this;
    };

    this.first = function first(callback, thisArg) {
      var pair, i = 0, pairs = this._pairs;
      if (callback == null) {
        if (pairs.length) return _returnPair(pairs[0]);
      }
      else if (typeof callback === 'function') {
        while (pair = pairs[i++]) {
          if (callback.call(thisArg, pair[1], pair[0], this))
            return _returnPair(pair);
        }
      }
      else {
        var count = 1 * callback, results = Fuse.List();
        if (isNaN(count)) return results;
        count = count < 1 ? 1 : count;
        while (i < count && (pair = pairs[i])) results[i++] = _returnPair(pair);
        return results;
      }
    };

    this.last = function last(callback, thisArg) {
      var pair, i = 0, pairs = this._pairs, length = pairs.length;
      if (callback == null) {
        if (length) return _returnPair(this._pairs.last());
      }
      else if (typeof callback === 'function') {
        while (length--) {
          pair = pairs[length];
          if (callback.call(thisArg, pair[1], pair[2], this))
            return _returnPair(pair);
        }
      }
      else {
        var count = 1 * callback, results = Fuse.List();
        if (isNaN(count)) return results;
        count = count < 1 ? 1 : count > length ? length : count;
        var  pad = length - count;
        while (i < count)
          results[i] = _returnPair(pairs[pad + i++]);
        return results;
      }
    };

    this.toArray = function toArray() {
      var pair, i = 0, pairs = this._pairs, results = Fuse.List();
      while (pair = pairs[i]) results[i++] = _returnPair(pair);
      return results;
    };

    // alias
    this.toList = this.toArray;

    // prevent JScript bug with named function expressions
    var _each = null, first = null, last = null, toArray = null;
  }).call(Fuse.Hash.Plugin);

  /*--------------------------------------------------------------------------*/

  (function($H) {
    this.clear = function clear() {
      this._data     = { };
      this._object   = { };
      this._keys     = Fuse.List();
      this._pairs    = Fuse.List();
      this._values   = Fuse.List();
      return this;
    };

    this.clone = function clone() {
      return $H(this);
    };

    this.contains = function contains(value, strict) {
      var pair, i = 0, pairs = this._pairs;
      if (strict) {
        while (pair = pairs[i++]) if (value === pair[1]) return true;
      } else {
        while (pair = pairs[i++]) if (value == pair[1]) return true;
      }
      return false;
    };

    this.filter = function filter(callback, thisArg) {
      var pair, i = 0, pairs = this._pairs, result = new $H();
      callback = callback || function(value) { return value != null };

      while (pair = pairs[i++]) {
        if (callback.call(thisArg, pair[1], pair[0], this))
          result.set(pair[0], pair[1]);
      }
      return result;
    };

    this.get = function get(key) {
      return this._data[expando + key];
    };

    this.grep = function grep(pattern, callback, thisArg) {
      if (!pattern || pattern == '' || Fuse.Object.isRegExp(pattern) &&
         !pattern.source) return this.clone();

      callback = callback || Fuse.K;
      var key, pair, value, i = 0, pairs = this._pairs, result = new $H();
      if (Fuse.Object.isString(pattern))
        pattern = new RegExp(Fuse.RegExp.escape(pattern));

      while (pair = pairs[i++]) {
        if (pattern.test(value = pair[1]))
          result.set(key = pair[0], callback.call(thisArg, value, key, this));
      }
      return result;
    };

    this.hasKey = function hasKey(key) {
      return (expando + key) in this._data;
    };

    this.inspect = function inspect() {
      var pair, i = 0, pairs = this._pairs, results = [];
      while (pair = pairs[i])
        results[i++] = pair[0].inspect() + ': ' + Fuse.Object.inspect(pair[1]);
      return '#<Hash:{' + results.join(', ') + '}>';
    };

    this.keyOf = function keyOf(value) {
      var pair, i = 0, pairs = this._pairs;
      while (pair = pairs[i++]) {
        if (value === pair[1])
          return pair[0];
      }
      return Fuse.Number(-1);
    };

    this.keys = function keys() {
      return Fuse.List.fromArray(this._keys);
    };

    this.map = function map(callback, thisArg) {
      if (!callback) return this;
      var key, pair, i = 0, pairs = this._pairs, result = new $H();

      if (thisArg) {
        while (pair = pairs[i++])
          result.set(key = pair[0], callback.call(thisArg, pair[1], key, this));
      } else {
        while (pair = pairs[i++])
          result.set(key = pair[0], callback(pair[1], key, this));
      }
      return result;
    };

    this.partition = function partition(callback, thisArg) {
      callback = callback || Fuse.K;
      var key, value, pair, i = 0, pairs = this._pairs,
       trues = new $H(), falses = new $H();

      while (pair = pairs[i++])
        (callback.call(thisArg, value = pair[1], key = pair[0], this) ?
          trues : falses).set(key, value);
      return Fuse.List(trues, falses);
    };

    this.size = function size() {
      return Fuse.Number(this._keys.length);
    };

    this.toArray = function toArray() {
      return Fuse.List.fromArray(this._pairs);
    };

    this.toObject = function toObject() {
      var pair, i = 0, pairs = this._pairs, result = Fuse.Object();
      while (pair = pairs[i++]) result[pair[0]] = pair[1];
      return result;
    };

    this.toQueryString = function toQueryString() {
      return Fuse.Object.toQueryString(this._object);
    };

    this.values = function values() {
      return Fuse.List.fromArray(this._values);
    };

    this.zip = function zip() {
      var callback = Fuse.K, args = slice.call(arguments, 0);
      if (typeof Fuse.List.Plugin.last.call(args) === 'function')
        callback = args.pop();

      var result = new $H(),
       hashes = prependList(Fuse.List.Plugin.map.call(args, $H), this),
       length = hashes.length;

      var j, key, pair, i = 0, pairs = this._pairs;
      while (pair = pairs[i++]) {
        j = 0; values = Fuse.List(); key = pair[0];
        while (j < length) values[j] = hashes[j++]._data[expando + key];
        result.set(key, callback(values, key, this));
      }
      return result;
    };

    // prevent JScript bug with named function expressions
    var clear =      null,
     clone =         null,
     contains =      null,
     filter =        null,
     get =           null,
     grep =          null,
     hasKey =        null,
     keys =          null,
     keyOf =         null,
     inspect =       null,
     map =           null,
     partition =     null,
     size =          null,
     toArray =       null,
     toObject =      null,
     toQueryString = null,
     values =        null,
     zip =           null;
  }).call(Fuse.Hash.Plugin, Fuse.Hash);

  /*--------------------------------------------------------------------------*/

  Fuse.addNS('Util');

  Fuse.Util.$H = Fuse.Hash.from = Fuse.Hash;

  /*------------------------------- LANG: RANGE ------------------------------*/

  Fuse.addNS('Range', Fuse.Enumerable, {
    'constructor': (function() {
      function Range(start, end, exclusive) {
        if (!(this instanceof Range))
          return new Range(start, end, exclusive);

        this.start = Fuse.Object(start);
        this.end = Fuse.Object(end);
        this.exclusive = exclusive;
      }
      return Range;
    })()
  });

  /*--------------------------------------------------------------------------*/

  (function() {
    function _buildCache(thisArg, callback) {
      callback = callback || Fuse.emptyFunction;
      var c = thisArg._cache = Fuse.List(), i = 0,
       value = c.start = thisArg.start = Fuse.Object(thisArg.start);

      c.end = thisArg.end = Fuse.Object(thisArg.end);
      c.exclusive = thisArg.exclusive;

      while (_inRange(thisArg, value)) {
        c.push(value);
        callback(value, i++, thisArg);
        value = value.succ();
      }
    }

    function _isExpired(thisArg) {
      var c = thisArg._cache, result = false;
      if (!c || thisArg.start != c.start || thisArg.end != c.end)
        result = true;
      else if (thisArg.exclusive !== c.exclusive) {
        c.exclusive = thisArg.exclusive;
        if (c.exclusive) c.pop();
        else {
          var last = c[c.length - 1];
          c.push(last.succ());
        }
      }
      return result;
    }

    function _inRange(thisArg, value) {
      if (value < thisArg.start)
        return false;
      if (thisArg.exclusive)
        return value < thisArg.end;
      return value <= thisArg.end;
    }

    this._each = function _each(callback) {
      if (_isExpired(this)) _buildCache(this, callback);
      else {
        var c = this._cache, i = 0, length = c.length;
        while (i < length) callback(c[i], i++ , this);
      }
    };

    this.max = (function(__max) {
      function max(callback, thisArg) {
        var result;
        if (!callback && !_isExpired(this))
          result = this._cache[this._cache.length - 1];
        else result = __max.call(this, callback, thisArg);
        return result;
      }
      return max;
    })(this.max);

    this.min = (function(__min) {
      function min(callback, thisArg) {
        return !callback
          ? this.start
          : __min.call(this, callback, thisArg);
      }
      return min;
    })(this.min);

    this.size = function size() {
      var c = this._cache, isNumber = Fuse.Object.isNumber;
      if (_isExpired(this)) {
        if (isNumber(this.start) && isNumber(this.end))
          return Fuse.Number(this.end - this.start + (this.exclusive ? 0 : 1));
        _buildCache(this);
      }
      return Fuse.Number(this._cache.length);
    };

    this.toArray = function toArray() {
      _isExpired(this) && _buildCache(this);
      return Fuse.List.fromArray(this._cache);
    };

    // prevent JScript bug with named function expressions
    var _each = null, size = null, toArray = null;
  }).call(Fuse.Range.Plugin);

  /*--------------------------------------------------------------------------*/
  
  Fuse.addNS('Util');

  Fuse.Util.$R = Fuse.Range;

  /*----------------------------- LANG: TEMPLATE -----------------------------*/

  Fuse.addNS('Template', {
    'constructor': (function() {
      function Template(template, pattern) {
        if (!(this instanceof Template))
          return new Template(template, pattern);

        this.template = Fuse.String(template);

        pattern = pattern || Fuse.Template.Pattern;
        if (!Fuse.Object.isRegExp(pattern))
          pattern = Fuse.RegExp(Fuse.RegExp.escape(pattern));
        if (!pattern.global)
          pattern = Fuse.RegExp.clone(pattern, { 'global': true });
        this.pattern = pattern;
      }
      return Template;
    })(),

    'evaluate': (function() {
      function evaluate(object) {
        if (object) {
          if (object instanceof Fuse.Hash)
            object = object._object;
          else if (typeof object.toTemplateReplacements === 'function')
            object = object.toTemplateReplacements();
          else if (typeof object.toObject === 'function')
            object = object.toObject();
        }

        return this.template.replace(this.pattern, function(match, before, escaped, expr) {
          before = before || '';
          if (before === '\\') return escaped;
          if (object == null) return before;

          // adds support for dot and bracket notation
          var comp, ctx = object, 
           value   = ctx,
           pattern = /^([^.[]+|\[((?:.*?[^\\])?)\])(\.|\[|$)/;

          match = pattern.exec(expr);
          if (match == null) return before;

          while (match != null) {
            comp  = match[1].indexOf('[') === 0 ? match[2].replace(/\\]/g, ']') : match[1];
            value = ctx[comp];
            if (!Fuse.Object.hasKey(ctx, comp) || value == null) {
              value = ''; break;
            }
            if ('' == match[3]) break;
            ctx   = value;
            expr  = expr.substring('[' == match[3] ? match[1].length : match[0].length);
            match = pattern.exec(expr);
          }
          return before + Fuse.String.interpret(value);
        });
      }
      return evaluate;
    })()
  });

  Fuse.Template.Pattern = /(\\)?(#\{([^}]*)\})/;

  /*------------------------ LANG: TIMER -----------------------*/

  Fuse.addNS('Timer', {
    'constructor': (function() {
      function Timer(callback, interval, options) {
        if (!(this instanceof Timer))
          return new Timer(callback, interval, options);

        this.callback     = callback;
        this.interval     = interval;
        this.executing    = false;
        this.onTimerEvent = Fuse.Function.bind(onTimerEvent, this);

        this.options = Fuse.Object._extend(Fuse.Object
          .clone(this.constructor.options), options);
      }

      function onTimerEvent() {
        if (!this.executing) {
          this.executing = true;

          // IE6 bug with try/finally, the finally does not get executed if the
          // exception is uncaught. So instead we set the flags and start the
          // timer before throwing the error.
          try {
            this.execute();
            this.executing = false;
            if (this.timerID !== null) this.start();
          }
          catch (e) {
            this.executing = false;
            if (this.timerID !== null) this.start();
            throw e;
          }
        }
      }
      return Timer;
    })()
  });

  (function() {
    this.execute = function execute() {
      this.callback(this);
    };

    this.start = function start() {
      this.timerID = global.setTimeout(this.onTimerEvent,
        this.interval * this.options.multiplier);
      return this;
    };

    this.stop = function stop() {
      var id = this.timerID;
      if (id === null) return;
      global.clearTimeout(id);
      this.timerID = null;
      return this;
    };

    // prevent JScript bug with named function expressions
    var execute = null, start = null, stop = null;
  }).call(Fuse.Timer.Plugin);

  Fuse.Timer.options = {
    'multiplier': 1
  };

  /*------------------------------- LANG: JSON -------------------------------*/

  (function() {
    Fuse.Hash.Plugin.toJSON = function toJSON() {
      return Fuse.Object.toJSON(this.toObject());
    };

    Fuse.List.Plugin.toJSON = function toJSON() {
      for (var value, i = 0, results = Fuse.List(), length = this.length; i < length; i++) {
        value = Fuse.Object.toJSON(this[i]);
        if (typeof value !== 'undefined') results.push(value);
      }
      return '[' + results.join(', ') + ']';
    };

    Fuse.Object.toJSON = function toJSON(value) {
      switch (typeof value) {
        case 'undefined':
        case 'function' :
        case 'unknown'  : return;
        case 'boolean'  : return Fuse.String(value);
      }

      if (value === null) return Fuse.String(null);
      var object = Fuse.Object(value);
      if (typeof object.toJSON === 'function') return object.toJSON();
      if (Fuse.Object.isElement(value)) return;

      var results = [];
      Fuse.Object._each(object, function(value, key) {
        value = Fuse.Object.toJSON(value);
        if (typeof value !== 'undefined')
          results.push(Fuse.String(key).toJSON() + ': ' + value);
      });
      return Fuse.String('{' + results.join(', ') + '}');
    };

    // ECMA-5 15.9.5.44
    if (!Fuse.Date.Plugin.toJSON)
      Fuse.Date.Plugin.toJSON = function toJSON() {
        return Fuse.String('"' + this.getUTCFullYear() + '-' +
          Fuse.Number(this.getUTCMonth() + 1).toPaddedString(2) + '-' +
          this.getUTCDate().toPaddedString(2)    + 'T' +
          this.getUTCHours().toPaddedString(2)   + ':' +
          this.getUTCMinutes().toPaddedString(2) + ':' +
          this.getUTCSeconds().toPaddedString(2) + 'Z"');
      };

    // ECMA-5 15.7.4.8
    if (!Fuse.Number.Plugin.toJSON)
      Fuse.Number.Plugin.toJSON = function toJSON() {
        return Fuse.String(isFinite(this) ? this : 'null');
      };

    // ECMA-5 15.5.4.21
    if (!Fuse.String.Plugin.toJSON)
      Fuse.String.Plugin.toJSON = function toJSON() {
        return Fuse.String(this).inspect(true);
      };

    // prevent JScript bug with named function expressions
    var toJSON = null;
  })();

  /*--------------------------------------------------------------------------*/

  // complementary JSON methods for String.Plugin
  (function() {
    this.evalJSON = function evalJSON(sanitize) {
      if (this == null) throw new TypeError;
      var string = Fuse.String(this), json = string.unfilterJSON();
      try {
        if (!sanitize || json.isJSON())
          return global.eval('(' + String(json) + ')');
      } catch (e) { }
      throw new SyntaxError('Badly formed JSON string: ' + string.inspect());
    };

    this.isJSON = function isJSON() {
      if (this == null) throw new TypeError;
      var string = String(this);
      if (/^\s*$/.test(string)) return false;

      string = string.replace(/\\./g, '@').replace(/"[^"\\\n\r]*"/g, '');
      return (/^[,:{}\[\]0-9.\-+Eaeflnr-u \n\r\t]*$/).test(string);
    };

    this.unfilterJSON = function unfilterJSON(filter) {
      if (this == null) throw new TypeError;
      return Fuse.String(String(this).replace(filter || Fuse.JSONFilter, '$1'));
    };

    // prevent JScript bug with named function expressions
    var evalJSON = null, isJSON = null, unfilterJSON = null;
  }).call(Fuse.String.Plugin);

  /*---------------------------------- AJAX ----------------------------------*/

  Fuse.addNS('Ajax');

  Fuse.Ajax.activeRequestCount = 0;

  Fuse.Ajax.getTransport = (function() {
    var getTransport = function getTransport() { return false };

    // check ActiveXObject first because IE7+ implementation of
    // XMLHttpRequest doesn't work with local files.
    if (Feature('ACTIVE_X_OBJECT')) {
      // the `Difference between MSXML2.XmlHttp and Microsoft.XmlHttp ProgIDs`
      // post explains why MSXML2.XmlHttp is not needed:
      // http://forums.asp.net/p/1000060/1622845.aspx
      getTransport = function getTransport() {
        return new ActiveXObject('Microsoft.XMLHTTP');
      };
    }
    else if (isHostObject(global, 'XMLHttpRequest')) {
      getTransport = function getTransport() {
        return new XMLHttpRequest();
      };
    }
    return getTransport;
  })();

  /*---------------------------- AJAX: RESPONDERS ----------------------------*/

  Fuse.addNS('Ajax.Responders');

  // TODO: Utilize custom events for responders
  (function() {
    this.responders = {
      'onCreate': Fuse.List(function() { Fuse.Ajax.activeRequestCount++ }),
      'onDone':   Fuse.List(function() { Fuse.Ajax.activeRequestCount-- })
    };

    this.dispatch = (function() {
      // This pattern, based on work by Dean Edwards and John Resig, allows a
      // responder to error out without stopping the other responders from firing.
      // http://groups.google.com/group/jquery-dev/browse_thread/thread/2a14c2da6bcbb5f
      function _dispatch(index, handlers, request, json) {
        index = index || 0;
        var error, length = handlers.length;
        try {
          while (index < length) {
            handlers[index](request, json);
            index++;
          }
        } catch (e) {
          error = e;
          _dispatch(index + 1, handlers, request, json);
        } finally {
          if (error) throw error;
        }
      }

      function dispatch(handlerName, request, json) {
        var handlers = this.responders[handlerName];
        if (handlers) _dispatch(0, handlers, request, json);
      }

      return dispatch;
    })();

    this.register = function register(responder) {
      var m, handler, handlers, name;
      if (responder instanceof Fuse.Hash)
        responder = responder._object;

      for (name in responder) {
        handlers = this.responders[name];
        m = responder[name];
        if (!handlers || !handlers.first(function(c) { return c.__method === m })) {
          (handler = Fuse.Function.bind(m, responder)).__method = m;
          if (!handlers) this.responders[name] = handlers = Fuse.List();
          handlers.push(handler);
        }
      }
    };

    this.unregister = function unregister(responder) {
      var name;
      if (responder instanceof Fuse.Hash)
        responder = responder._object;

      for (name in responder) {
        var handlers = this.responders[name];
        if (handlers)
          this.responders[name] =
            handlers.filter(function(c) { return c.__method !== responder[name] });
      }
    };

    // prevent JScript bug with named function expressions
    var dispatch = null, register = null, unregister =  null;
  }).call(Fuse.Ajax.Responders);

 /*------------------------------- AJAX: BASE -------------------------------*/

  Fuse.addNS('Ajax.Base', {
    'constructor': (function() {
      function Base(url, options) {
        var customHeaders, queryString,
         body = null,
         defaultOptions = Fuse.Ajax.Base.options,
         defaultHeaders = defaultOptions.headers,
         location = global.location;

        // remove headers from user options to be added in further down
        if (options && options.headers) {
          customHeaders = options.headers;
          delete options.headers;
        }

        // clone default options/headers and overwrite with user options
        delete defaultOptions.headers;
        defaultOptions = Fuse.Object.clone(defaultOptions);
        Fuse.Ajax.Base.options.headers = defaultHeaders;

        defaultOptions.headers = Fuse.Object.clone(defaultHeaders);
        options = this.options = Fuse.Object._extend(defaultOptions, options);

        var encoding = options.encoding,
         headers = options.headers,
         method = options.method.toLowerCase(),
         params = options.parameters;

        // if no url is provided use the window's location data
        if (!url || url == '') {
          url = location.protocol + '//' + location.host + location.pathname;
          if (!params || params == '')
            params = location.search.slice(1);
        }

        // convert string/hash parameters to an object
        if (Fuse.Object.isString(params))
          params = Fuse.String(params).toQueryParams();
        else if (params instanceof Fuse.Hash)
          params = params.toObject();
        else params = Fuse.Object.clone(params);

        // simulate other verbs over post
        if (!/^(get|post)$/.test(method)) {
          params['_method'] = method;
          method = 'post';
        }

        // when GET request, append parameters to URL
        queryString = Fuse.Object.toQueryString(params);
        if ( method == 'get' && queryString != '')
          url += (url.indexOf('?') > -1 ? '&' : '?') + queryString;

        // add in user defined array/hash/object headers over the default
        if (typeof customHeaders === 'object') {
          if (Fuse.List.isArray(customHeaders)) {
            for (var i = 0, length = customHeaders.length; i < length; i += 2)
              headers[customHeaders[i]] = customHeaders[i + 1];
          } else {
            if (customHeaders instanceof Fuse.Hash) customHeaders = customHeaders._object;
            for (key in customHeaders) headers[key] = customHeaders[key];
          }
        }

        // ensure character encoding is set in headers of POST requests
        if (method == 'post' && (headers['Content-type'] || '').indexOf('charset=') == -1) {
          headers['Content-type'] = options.contentType +
            (encoding ? '; charset=' + encoding : '');
        }

        // Playing it safe here, even though we could not reproduce this bug,
        // jQuery tickets #2570, #2865 report versions of Opera will display a
        // login prompt when passing null-like values for username/password when
        // no authorization is needed.
        if (!options.username) options.username = options.password = '';

        // body is null for every method except POST
        if (method == 'post')
          body = options.postBody || queryString;

        this.body       = body;
        this.method     = Fuse.String(method);
        this.parameters = params;
        this.url        = Fuse.String(url);
      }

      return Base;
    })()
  });

  Fuse.Ajax.Base.options = {
    'asynchronous': true,
    'contentType':  'application/x-www-form-urlencoded',
    'encoding':     'UTF-8',
    'evalJS':       true,
    'evalJSON':     true,
    'forceMethod':  false,
    'method':       'post',
    'parameters':   '',
    'headers':      {
      'Accept': 'text/javascript, text/html, application/xml, text/xml, */*',
      'X-Fuse-Version': Fuse.Version,
      'X-Requested-With': 'XMLHttpRequest'
    }
  };

  /*---------------------------- AJAX: REQUEST -------------------------------*/

  Fuse.addNS('Ajax.Request', Fuse.Ajax.Base, {
    'constructor': (function() {
      function Request(url, options) {
        if (!(this instanceof Request))
          return new Request(url, options);

        this.transport = Fuse.Ajax.getTransport();
        this.onStateChange = Fuse.Function.bind(this.onStateChange, this);
        this.request(url, options);
      }
      return Request;
    })()
  });

  Fuse.Ajax.Request.Events =
    Fuse.List('Unsent', 'Opened', 'HeadersRecieved', 'Loading', 'Done');

  /*--------------------------------------------------------------------------*/

  (function() {
    this.aborted      = false;
    this.readyState   = Fuse.Number(0);
    this.responseText = Fuse.String('');
    this.status       = Fuse.Number(0);
    this.statusText   = Fuse.String('');

    this.headerJSON = this.responseJSON = this.responseXML = null;

    this.abort = function abort() {
      var transport = this.transport;
      if (this.readyState != 4) {
        try {
          // clear onreadystatechange handler to stop some browsers calling
          // it when the request is aborted
          transport.onreadystatechange = Fuse.emptyFunction;
          transport.abort();

          // skip to complete readyState
          this.aborted = true;
          this.setReadyState(4);
        }
        catch (e) { }
      }
    };

    this.dispatch = function dispatch(eventName, callback) {
      try {
        callback && callback(this, this.headerJSON);
      } catch (e) {
        this.dispatchException(e);
      }
      Fuse.Ajax.Responders.dispatch(eventName, this, this.headerJSON);
    };

    this.dispatchException = function dispatchException(exception) {
      var callback = this.options.onException;
      callback && callback(this, exception);
      Fuse.Ajax.Responders.dispatch('onException', this, exception);
    };

    this.getAllHeaders = function getAllHeaders() {
      var result;
      try { result = this.transport.getAllResponseHeaders() } catch (e) { }
      return Fuse.String(result || '');
    };

    this.getHeader = function getHeader(name) {
      var result;
      try { result = this.transport.getResponseHeader(name) } catch (e) { }
      return result ? Fuse.String(result) : null;
    };

    this.onStateChange = function onStateChange(event, forceState) {
      // ensure all states are fired and only fired once per change
      var endState = this.transport.readyState, readyState = this.readyState;
      if (readyState < 4) {
        if (forceState != null) readyState = forceState - 1;
        while (readyState < endState)
          this.setReadyState(++readyState);
      }
    };

    this.request = function request(url, options) {
      // treat request() as the constructor and call Base as $super
      // if first call or new options are passed
      if (!this.options || options)
        Fuse.Ajax.Base.call(this, url, options);

      var key,
       options   = this.options,
       async     = options.asynchronous,
       body      = this.body,
       headers   = options.headers,
       transport = this.transport;

      // fire onCreate callbacks
      this.dispatch('onCreate', options.onCreate);

      // trigger uninitialized readyState 0
      this.onStateChange(null, 0);

      try {
        // attach onreadystatechange event after open() to avoid some browsers
        // firing duplicate readyState events
        transport.open(this.method.toUpperCase(), this.url, async,
          options.username, options.password);
        transport.onreadystatechange = this.onStateChange;

        // set headers
        for (key in headers)
          transport.setRequestHeader(key, headers[key]);

        // if body is a string ensure it's a primitive
        transport.send(Fuse.Object.isString(body) ? String(body) : body);

        // force Firefox to handle readyState 4 for synchronous requests
        if (!async) this.onStateChange();
      }
      catch (e) {
        this.dispatchException(e);
      }
    };

    this.setReadyState = function setReadyState(readyState) {
      var eventName, successOrFailure, i = 0,
       eventNames = [],
       skipped    = { },
       aborted    = this.aborted,
       options    = this.options,
       url        = this.url,
       transport  = this.transport;

      // clear response values on readyState 0 or aborted requests
      if (readyState == 0 || aborted) {
        this.headerJSON = this.responseJSON = this.responseXML = null;
        this.responseText = Fuse.String('');
      }

      if (readyState == 2) {
        // exit if no headers and wait for state 3 to fire states 2 and 3
        if (this.getAllHeaders() == '' && transport.readyState === 2)
          return;

        this.readyState = Fuse.Number(readyState);

        // set headerJSON
        var json = this.getHeader('X-JSON');
        if (json && json != '') {
          try {
            this.headerJSON = json.evalJSON(options.sanitizeJSON ||
              !Fuse.Object.isSameOrigin(url));
          } catch (e) {
            this.dispatchException(e);
          }
        }
      }
      else if (readyState > 2) {

        this.readyState = Fuse.Number(readyState);

        // set status
        try { this.status = Fuse.Number(transport.status || 0) } catch (e) { }

        // set statusText
        try { this.statusText = Fuse.String(transport.statusText || '') } catch (e) { }

        // set responseText
        if (!aborted)
          this.responseText = Fuse.String.interpret(transport.responseText);

        if (readyState == 4) {
          var responseXML,
           contentType = this.getHeader('Content-type') || '',
           evalJS = options.evalJS,
           evalJSON = options.evalJSON,
           responseText = this.responseText,
           status = String(this.status);

          if (aborted) {
            eventNames.push('Abort', status);
          }
          else {
            // don't call global/request onSuccess/onFailure callbacks on aborted requests
            successOrFailure = this.isSuccess() ? 'Success' : 'Failure';
            eventNames.push(status, successOrFailure);

            // skip success/failure request events if status handler exists
            skipped['on' + (options['on' + status] ?
              successOrFailure : status)] = 1;

            // remove event handler to avoid memory leak in IE
            transport.onreadystatechange = Fuse.emptyFunction;

            // set responseXML
            responseXML = transport.responseXML;
            if (responseXML) this.responseXML = responseXML;

            // set responseJSON
            if (evalJSON == 'force' || (evalJSON && !responseText.blank() &&
                contentType.indexOf('application/json') > -1)) {
              try {
                this.responseJSON = responseText.evalJSON(options.sanitizeJSON ||
                  !Fuse.Object.isSameOrigin(url));
              } catch (e) {
                this.dispatchException(e);
              }
            }

            // eval javascript
            if (responseText && (evalJS == 'force' || evalJS &&
                Fuse.Object.isSameOrigin(url) &&
                contentType.match(/^\s*(text|application)\/(x-)?(java|ecma)script(;|\s|$)/i))) {
              try {
                global.eval(String(Fuse.String.unfilterJSON(responseText)));
              } catch (e) {
                this.dispatchException(e);
              }
            }
          }
        }
      }

      // add readyState to the list of events to dispatch
      eventNames.push(Fuse.Ajax.Request.Events[readyState]);

      while (eventName = eventNames[i++]) {
        eventName = 'on' + eventName;
        this.dispatch(eventName, !skipped[eventName] && options[eventName]);
      }
    };

    this.isSuccess = function isSuccess() {
      var status = this.status;
      return status == 0 || (status >= 200 && status < 300);
    };

    // prevent JScript bug with named function expressions
    var abort =          null,
     dispatch =          null,
     dispatchException = null,
     getHeader =         null,
     getAllHeaders =     null,
     isSuccess =         null,
     onStateChange =     null,
     request =           null,
     setReadyState =     null;
  }).call(Fuse.Ajax.Request.Plugin);

  /*------------------------------ AJAX: UPDATER -----------------------------*/

  Fuse.addNS('Ajax.Updater', Fuse.Ajax.Request, {
    'constructor': (function() {
      function Updater(container, url, options) {
        if (!(this instanceof Updater))
          return new Updater(container, url, options);

        this.container = {
          'success': (container.success || container),
          'failure': (container.failure || (container.success ? null : container))
        };

        options = Fuse.Object.clone(options);
        var updater = this, callbackName = 'on' + Request.Events[4],
         onDone = options[callbackName];

        options[callbackName] = function(request, json) {
          updater.updateContent(request.responseText);
          onDone && onDone(request, json);
        };

        // this._super() equivalent
        Fuse.Ajax.Request.call(this, url, options);
      }

      var Request = Fuse.Ajax.Request;
      return Updater;
    })(),

    'updateContent': (function() {
      function updateContent(responseText) {
        var receiver = this.container[this.isSuccess() ? 'success' : 'failure'],
         options = this.options;

        if (!options.evalScripts)
          responseText = responseText.stripScripts();
        if (receiver = $(receiver)) {
          if (options.insertion) {
            if (Fuse.Object.isString(options.insertion)) {
              var insertion = { }; insertion[options.insertion] = responseText;
              receiver.insert(insertion);
            }
            else options.insertion(receiver, responseText);
          }
          else receiver.update(responseText);
        }
      }
      return updateContent;
    })()
  });

  /*------------------------ AJAX: PERIODICAL UPDATER ------------------------*/

  Fuse.addNS('Ajax.TimedUpdater', Fuse.Ajax.Base, {
    'constructor': (function() {
      function TimedUpdater(container, url, options) {
        if (!(this instanceof TimedUpdater))
          return new TimedUpdater(container, url, options);

        options = Fuse.Object._extend(Fuse.Object
          .clone(this.constructor.options), options);

        // this._super() equivalent
        Fuse.Ajax.Base.call(this, url, options);
        options = this.options;

        this.onStop = options.onStop;
        this.onTimerEvent = Fuse.Function.bind(this.start, this);

        this.updater = { };
        this.container = container;
        this.url = url;

        // dynamically set readyState eventName to allow for easy customization
        var callbackName = 'on' + Request.Events[4],
         onDone = options[callbackName];

        options[callbackName] = Fuse.Function.bind(function(request, json) {
          if (!request.aborted) {
            this.updateDone(request);
            onDone && onDone(request, json);
          }
        }, this);

        this.start();
      }

      var Request = Fuse.Ajax.Request;
      return TimedUpdater;
    })()
  });

  (function() {
    this.updateDone = function updateDone(request) {
      var options = this.options, responseText = request.responseText;
      if (options.decay) {
        this.decay = (responseText == this.lastText ?
          this.decay * options.decay : options.decay);

        this.lastText = responseText;
      }
      this.timer = Fuse.Function.delay(this.onTimerEvent,
        Math.min(this.decay * options.frequency || options.maxDecay));
    };

    this.start = function start() {
      this.updater = new Fuse.Ajax.Updater(this.container, this.url, this.options);
    };

    this.stop = function stop() {
      this.updater.abort();
      global.clearTimeout(this.timer);
      this.onStop && this.onStop.apply(this, arguments);
    };

    // prevent JScript bug with named function expressions
    var updateDone = null, start = null, stop = null;
  }).call(Fuse.Ajax.TimedUpdater.Plugin);

  Fuse.Ajax.TimedUpdater.options = {
    'decay':     1,
    'frequency': 2,
    'maxDecay':  Infinity
  };

  /*-------------------------------- ELEMENT ---------------------------------*/

  Fuse.addNS('Util');

  $ = Fuse.Util.$ = (function() {
    function $(element) {
      if (arguments.length > 1) {
        for (var i = 0, elements = Fuse.List(), length = arguments.length; i < length; i++)
          elements.push($(arguments[i]));
        return elements;
      }
      if (Fuse.Object.isString(element))
        element = Fuse._doc.getElementById(element || expando);
      return Element.extend(element);
    }
    return $;
  })();

  /*--------------------------------------------------------------------------*/

  // cache Element capabilities before overwriting the Element object
  Feature('ELEMENT_EXTENSIONS');
  Feature('ELEMENT_SPECIFIC_EXTENSIONS');

  (function() {
    var original = global.Element;

    function createElement(tagName, attributes) {
      if (!Element.cache[tagName])
        Element.cache[tagName] = Element.extend(Fuse._doc.createElement(tagName));
      return Element.writeAttribute(Element.cache[tagName]
        .cloneNode(false), attributes);
    }

    global.Element = function(tagName, attributes) {
      return createElement(tagName.toUpperCase(), attributes);
    };

    if (Feature('CREATE_ELEMENT_WITH_HTML')) {
      global.Element = function(tagName, attributes) {
        // setAttribute is broken in IE when setting name and type attributes
        // see: http://msdn.microsoft.com/en-us/library/ms536389.aspx
        tagName = tagName.toUpperCase();
        if (attributes && (attributes.name || attributes.type)) {
          tagName = '<' + tagName +
           (attributes.name ? ' name="' + attributes.name + '"' : '') +
            (attributes.type ? ' type="' + attributes.type + '"' : '') + '>';
          delete attributes.name; delete attributes.type;
        }
        return createElement(tagName, attributes);
      };
    }

    if (original) {
      // avoid Fuse.Object.extend() because IE8 cannot set any
      // variable/property reference to Element.toString
      Fuse.Object._extend(global.Element, original);
      global.Element.prototype = original.prototype;
    }
  })();

  Element.cache = { };
  Element.idCounter = 1;

  /*--------------------------------------------------------------------------*/

  Element.extend = (function() {
    var Methods, ByTag, revision = 0;

    function _createRevisionGetter(r) {
      return function() { return r };
    }

    function _extendElement(element, nodeName) {
      nodeName = nodeName || getNodeName(element);
      var pair, methods = ByTag[nodeName] || Methods, length = methods.length;
      while (length--) {
        pair = methods[length];
        if (!Fuse.Object.hasKey(element, pair[0]))
          element[pair[0]] = pair[1];
      }

      // avoid using Fuse.K.curry(revision) for speed
      element._extendedByFuse = _createRevisionGetter(revision);
      return element;
    }

    function extend(element) {
      // Bail on elements that don't need extending,
      // XML nodes (IE errors on them), document, window objects
      if (!element || (typeof element._extendedByFuse !== 'undefined' &&
        element._extendedByFuse() >= revision) ||
        element.nodeType !== 1 || element == getWindow(element) ||
        !element.ownerDocument.body) return element;

      return _extendElement(element);
    }

    function refresh() {
      var tagName; Methods = []; ByTag = { };

      Fuse.Object._each(Element.Methods, function(value, key, object) {
        if (key !== 'Simulated' && key !== 'ByTag')
          Methods.push([key, Fuse.Function.methodize([key, object])]);
      });

      Fuse.Object._each(Element.Methods.Simulated, function(value, key, object) {
        Methods.push([key, Fuse.Function.methodize([key, object])]);
      });

      for (tagName in Element.Methods.ByTag) {
        ByTag[tagName] = slice.call(Methods, 0);
        Fuse.Object._each(Element.Methods.ByTag[tagName], function(value, key, object) {
          ByTag[tagName].push([key, Fuse.Function.methodize([key, object])]);
        });
      }
      revision++;
    }

    // Browsers with specific element extensions
    // don't need their elements extended UNLESS
    // they belong to a different document
    if (Feature('ELEMENT_SPECIFIC_EXTENSIONS')) {
      extend = (function(_extend) {
        function extend(element) {
          return (element && element.ownerDocument &&
            element.ownerDocument !== Fuse._doc) ? _extend(element) : element;
        }
        return extend;
      })(extend);
    }

    // In IE8 APPLET, EMBED, and OBJECT elements don't inherit from their prototypes
    if (Bug('ELEMENT_OBJECT_AND_RELATIVES_FAILS_TO_INHERIT_FROM_PROTOTYPE')) {
      extend = (function(_extend) {
        function extend(element) {
          var nodeName = element && getNodeName(element);
          if (BUGGY[nodeName]) {
            return (typeof element._extendedByFuse !== 'undefined' &&
              element._extendedByFuse() >= revision) ?
                element : _extendElement(element, nodeName);
          }
          return _extend(element);
        }
        var BUGGY = { 'APPLET': 1, 'EMBED': 1, 'OBJECT': 1 };
        return extend;
      })(extend);
    }

    extend.refresh = refresh;
    return extend;
  })();

  /*--------------------------------------------------------------------------*/

  Element.addMethods = (function() {
    // add HTMLElement for Safari 2
    if (Feature('OBJECT__PROTO__') && !Feature('HTML_ELEMENT_CLASS')) {
      Feature.set({ 'HTML_ELEMENT_CLASS': true, 'ELEMENT_EXTENSIONS': true });
      _emulateDOMClass('HTMLElement', 'DIV');
    }

    var tagNameClassLookup = {
      'A':        'Anchor',
      'CAPTION':  'TableCaption',
      'COL':      'TableCol',
      'COLGROUP': 'TableCol',
      'DEL':      'Mod',
      'DIR':      'Directory',
      'DL':       'DList',
      'H1':       'Heading',
      'H2':       'Heading',
      'H3':       'Heading',
      'H4':       'Heading',
      'H5':       'Heading',
      'H6':       'Heading',
      'IFRAME':   'IFrame',
      'IMG':      'Image',
      'INS':      'Mod',
      'FIELDSET': 'FieldSet',
      'FRAMESET': 'FrameSet',
      'OL':       'OList',
      'OPTGROUP': 'OptGroup',
      'P':        'Paragraph',
      'Q':        'Quote',
      'TBODY':    'TableSection',
      'TD':       'TableCell',
      'TEXTAREA': 'TextArea',
      'TH':       'TableCell',
      'TFOOT':    'TableSection',
      'THEAD':    'TableSection',
      'TR':       'TableRow',
      'UL':       'UList'
    },

    EMULATE_ELEMENT_CLASSES_WITH_PROTO =
      Feature('EMULATE_ELEMENT_CLASSES_WITH_PROTO'),

    // supports IE8 as well as EOMB
    elementPrototype = Feature('HTML_ELEMENT_CLASS') ?
      global.HTMLElement.prototype : Feature('ELEMENT_CLASS') ?
        global.Element.prototype : false;

    function _copy(methods, destination, onlyIfAbsent) {
      onlyIfAbsent = onlyIfAbsent || false;
      Fuse.Object._each(methods, function(value, key) {
        if (typeof value === 'function' && 
           (!onlyIfAbsent || !(key in destination)))
          destination[key] = Fuse.Function.methodize([key, methods]);
      });
    }

    function _emulateDOMClass(className, tagName) {
      (global[className] = { }).prototype =
        Fuse._doc.createElement(tagName)['__proto__'];
      return global[className];
    }

    function _extend(tagName, methods) {
      tagName = tagName.toUpperCase();
      if (!Element.Methods.ByTag[tagName])
        Element.Methods.ByTag[tagName] = { };
      Fuse.Object.extend(Element.Methods.ByTag[tagName], methods);
    }

    function _findDOMClass(tagName) {
      // catch most classes like HTMLUListElement and HTMLSelectElement
      var className = 'HTML' + (tagNameClassLookup[tagName] ||
        Fuse.String(tagName).capitalize()) + 'Element';
      if (global[className])
        return global[className];
      // catch element classes like HTMLLIElement
      className = 'HTML' + tagName + 'Element';
      if (global[className])
        return global[className];
      // emulate the class (not used by any browser)
      if (EMULATE_ELEMENT_CLASSES_WITH_PROTO)
        return _emulateDOMClass(className, tagName);
    }

    return function(methods) {
      var tagName, T = Element.Methods.ByTag;

      if (arguments.length < 2) {
        Fuse.Object.extend(Form, Form.Methods);
        Fuse.Object.extend(Form.Element, Form.Element.Methods);
        Fuse.Object.extend(Element.Methods.ByTag, {
          'BUTTON':   Fuse.Object.clone(Form.Element.Methods),
          'FORM':     Fuse.Object.clone(Form.Methods),
          'INPUT':    Fuse.Object.clone(Form.Element.Methods),
          'SELECT':   Fuse.Object.clone(Form.Element.Methods),
          'TEXTAREA': Fuse.Object.clone(Form.Element.Methods)
        });
      } else {
        tagName = methods;
        methods = arguments[1];
      }

      if (!tagName || tagName == '')
        Fuse.Object.extend(Element.Methods, methods);
      else {
        Fuse.List.isArray(tagName)
          ? tagName._each(function(name) { _extend(name, methods) })
          : _extend(tagName, methods);
      }

      if (Feature('ELEMENT_EXTENSIONS')) {
        _copy(Element.Methods, elementPrototype);
        _copy(Element.Methods.Simulated, elementPrototype, true);
      }

      if (Feature('ELEMENT_SPECIFIC_EXTENSIONS')) {
        var klass, infiniteRevision = function() { return Infinity };
        for (tagName in Element.Methods.ByTag) {
          klass = _findDOMClass(tagName);
          if (typeof klass === 'undefined') continue;
          _copy(T[tagName], klass.prototype);
        }
        elementPrototype._extendedByFuse = infiniteRevision;
      }

      Fuse.Object.extend(Element, Element.Methods);
      delete Element.ByTag;

      Element.extend.refresh();
      Element.cache = { };
    };
  })();

  /*--------------------------------------------------------------------------*/

  Element._insertionTranslations = {
    'tags': {
      'COLGROUP': ['<table><colgroup>',      '<\/colgroup><tbody><\/tbody><\/table>', 2],
      'SELECT':   ['<select>',               '<\/select>',                            1],
      'TABLE':    ['<table>',                '<\/table>',                             1],
      'TBODY':    ['<table><tbody>',         '<\/tbody><\/table>',                    2],
      'TR':       ['<table><tbody><tr>',     '<\/tr><\/tbody><\/table>',              3],
      'TD':       ['<table><tbody><tr><td>', '<\/td><\/tr><\/tbody><\/table>',        4]
    }
  };

  (function() {
    // TODO: Opera fails to render optgroups when set with innerHTML
    Fuse.Object._extend(this.tags, {
      'OPTGROUP': this.tags.SELECT,
      'TFOOT':    this.tags.TBODY,
      'TH':       this.tags.TD,
      'THEAD':    this.tags.TBODY
    });

    this.before = function before(element, node) {
      element.parentNode &&
        element.parentNode.insertBefore(node, element);
    };

    this.top = function top(element, node) {
      element.insertBefore(node, element.firstChild);
    };

    this.bottom = function bottom(element, node) {
      element.appendChild(node);
    };

    this.after = function after(element, node) {
      element.parentNode &&
        element.parentNode.insertBefore(node, element.nextSibling);
    };

    // prevent JScript bug with named function expressions
    var after = null, before = null,  bottom = null, top = null;
  }).call(Element._insertionTranslations);

  /*--------------------------------------------------------------------------*/

  Element.Methods = {
    'ByTag':     { },
    'Simulated': { }
  };

  (function() {
    this.cleanWhitespace = function cleanWhitespace(element) {
      // removes whitespace-only text node children
      element = $(element);
      var nextNode, node = element.firstChild;
      while (node) {
        nextNode = node.nextSibling;
        if (node.nodeType === 3 && !/\S/.test(node.nodeValue))
          element.removeChild(node);
        node = nextNode;
      }
      return element;
    };

    this.empty = function empty(element) {
      return Fuse.String($(element).innerHTML).blank();
    };

    this.getDimensions = function getDimensions(element) {
      return { 'width': Element.getWidth(element), 'height': Element.getHeight(element) };
    };

    this.getOffsetParent = (function() {
      var END_ON_NODE = { 'BODY': 1, 'HTML': 1 },
       OFFSET_PARENTS = { 'TABLE': 1, 'TD': 1, 'TH': 1 };

      function getOffsetParent(element) {
        // http://www.w3.org/TR/cssom-view/#offset-attributes
        element = $(element);
        var original = element, nodeName = getNodeName(element);
        if (nodeName === 'AREA') return Element.extend(element.parentNode); 

        // IE throws an error if the element is not in the document.
        // Many browsers report offsetParent as null if the element's
        // style is display:none.
        if (Element.isFragment(element) || element.nodeType === 9 || END_ON_NODE[nodeName] ||
           !element.offsetParent && Element.getStyle(element, 'display') != 'none')
          return null;

        while (element = element.parentNode) {
          nodeName = getNodeName(element);
          if (END_ON_NODE[nodeName]) break;
          if (OFFSET_PARENTS[nodeName] ||
              Element.getStyle(element, 'position') != 'static')
            return Element.extend(element);
        }
        return Element.extend(getDocument(original).body);
      };
      return getOffsetParent;
    })();

    this.identify = function identify(element) {
      // use readAttribute to avoid issues with form elements and
      // child controls with ids/names of "id"
      var id = Element.readAttribute(element, 'id');
      if (id.length) return id;

      var ownerDoc = element.ownerDocument;
      do { id = 'anonymous_element_' + Element.idCounter++ }
      while (ownerDoc.getElementById(id));
      Element.writeAttribute(element, 'id', id);
      return Fuse.String(id);
    };

    this.inspect = function inspect(element) {
      element = $(element);
      var attribute, property, value,
       result = '<' + element.nodeName.toLowerCase(),
       translation = { 'id': 'id', 'className': 'class' };

      for (property in translation) {
        attribute = translation[property];
        value = element[property] || '';
        if (value) result += ' ' + attribute + '=' + Fuse.String(value).inspect(true);
      }
      return Fuse.String(result + '>');
    };

    this.isFragment = (function() {
      var isFragment = function isFragment(element) {
        element = $(element);
        var nodeType = element.nodeType;
        return nodeType === 11 || (nodeType === 1 && !(element.parentNode &&
          Element.descendantOf(element, element.ownerDocument)));
      };

      if (Feature('ELEMENT_SOURCE_INDEX', 'DOCUMENT_ALL_COLLECTION')) {
        isFragment = function isFragment(element) {
          element = $(element);
          var nodeType = element.nodeType;
          return nodeType === 11 || (nodeType === 1 &&
            element.ownerDocument.all[element.sourceIndex] !== element);
        };
      }
      if (Feature('ELEMENT_COMPARE_DOCUMENT_POSITION')) {
        isFragment = function isFragment(element) {
          /* DOCUMENT_POSITION_DISCONNECTED = 0x01 */
          element = $(element);
          var nodeType = element.nodeType;
          return nodeType === 11 || (nodeType === 1 &&
            (element.ownerDocument.compareDocumentPosition(element) & 1) === 1);
        };
      }
      return isFragment;
    })();

    this.hide = function hide(element) {
      element = $(element);
      var display = element.style.display;
      if (display && display !== 'none')
        element._originalDisplay = display;
      element.style.display = 'none';
      return element;
    };

    this.show = function show(element) {
      element = $(element);
      var display = element.style.display;
      if (display === 'none')
        element.style.display = element._originalDisplay || '';
      element._originalDisplay = null;
      return element;
    };

    this.scrollTo = function scrollTo(element) {
      var pos = Element.cumulativeOffset(element);
      global.scrollTo(pos[0], pos[1]);
      return $(element);
    };

    this.remove = function remove(element) {
      element = $(element);
      element.parentNode &&
      element.parentNode.removeChild(element);
      return element;
    };

    this.toggle = function toggle(element) {
      return Element[Element.isVisible(element) ?
        'hide' : 'show'](element);
    };

    this.isVisible = (function() {
      function isVisible(element) {
        if (!Fuse._body) return false;

        var isVisible = function isVisible(element) {
          // handles IE and the fallback solution
          element = $(element);
          var style = element.currentStyle;
          return style !== null && (style || element.style).display !== 'none' &&
            !!(element.offsetHeight || element.offsetWidth);
        };

        if (Feature('ELEMENT_COMPUTED_STYLE')) {
          isVisible = function isVisible(element) {
            element = $(element);
            var style = element.ownerDocument.defaultView.getComputedStyle(element, null);
            return !!(style && (element.offsetHeight || element.offsetWidth));
          };
        }
        if (Bug('TABLE_ELEMENTS_RETAIN_OFFSET_DIMENSIONS_WHEN_HIDDEN')) {
          var _isVisible = isVisible;
          isVisible = function isVisible(element) {
            element = $(element);
            if (_isVisible(element)) {
              var nodeName = getNodeName(element);
              if ((nodeName === 'THEAD' || nodeName === 'TBODY' || nodeName === 'TR') &&
                 (element = element.parentNode))
                return Element.isVisible(element);
              return true;
            }
            return false;
          };
        }
        // update API hooks
        Element.isVisible = Element.Methods.isVisible = isVisible;
        return isVisible(element);
      }
      return isVisible;
    })();

    this.wrap = function wrap(element, wrapper, attributes) {
      element = $(element);
      if (Fuse.Object.isElement(wrapper))
        $(wrapper).writeAttribute(attributes);
      else if (Fuse.Object.isString(wrapper))
        wrapper = new Element(wrapper, attributes);
      else wrapper = new Element('div', wrapper);
      if (element.parentNode)
        element.parentNode.replaceChild(wrapper, element);
      wrapper.appendChild(element);
      return wrapper;
    };

    // prevent JScript bug with named function expressions
    var cleanWhitespace = null,
     empty =              null,
     getDimensions =      null,
     getOffsetParent =    null,
     hide =               null,
     identify =           null,
     inspect =            null,
     isFragment =         null,
     isVisible =          null,
     remove =             null,
     scrollTo =           null,
     show =               null,
     toggle =             null,
     wrap =               null;
  }).call(Element.Methods);

  /*--------------------------------------------------------------------------*/

  (function() {
    function _isInsertable(node) {
      return _isInsertable.nodeType[node.nodeType];
    }
    _isInsertable.nodeType = { '1': 1, '3': 1, '8': 1, '10': 1, '11': 1 };

    function _replaceElement(element, node) {
      element.parentNode.replaceChild(node, element);
    }

    this.insert = function insert(element, insertions) {
      element = $(element);
      var content, fragment, insertContent, position, nodeName;

      if (insertions) {
        if (insertions instanceof Fuse.Hash)
          insertions = insertions._object;

        if (Fuse.Object.isString(insertions) ||
            Fuse.Object.isNumber(insertions) || _isInsertable(insertions) ||
            insertions.toElement || insertions.toHTML)
          insertions = { 'bottom': insertions };
      }

      for (position in insertions) {
        content  = insertions[position];
        position = position.toLowerCase();
        insertContent = Element._insertionTranslations[position];

        if (content && content != '') {
          if (content.toElement) content = content.toElement();
          if (_isInsertable(content)) {
            insertContent(element, content);
            continue;
          }
          content = Fuse.Object.toHTML(content);
        }
        else continue;

        nodeName = getNodeName(position === 'before' || position === 'after'
          ? element.parentNode : element);

        fragment = Element._getContentFromAnonymousElement(
          element.ownerDocument, nodeName, content.stripScripts());

        insertContent(element, fragment);
        Fuse.Function.defer(Fuse.Function.bind(content.evalScripts, content));
      }
      return element;
    };

    this.replace = (function() {
      var _createContextualFragment = function(element, content) {
        return Element._getContentFromAnonymousElement(element.ownerDocument,
          getNodeName(element.parentNode), content);
      };

      if (Feature('DOCUMENT_RANGE_CREATE_CONTEXTUAL_FRAGMENT'))
        (function(fn) {
          _createContextualFragment = function(element, content) {
            try {
              // Konqueror throws when trying to create a fragment from
              // incompatible markup such as table rows. Similar to IE's issue
              // with setting table's innerHTML.

              // WebKit and KHTML throw when creating contextual fragments from orphaned elements
              var range = element.ownerDocument.createRange();
              range.selectNode(element);
              return range.createContextualFragment(content);
            } catch (e) {
              return fn(element, content);
            }
          };
        })(_createContextualFragment);

      function replace(element, content) {
        element = $(element);
        if (!content || content == '')
          return element.parentNode.removeChild(element);
        if (content.toElement)
          content = content.toElement();
        else if (!_isInsertable(content)) {
          content = Fuse.Object.toHTML(content);
          Fuse.Function.defer(Fuse.Function.bind(content.evalScripts, content));
          content = _createContextualFragment(element, content.stripScripts());
        }
        _replaceElement(element, content);
        return element;
      }

      return replace;
    })();

    this.update = function update(element, content) {
      element = $(element);
      if (getNodeName(element) === 'SCRIPT') {
        element.text = Fuse.String.interpret(content);
      } else {
        if (content && content != '') {
          if (content.toElement)
            content = content.toElement();
          if (_isInsertable(content)) {
            element.innerHTML = '';
            element.appendChild(content);
            return element;
          }
          content = Fuse.Object.toHTML(content);
          element.innerHTML = content.stripScripts();
          Fuse.Function.defer(Fuse.Function.bind(content.evalScripts, content));
        } else element.innerHTML = '';
      }
      return element;
    };

    // fix browsers with buggy innerHTML implementations
    (function() {
      function update(element, content) {
        element = $(element);
        var nodeName = getNodeName(element), isBuggy = BUGGY[nodeName];
        if (nodeName === 'SCRIPT') {
          element.text = Fuse.String.interpret(content);
        } else {
          // remove children
          if (isBuggy) {
            while (element.lastChild)
              element.removeChild(element.lastChild);
          } else element.innerHTML = '';

          if (content && content != '') {
            if (content.toElement) content = content.toElement();
            if (_isInsertable(content)) element.appendChild(content);
            else {
              content = Fuse.Object.toHTML(content);
              if (isBuggy)
                element.appendChild(Element._getContentFromAnonymousElement(
                  element.ownerDocument, nodeName, content.stripScripts()));
              else element.innerHTML = content.stripScripts();
              Fuse.Function.defer(Fuse.Function.bind(content.evalScripts, content));
            }
          }
        }
        return element;
      };

      var BUGGY = { };
      if (Bug('ELEMENT_COLGROUP_INNERHTML_BUGGY'))
        BUGGY.COLGROUP = 1;
      if (Bug('ELEMENT_OPTGROUP_INNERHTML_BUGGY'))
        BUGGY.OPTGROUP = 1;
      if (Bug('ELEMENT_SELECT_INNERHTML_BUGGY'))
        BUGGY.SELECT   = 1;
      if (Bug('ELEMENT_TABLE_INNERHTML_BUGGY'))
        BUGGY.TABLE = BUGGY.TBODY = BUGGY.TR = BUGGY.TD = 
        BUGGY.TFOOT = BUGGY.TH    = BUGGY.THEAD = 1;

      if (!Fuse.Object.isEmpty(BUGGY))
        this.update = update;
    }).call(this);

    // fix Safari <= 2.0.2 inserting script elements
    (function() {
      function getByTagName(node, tagName) {
        var results = [], child = node.firstChild;
        while (child) {
          if (getNodeName(child) === tagName)
            results.push(child);
          else if (child.getElementsByTagName) {
            // concatList implementation for nodeLists
            var i = 0, pad = results.length, nodes = child.getElementsByTagName(tagName);
            while (results[pad + i] = nodes[i++]) { }
            results.length--;
          }
          child = child.nextSibling;
        }
        return results;
      }

      function wrapper(proceed, element, node) {
        var i = 0, scripts = [];
        if (_isInsertable(node)) {
          if (getNodeName(node) === 'SCRIPT')
            scripts = [node];
          else if (node.getElementsByTagName)
            scripts = node.getElementsByTagName('SCRIPT');
          // Safari 2 fragments don't have GEBTN
          else scripts = getByTagName(node, 'SCRIPT');
        }
        proceed(element, node);
        while (scripts[i]) global.eval(String(scripts[i++].text));
      }

      if (Bug('ELEMENT_SCRIPT_FAILS_TO_EVAL_TEXT_PROPERTY_ON_INSERT')) {
        _replaceElement = Fuse.Function.wrap(_replaceElement, wrapper);

        Fuse.Util.$w('before top bottom after').each(function(method) {
          this[method] = Fuse.Function.wrap(this[method], wrapper);
        }, Element._insertionTranslations);
      }
    })();

    // prevent JScript bug with named function expressions
    var insert = null, update = null;
  }).call(Element.Methods);

  /*--------------------------------------------------------------------------*/

  // define Element#getWidth() and Element#getHeight()
  Fuse.Util.$w('Width Height')._each(function(D) {
    Element.Methods['get' + D] = (function() {
      var property = 'offset' + D;
      return function(element) {
        element = $(element);

        // offsetHidth/offsetWidth properties return 0 on elements
        // with display:none, so show the element temporarily
        var result;
        if (!Element.isVisible(element)) {
          var s = element.style, backup = s.cssText;
          s.cssText += ';display:block;visibility:hidden;';
          result = element[property];
          s.cssText = backup;
        }
        else result = element[property];
        return Fuse.Number(result);
      };
    })();
  });

  /*--------------------------- ELEMENT: ATTRIBUTE ---------------------------*/

  Element.Attribute = {
    'contentNames': { },
    'read':         { },
    'write':        { },
    'names': { 'htmlFor':'for', 'className':'class' }
  };

  (function() {
    var ATTRIBUTE_NODES_PERSIST_ON_CLONED_ELEMENTS =
      Bug('ATTRIBUTE_NODES_PERSIST_ON_CLONED_ELEMENTS');

    this.readAttribute = function readAttribute(element, name) {
      element = $(element);
      var result, T = Element.Attribute;
      name = T.names[name] || name;

      if (T.read[name])
        result = T.read[name](element, name);
      else result = (result = element.getAttributeNode(name)) && result.value;
      return Fuse.String(result || '');
    };

    this.writeAttribute = function writeAttribute(element, name, value) {
      element = $(element);
      var node, contentName, attr, attributes = { }, T = Element.Attribute;

      if (name instanceof Fuse.Hash) attributes = name._object;
      else if (!Fuse.Object.isString(name)) attributes = name;
      else attributes[name] = (typeof value === 'undefined') ? true : value;

      for (attr in attributes) {
        name = T.names[attr] || attr;
        contentName = T.contentNames[attr] || attr;
        value = attributes[attr];

        if (T.write[name])
          T.write[name](element, value);
        else if (value === false || value === null)
          element.removeAttribute(contentName);
        else if (value === true)
          element.setAttribute(contentName, contentName);
        else {
          if (ATTRIBUTE_NODES_PERSIST_ON_CLONED_ELEMENTS &&
              Element.hasAttribute(element, name))
            element.removeAttribute(contentName);
          element.setAttribute(contentName, String(value));
        }
      }
      return element;
    };
  }).call(Element.Methods);

  // No use of $ in this function in order to keep things fast.
  // Used by the Selector class. 
  Element.Methods.Simulated.hasAttribute = (function() {
    function hasAttribute(element, attribute) {
      var node = element.getAttributeNode(attribute);
      return !!node && node.specified;
    }
    return hasAttribute;
  })();

  Element.hasAttribute = (function() {
    function hasAttribute(element, attribute) {
      return element.hasAttribute(attribute);
    }

    if (isHostObject(Fuse._docEl, 'hasAttribute'))
      return hasAttribute;
    return Element.Methods.Simulated.hasAttribute;
  })();

  /*--------------------------------------------------------------------------*/

  (function(T) {
    function getAttribute(element, attribute) {
      return element.getAttribute(attribute);
    }

    function getEvent(element, attribute) {
      var node = element.getAttributeNode(attribute);
      return node && node.specified && node.value;
    }

    function getExact(element, attribute) {
      return element.getAttribute(attribute, 2);
    }

    function getFlag(attribute) {
      var lower = attribute.toLowerCase();
      return function(element) {
        return Element.hasAttribute(element, attribute) ? lower : '';
      };
    }

    function getStyle(element) {
      return element.style.cssText.toLowerCase();
    }

    function setChecked(element, value) {
      element.checked = !!value;
    }

    function setNode(name) {
      return function(element, value) {
        var attr = element.getAttributeNode(name);
        if (!attr) {
          attr = element.ownerDocument.createAttribute(name);
          element.setAttributeNode(attr);
        }
        attr.value = String(value);
      };
    }

    function setStyle(element, value) {
      element.style.cssText = String(value || '');
    }

    // mandate getter / setters
    T.read.type     = getAttribute;
    T.write.checked = setChecked;

    // mandate flag attributes return their name
    Fuse.Util.$w('checked disabled isMap multiple readOnly')._each(function(attr) {
      T.read[attr] = getFlag(attr);
    });

    // mandate event attribute getter
    Fuse.Util.$w('blur change click contextmenu dblclick error focus load keydown ' +
       'keypress keyup mousedown mousemove mouseout mouseover mouseup ' +
       'readystatechange reset submit select unload')._each(function(attr) {
      T.read['on' + attr] = getEvent;
    });

    // add camel-cased attributes to name translations
    Fuse.Util.$w('bgColor codeBase codeType cellPadding cellSpacing colSpan rowSpan ' +
       'vAlign vLink aLink dateTime accessKey tabIndex encType maxLength ' +
       'readOnly longDesc frameBorder isMap useMap noHref noResize noShade ' +
       'noWrap marginWidth marginHeight')._each(function(attr) {
      var lower = attr.toLowerCase();
      T.contentNames[lower] = T.names[lower] = attr;
    });

    // capability checks
    (function() {
      var node, value, form = Fuse._doc.createElement('form'),
       label  = Fuse._doc.createElement('label'),
       button = Fuse._doc.createElement('button');

      label.htmlFor = label.className = 'x';
      label.setAttribute('style', 'display:block');
      form.setAttribute('encType', 'multipart/form-data');
      button.appendChild(Fuse._doc.createTextNode('y'));
      button.setAttribute('value', 'x');

      // translate content name `htmlFor`
      if (label.getAttribute('htmlFor') === 'x')
        T.contentNames['for'] = 'htmlFor';
      else T.contentNames.htmlFor = 'for';

      // translate content name `className`
      if (label.getAttribute('className') === 'x')
        T.contentNames['class'] = 'className';
      else T.contentNames.className = 'class';

      // set `encType`
      if ((node = form.getAttributeNode('encType')) && 
          node.value !== 'multipart/form-data') {
        T.write.encType = setNode('encType');
      }

      // set `value`
      // http://www.w3.org/TR/DOM-Level-2-HTML/html.html#ID-30233917
      value = (node = button.getAttributeNode('value')) && node.value;
      if (value !== 'x') T.write.value = setNode('value');

      // get and set `style` attribute
      value = (node = label.getAttributeNode('style')) && node.value;
      if (typeof value !== 'string' || value.indexOf('display:block') != 0) {
        T.read.style  = getStyle;
        T.write.style = setStyle;
      }

      // get `href` and other uri attributes
      // TODO: Check others attributes like cite, codeBase, lowsrc, and useMap.
      if (Feature('ELEMENT_GET_ATTRIBUTE_IFLAG')) {
        // Exclude `action` attribute because:
        // Opera 9.25 will automatically translate the URI from relative to absolute.
        // In IE this fix has the reverse effect.
        Fuse.Util.$w('data href longDesc src')
          ._each(function(attr) { T.read[attr] = getExact });
      }
    })();
  })(Element.Attribute);
  /*----------------------------- ELEMENT: STYLE -----------------------------*/

  (function() {
    this.classNames = function classNames(element) {
      var results = Fuse.String($(element).className).split(/\s+/);
      return results[0].length ? results : Fuse.List();
    };

    this.hasClassName = function hasClassName(element, className) {
      element = $(element);
      var elementClassName = element.className;
      return (elementClassName.length > 0 && (elementClassName === className || 
        (' ' + elementClassName + ' ').indexOf(' ' + className + ' ') > -1));
    };

    this.addClassName = function addClassName(element, className) {
      element = $(element);
      if (!Element.hasClassName(element, className))
        element.className += (element.className ? ' ' : '') + className;
      return element;
    };

    this.removeClassName = function removeClassName(element, className) {
      element = $(element);
      element.className = Fuse.String(element.className.replace(
        new RegExp('(^|\\s+)' + className + '(\\s+|$)'), ' ')).trim();
      return element;
    };

    this.toggleClassName = function toggleClassName(element, className) {
      return Element[Element.hasClassName(element, className) ?
        'removeClassName' : 'addClassName'](element, className);
    };

    this.getOpacity = (function() {
      var getOpacity = function getOpacity(element) {
        return Fuse.Number(parseFloat($(element).style.opacity));
      };

      if (Feature('ELEMENT_COMPUTED_STYLE')) {
        getOpacity = function getOpacity(element) {
          element = $(element);
          var style = element.ownerDocument.defaultView.getComputedStyle(element, null);
          return Fuse.Number(
            parseFloat(style ? style.opacity : element.style.opacity));
        };
      }
      else if (Feature('ELEMENT_MS_CSS_FILTERS')) {
        getOpacity = function getOpacity(element) {
          element = $(element);
          var s = element.currentStyle || element.style,
            value = s['filter'].match(/alpha\(opacity=(.*)\)/);
          return Fuse.Number(value && value[1] ? parseFloat(value[1]) / 100 : 1.0);
        };
      }
      return getOpacity;
    })();

    this.setOpacity = (function() {
      var setOpacity = function setOpacity(element, value) {
        element = $(element);
        element.style.opacity = (value == 1 || value == '' && Fuse.Object.isString(value)) ? '' :
          (value < 0.00001) ? '0' : value;
        return element;
      };

      if (Fuse.Browser.Agent.WebKit && (userAgent.match(/AppleWebKit\/(\d+)/) || [])[1] < 500) {
        var _setOpacity = setOpacity;
        setOpacity = function setOpacity(element, value) {
          element = _setOpacity(element, value);
          // TODO: Is this really needed or the best approach ?
          if (value == 1) {
            if (getNodeName(element) === 'IMG' && element.width) {
              element.width++; element.width--;
            } else try {
              var n = element.ownerDocument.createTextNode(' ');
              element.removeChild(element.appendChild(n));
            } catch (e) { }
          }
          return element;
        };
      }
      else if (Fuse.Browser.Agent.Gecko && /rv:1\.8\.0/.test(userAgent)) {
        setOpacity = function setOpacity(element, value) {
          element = $(element);
          element.style.opacity = (value == 1) ? 0.999999 : 
            (value == '' && Fuse.Object.isString(value)) ? '' :
              (value < 0.00001) ? 0 : value;
          return element;
        };
      }
      else if (Feature('ELEMENT_MS_CSS_FILTERS')) {
        setOpacity = function setOpacity(element, value) {
          element = $(element);
          if (!Element._hasLayout(element))
            element.style.zoom = 1;

          var style = element.style,
           filter = Element.getStyle(element, 'filter');

          // strip alpha
          filter = filter.replace(/alpha\([^)]*\)/gi, '');

          if (value == 1 || value == '' && Fuse.Object.isString(value)) {
            if (filter) style.filter = filter;
            else style.removeAttribute('filter');
            return element;
          }
          else if (value < 0.00001) value = 0;

          style.filter = filter + 'alpha(opacity=' + (value * 100) + ')';
          return element;   
        };
      }
      return setOpacity;
    })();

    // prevent JScript bug with named function expressions
    var addClassName = null,
     hasClassName =    null,
     removeClassName = null,
     classNames =      null,
     toggleClassName = null,
     getOpacity =      null,
     setOpacity =      null;
  }).call(Element.Methods);

  /*--------------------------------------------------------------------------*/

  (function() {
    var DIMENSION_NAMES = { 'height': 1, 'width': 1 };

    var FLOAT_TRANSLATIONS = typeof Fuse._docEl.style.styleFloat !== 'undefined'
     ? { 'float': 'styleFloat', 'cssFloat': 'styleFloat' }
     : { 'float': 'cssFloat' };

    this.setStyle = function setStyle(element, styles) {
      element = $(element);
      var key, name, elementStyle = element.style;

      if (Fuse.Object.isString(styles)) {
        element.style.cssText += ';' + styles;
        return styles.indexOf('opacity') > -1
          ? Element.setOpacity(element, styles.match(/opacity:\s*(\d?\.?\d*)/)[1])
          : element;
      }
      if (styles instanceof Fuse.Hash)
        styles = styles._object;

      for (key in styles) {
        name = FLOAT_TRANSLATIONS[key] || key;
        if (name === 'opacity') Element.setOpacity(element, styles[key]);
        else elementStyle[name] = styles[key];
      }
      return element;
    };

    this.getStyle =
      Feature('ELEMENT_COMPUTED_STYLE') || !Feature('ELEMENT_CURRENT_STYLE') ?
      (function() {
        function _getComputedStyle(element, name) {
          name = FLOAT_TRANSLATIONS[name] || name;
          var css = element.ownerDocument.defaultView.getComputedStyle(element, null);
          if (css) return _getResult(name, css[name]);
          return _getStyleValue(element, name);
        }

        function _getStyleValue(element, name) {
          name = FLOAT_TRANSLATIONS[name] || name;
          return _getResult(name, element.style[name]);
        }

        function _getResult(name, value) {
          if (name == 'opacity')
            return Fuse.String(value === '1' ? '1.0' : parseFloat(value) || '0');
          return value === 'auto' || value === '' ? null : Fuse.String(value);
        }

        function _isNull(element, name) {
          var length = _isNull.handlers.length;
          while (length--) {
            if (_isNull.handlers[length](element, name))
            return true;
          }
          return false;
        }

        _isNull.handlers = [];

        if (Bug('ELEMENT_COMPUTED_STYLE_DEFAULTS_TO_ZERO')) {
          _isNull.handlers.push((function() {
            var POSITION_NAMES = { 'bottom': 1, 'left': 1, 'right': 1, 'top': 1 };
            return function(element, name) {
              return POSITION_NAMES[name] && 
                _getComputedStyle(element, 'position') === 'static';
            };
          })());
        }
        if (Bug('ELEMENT_COMPUTED_STYLE_HEIGHT_IS_ZERO_WHEN_HIDDEN')) {
          _isNull.handlers.push(function(element, name) {
            return DIMENSION_NAMES[name] && element.style.display === 'none';
          });
        }

        var getStyle;
        if (!Feature('ELEMENT_COMPUTED_STYLE')) {
          getStyle = function getStyle(element, name) {
            return _getStyleValue(element, name);
          };
        }
        else if (Bug('ELEMENT_COMPUTED_STYLE_DIMENSIONS_EQUAL_BORDER_BOX')) {
          // Opera 9.2x
          getStyle = function getStyle(element, name) {
            element = $(element);
            name = Fuse.String(name).camelize();
            if (_isNull(element, name)) return null;
            var value = _getComputedStyle(element, name);

            // returns the border-box dimensions rather than the content-box
            // dimensions, so we subtract padding and borders from the value
            if (DIMENSION_NAMES[name]) {
              var D = name.capitalize(), dim = parseFloat(value) || 0;
              if (dim !== element['offset' + D]) return Fuse.String(dim + 'px');
              return Fuse.String(Element['_getCss' + D](element) + 'px');
            }
            return Fuse.String(value);
          };
        }
        else { // Firefox, Safari, Opera 9.5+
          getStyle = function getStyle(element, name) {
            element = $(element);
            name = Fuse.String(name).camelize();
            return _isNull(element, name) ? null : _getComputedStyle(element, name);
          };
        }
        return getStyle;
      })() :

      // IE
      (function() {
        var RELATIVE_CSS_UNITS = { 'em': 1, 'ex': 1 };

        // We need to insert into element a span with the M character in it.
        // The element.offsetHeight will give us the font size in px units.
        // Inspired by Google Doctype:
        // http://code.google.com/p/doctype/source/browse/trunk/goog/style/style.js#1146
        var span = Fuse._doc.createElement('span');
        span.style.cssText = 'position:absolute;visibility:hidden;height:1em;lineHeight:0;padding:0;margin:0;border:0;';
        span.innerHTML = 'M';

        function getStyle(element, name) {
          var value;
          if (name == 'opacity') {
            value = String(Element.getOpacity(element));
            if (value.indexOf('.') < 0) value += '.0';
            return Fuse.String(value);
          }

          element = $(element);
          name = Fuse.String(name).camelize();
          name = FLOAT_TRANSLATIONS[name] || name;

          // get cascaded style
          var s = element.currentStyle || element.style;
          value = s[name];
          if (value === 'auto') {
            if (DIMENSION_NAMES[name] && s.display !== 'none')
              return Fuse.String(element['offset' + name.capitalize()] + 'px');
            return null;
          }

          // If the unit is something other than a pixel (em, pt, %),
          // set it on something we can grab a pixel value from.
          // Inspired by Dean Edwards' comment
          // http://erik.eae.net/archives/2007/07/27/18.54.15/#comment-102291
          if (/^\d+(\.\d+)?(?!px)[%a-z]+$/i.test(value)) {
            if (name == 'fontSize') {
              var unit = value.match(/\D+$/)[0];
              if (unit === '%') {
                var size = element.appendChild(span).offsetHeight;
                element.removeChild(span);
                return Math.round(size) + 'px';
              } 
              else if (RELATIVE_CSS_UNITS[unit])
                element = element.parentNode;
            }

            // backup values
            var pos = Fuse.String(name == 'height' ? 'top' : 'left'),
             stylePos = element.style[pos], runtimePos = element.runtimeStyle[pos];

            // set runtimeStyle so no visible shift is seen
            element.runtimeStyle[pos] = s[pos];
            element.style[pos] = value;
            value = element.style['pixel' + pos.capitalize()] + 'px';

            // revert changes
            element.style[pos] = stylePos;
            element.runtimeStyle[pos] = runtimePos;
          }
          return Fuse.String(value);
        }
        return getStyle;
      })();

    // prevent JScript bug with named function expressions
    var setStyle = null;
  }).call(Element.Methods);

  /*--------------------------- ELEMENT: TRAVERSAL ---------------------------*/

  Fuse.addNS('Util');

  Fuse.addNS('Dom.Selector');

  (function(Selector) {
    Fuse.Util.$$ = Fuse.query = (function() {
      function query(selectors, context) {
        return Selector.select(selectors, $(context));
      }
      return query;
    })();

    /*------------------------------------------------------------------------*/

    (function() {
      this.childElements = function childElements(element, selectors) {
        if (!(element = $(element).firstChild)) return Fuse.List();
        while (element && element.nodeType !== 1) element = element.nextSibling;
        if (!element) return Fuse.List();

        selectors = selectors && selectors.length && selectors;
        return !selectors || selectors && Selector.match(element, selectors)
          ? prependList(Element.nextSiblings(element, selectors), element, Fuse.List())
          : Element.nextSiblings(element, selectors);
      };

      this.match = function match(element, selectors) {
        return Fuse.Object.isString(selectors)
          ? Selector.match($(element), selectors)
          : selectors.match($(element));
      };

      this.query = function query(element, selectors) {
        return Selector.select(selectors, $(element));
      };

      this.siblings = function siblings(element, selectors) {
        var results = Fuse.List(), original = element = $(element);
        element = element.parentNode && element.parentNode.firstChild;

        if (selectors && selectors.length) {
          var match = Selector.match;
          while (element) {
            if (element !== original && element.nodeType === 1 && match(element, selectors))
              results.push(Element.extend(element));
            element = element.nextSibling;
          }
        } else {
          while (element) {
            if (element !== original && element.nodeType === 1)
              results.push(Element.extend(element));
            element = element.nextSibling;
          }
        }
        return results;
      };

      // prevent JScript bug with named function expressions
      var childElements = null, match = null, query = null, siblings = null;
    }).call(Element.Methods);

    /*------------------------------------------------------------------------*/

    (function() {
      this.descendants = (function() {
        var descendants = function descendants(element, selectors) {
          var node, i = 0, results = Fuse.List(), nodes = $(element).getElementsByTagName('*');
          if (selectors && selectors.length) {
            var match = Selector.match;
            while (node = nodes[i++])
              if (match(element, selectors))
                results.push(Element.extend(element));
          }
          else while (node = nodes[i]) results[i++] = Element.extend(node);
          return results;
        };

        if (Bug('GET_ELEMENTS_BY_TAG_NAME_RETURNS_COMMENT_NODES')) {
          descendants = function descendants(element, selectors) {
            var node, i = 0, results = Fuse.List(), nodes = $(element).getElementsByTagName('*');
            if (selectors && selectors.length) {
              var match = Selector.match;
              while (node = nodes[i++])
                if (node.nodeType === 1 && match(element, selectors))
                  results.push(Element.extend(node));
            } else {
              while (node = nodes[i++])
                if (node.nodeType === 1)
                  results.push(Element.extend(node));
            }
            return results;
          };
        }
        return descendants;
      })();

      this.descendantOf = (function() {
        var descendantOf = function descendantOf(element, ancestor) {
         element = $(element); ancestor = $(ancestor);
          while (element = element.parentNode)
            if (element === ancestor) return true;
          return false;
        };

        if (Feature('ELEMENT_COMPARE_DOCUMENT_POSITION')) {
          descendantOf = function descendantOf(element, ancestor) {
            /* DOCUMENT_POSITION_CONTAINS = 0x08 */
            element = $(element); ancestor = $(ancestor);
            return (element.compareDocumentPosition(ancestor) & 8) === 8;
          };
        }
        else if (Feature('ELEMENT_CONTAINS')) {
          var _descendantOf = descendantOf;
          descendantOf = function descendantOf(element, ancestor) {
            if (ancestor.nodeType !== 1) return _descendantOf(element, ancestor);
            element = $(element); ancestor = $(ancestor);
            return ancestor.contains(element) && ancestor !== element;
          };
        }
        return descendantOf;
      })();

      this.down = (function() {
        function _getNth(nodes, index) {
          var count = 0, i = 0;
          while (node = nodes[i++])
            if (count++ === index) return Element.extend(node);
        }

        function _getNthBySelector(nodes, selectors, index) {
          var count = 0, i = 0, match = Selector.match;
          while (node = nodes[i++])
            if (match(node, selectors) && count++ === index)
              return Element.extend(node);
        }

        if (Bug('GET_ELEMENTS_BY_TAG_NAME_RETURNS_COMMENT_NODES')) {
          _getNth = function(nodes, index) {
            var count = 0, i = 0;
            while (node = nodes[i++])
              if (node.nodeType === 1 && count++ === index)
                return Element.extend(node);
          };

          _getNthBySelector = function(nodes, selectors, index) {
            var count = 0, i = 0, match = Selector.match;
            while (node = nodes[i++])
              if (node.nodeType === 1 && match(node, selectors) && count++ === index)
                return Element.extend(node);
          };
        }

        function down(element, selectors, index) {
          if (arguments.length === 1) return Element.firstDescendant(element);
          if (Fuse.Object.isNumber(selectors)) {
            index = selectors; selectors = null;
          } else index = index || 0;

          var nodes = $(element).getElementsByTagName('*');
          return selectors && selectors.length
            ? _getNthBySelector(nodes, selectors, index)
            : _getNth(nodes, index);
        }
        return down;
      })();

      this.firstDescendant = function firstDescendant(element) {
        element = $(element).firstChild;
        while (element && element.nodeType !== 1) element = element.nextSibling;
        return Element.extend(element);
      };

      // prevent JScript bug with named function expressions
      var firstDescendant = null;
    }).call(Element.Methods);

    /*------------------------------------------------------------------------*/

    (function() {
      function _getNth(element, property, selectors, index) {
        element = $(element);
        var count = 0;

        if (Fuse.Object.isNumber(selectors)) {
          index = selectors; selectors = null;
        } else index = index || 0;

        if (selectors && selectors.length) {
          var match = Selector.match;
          while (element = element[property])
            if (element.nodeType === 1 && match(element, selectors) && count++ === index)
              return Element.extend(element);
        } else {
          while (element = element[property])
            if (element.nodeType === 1 && count++ === index)
              return Element.extend(element);
        }
      }

      this.next = function next(element, selectors, index) {
        return _getNth(element, 'nextSibling', selectors, index);
      };

      this.previous = function previous(element, selectors, index) {
        return _getNth(element, 'previousSibling', selectors, index);
      };

      this.up = function up(element, selectors, index) {
        return arguments.length === 1
          ? Element.extend($(element).parentNode)
          : _getNth(element, 'parentNode', selectors, index);
      };

      // prevent JScript bug with named function expressions
      var next = null, previous = null, up = null;
    }).call(Element.Methods);

    /*------------------------------------------------------------------------*/

    (function() {
      function _collect(element, property, selectors) {
        element = $(element);
        var results = Fuse.List();
        if (selectors && selectors.length) {
          var match = Selector.match;
          while (element = element[property])
            if (element.nodeType === 1 && match(element, selectors))
              results.push(Element.extend(element));
        } else {
          while (element = element[property])
            if (element.nodeType === 1)
              results.push(Element.extend(element));
        }
        return results;
      }

      this.ancestors = function ancestors(element, selectors) {
        return _collect(element, 'parentNode', selectors);
      };

      this.nextSiblings = function nextSiblings(element, selectors) {
        return _collect(element, 'nextSibling', selectors);
      };

      this.previousSiblings = function previousSiblings(element, selectors) {
        return _collect(element, 'previousSibling', selectors);
      };

      // prevent JScript bug with named function expressions
      var ancestors = null, nextSiblings = null, previousSiblings = null;
    }).call(Element.Methods);

  })(Fuse.Dom.Selector);

  /*---------------------------- ELEMENT: POSITION ---------------------------*/

  (function() {
    this.makeAbsolute = function makeAbsolute(element) {
      element = $(element);
      if (Element.getStyle(element, 'position') == 'absolute')
        return element;

      var s = element.style,
       cssWidth  = Element._getCssWidth(element),
       cssHeight = Element._getCssHeight(element),
       offsets   = Element.positionedOffset(element),
       before    = Element.getDimensions(element);

      element._originalLeft       = s.left;
      element._originalTop        = s.top;
      element._originalWidth      = s.width;
      element._originalHeight     = s.height;   
      element._originalMarginTop  = s.marginTop;
      element._originalMarginLeft = s.marginLeft;

      s.position   = 'absolute';
      s.marginTop  = '0';
      s.marginLeft = '0';
      s.top        = offsets.top  + 'px';
      s.left       = offsets.left + 'px';
      s.width      = cssWidth     + 'px';
      s.height     = cssHeight    + 'px';

      var after = Element.getDimensions(element);
      s.width   = Math.max(0, cssWidth  + (before.width  - after.width))  + 'px';
      s.height  = Math.max(0, cssHeight + (before.height - after.height)) + 'px';

      return element;
    },

    this.undoAbsolute = function undoAbsolute(element) {
      element = $(element);
      if (Element.getStyle(element, 'position') == 'relative')
        return element;
      if (typeof element._originalTop === 'undefined')
        throw new Error('Element#makeAbsolute must be called first.');

      var s = element.style;
      s.position   = 'relative';
      s.marginLeft = element._originalMarginLeft;
      s.marginTop  = element._originalMarginTop;
      s.top        = element._originalTop;
      s.left       = element._originalLeft;
      s.width      = element._originalHeight;
      s.height     = element._originalWidth;

      element.removeAttribute('_originalTop');
      if (typeof element._originalTop !== 'undefined')
        delete element._originalTop;
      return element;
    };

    this.makeClipping = function makeClipping(element) {
      element = $(element);
      if (element._overflow) return element;
      element._overflow = Element.getStyle(element, 'overflow') || 'auto';
      if (element._overflow != 'hidden')
        element.style.overflow = 'hidden';
      return element;
    };

    this.undoClipping = function undoClipping(element) {
      element = $(element);
      if (!element._overflow) return element;
      element.style.overflow = element._overflow == 'auto' ? '' : element._overflow;
      element._overflow = null;
      return element;
    };

    this.makePositioned = function makePositioned(element) {
      element = $(element);
      var pos = Element.getStyle(element, 'position');
      if (!pos || pos == 'static') {
        element._madePositioned = true;
        element.style.position  = 'relative';
        // Opera returns the offset relative to the positioning context, when an
        // element is position relative but top and left have not been defined
        element.style.top = element.style.left = '0';
      }
      return element;
    };

    this.undoPositioned = function undoPositioned(element) {
      element = $(element);
      if (element._madePositioned) {
        element._madePositioned = undefined;
        element.style.position  =
         element.style.top      =
         element.style.left     =
         element.style.bottom   =
         element.style.right    = '';   
      }
      return element;
    };

   this.clonePosition = function clonePosition(element, source, options) {
      element = $(element);
      source  = $(source);
      options = Fuse.Object._extend({
        'offsetLeft': 0,
        'offsetTop':  0,
        'setLeft':    true,
        'setTop':     true,
        'setWidth':   true,
        'setHeight':  true
      }, options);

      var elementBackupStyle, sourceBackupStyle,
       s = element.style,
       isElementVisible = Element.isVisible(element),
       isSourceVisible = Element.isVisible(source);

       if (!isSourceVisible) {
         sourceBackupStyle = source.style.cssText;
         source.style.cssText += ';display:block;visibility:hidden;';
       }
       if (!isElementVisible) {
         elementBackupStyle = { 'display': s.display, 'visibility': s.visibility };
         s.cssText += ';display:block;visibility:hidden;';
       }

      // Get element size without border or padding then add
      // the difference between the source and element padding/border
      // to the height and width in an attempt to keep the same dimensions.
      if (options.setHeight)
        s.height = Math.max(0, Element._getCssHeight(source) +
          (Element._getPaddingHeight(source) - Element._getPaddingHeight(element)) +
          (Element._getBorderHeight(source)  - Element._getBorderHeight(element))) + 'px';

      if (options.setWidth)
        s.width = Math.max(0, Element._getCssWidth(source) +
          (Element._getPaddingWidth(source) - Element._getPaddingWidth(element)) +
          (Element._getBorderWidth(source)  - Element._getBorderWidth(element))) + 'px';

      // bail if skipping setLeft and setTop
      if (!options.setLeft && !options.setTop)
        return element;

      // clear element coords before getting
      // the cumulativeOffset because Opera
      // will fumble the calculations if
      // you try to subtract the coords after
      if (options.setLeft) s.left = s.marginLeft = '0';
      if (options.setTop)  s.top  = s.marginTop  = '0';

      var p, position = Element.getStyle(element, 'position');

      var delta = position == 'relative'
        ? Element.cumulativeOffset(element)
        : [0, 0];

      if (position == 'absolute' && Element.descendantOf(element, source)) {
        p = Element.cumulativeOffset(element, source);
        p[0] *= -1; p[1] *= -1;
      } else p = Element.cumulativeOffset(source);

      // set position
      if (options.setLeft) s.left = (p[0] - delta[0] + options.offsetLeft) + 'px';
      if (options.setTop)  s.top  = (p[1] - delta[1] + options.offsetTop)  + 'px';
      
      if (!isSourceVisible)
        source.style.cssText = sourceBackupStyle;
      if (!isElementVisible) {
        s.display = elementBackupStyle.display;
        s.visibility = elementBackupStyle.visibility;
      }

      return element;
    };

    this.cumulativeOffset = (function() {
      function getOffset(element, ancestor) {
        // TODO: overhaul with a thorough solution for finding the correct
        // offsetLeft and offsetTop values
        var offsetParent, position, valueT = 0, valueL = 0,
         BODY_OFFSETS_INHERIT_ITS_MARGINS = Bug('BODY_OFFSETS_INHERIT_ITS_MARGINS'),
         ELEMENT_COORD_OFFSETS_DONT_INHERIT_ANCESTOR_BORDER_WIDTH =
           Bug('ELEMENT_COORD_OFFSETS_DONT_INHERIT_ANCESTOR_BORDER_WIDTH');
        do {
          valueT += element.offsetTop  || 0;
          valueL += element.offsetLeft || 0;

          offsetParent = Element.getOffsetParent(element);
          position     = Element.getStyle(element, 'position');

          if (offsetParent && ELEMENT_COORD_OFFSETS_DONT_INHERIT_ANCESTOR_BORDER_WIDTH) {
            valueT += parseFloat(Element.getStyle(offsetParent, 'borderTopWidth'))  || 0;
            valueL += parseFloat(Element.getStyle(offsetParent, 'borderLeftWidth')) || 0;
          }
          if (position == 'fixed' || offsetParent && (offsetParent === ancestor ||
             (BODY_OFFSETS_INHERIT_ITS_MARGINS && position == 'absolute' && 
              getNodeName(offsetParent) === 'BODY'))) {
            break;
          }
        } while (element = offsetParent);
        return Element._returnOffset(valueL, valueT);
      }

      function cumulativeOffset(element, ancestor) {
        element = Element._ensureLayout(element);
        ancestor = $(ancestor);
        if (!Fuse.Object.isElement(ancestor)) ancestor = null;

        // offsetLeft/offsetTop properties return 0 on elements
        // with display:none, so show the element temporarily
        var result;
        if (!Element.isVisible(element)) {
          var s = element.style, backup = s.cssText;
          s.cssText += ';display:block;visibility:hidden;';
          result = getOffset(element, ancestor);
          s.cssText = backup;
        }
        else result = getOffset(element, ancestor);
        return result;
      }

      if (Feature('ELEMENT_BOUNDING_CLIENT_RECT')) {
        getOffset = (function() {
          var _getOffset = getOffset;
          return function(element, ancestor) {
            if (ancestor) return _getOffset(element, ancestor);
            var valueT = 0, valueL = 0;
            if (!Element.isFragment(element)) {
              var doc = getDocument(element),
               info = Fuse._info,
               rect = element.getBoundingClientRect(),
               root = doc[info.root.property],
               scrollEl = doc[info.scrollEl.property];

              valueT = Math.round(rect.top)  -
                (root.clientTop  || 0) + (scrollEl.scrollTop  || 0);
              valueL = Math.round(rect.left) -
                (root.clientLeft || 0) + (scrollEl.scrollLeft || 0);
            }
            return Element._returnOffset(valueL, valueT);
          };
        })();
      }
      return cumulativeOffset;
    })();

    this.cumulativeScrollOffset = function cumulativeScrollOffset(element, onlyAncestors) {
      element = $(element);
      var nodeName, original = element, valueT = 0, valueL = 0,
       doc = getDocument(element), info = Fuse._info,
       scrollEl = doc[info.scrollEl.property],
       skipEl = info.scrollEl.nodeName === 'HTML'
         ? doc[info.body.property]
         : doc[info.docEl.property];

      do {
        if (element !== skipEl) {
          valueT += element.scrollTop  || 0;
          valueL += element.scrollLeft || 0;

          if (element === scrollEl ||
              Element.getStyle(element, 'position') == 'fixed') {
            break;
          }
        }
        element = element.parentNode;
      } while (element && element.nodeType === 1);

      if (onlyAncestors || ((nodeName = getNodeName(original)) && 
          nodeName === 'TEXTAREA' || nodeName === 'INPUT')) {
        valueT -= original.scrollTop  || 0;
        valueL -= original.scrollLeft || 0;
      }

      return Element._returnOffset(valueL, valueT);
    };

    this.positionedOffset = function positionedOffset(element) {
      element = Element._ensureLayout(element);
      var valueT = 0, valueL = 0;
      do {
        valueT += element.offsetTop  || 0;
        valueL += element.offsetLeft || 0;
        element = Element.getOffsetParent(element);
      } while (element && getNodeName(element) !== 'BODY' &&
          Element.getStyle(element, 'position') == 'static');

      return Element._returnOffset(valueL, valueT);
    },

    this.viewportOffset = (function() {
      var viewportOffset = function viewportOffset(element) {
        element = $(element);
        var scrollOffset = Element.cumulativeScrollOffset(element, /*onlyAncestors*/ true),
         cumulativeOffset = Element.cumulativeOffset(element),
         valueT = cumulativeOffset.top, valueL = cumulativeOffset.left;

        // subtract the the scrollOffset totals from the element offset totals.
        valueT -= scrollOffset.top;
        valueL -= scrollOffset.left;
        return Element._returnOffset(valueL, valueT);
      };

      if (Feature('ELEMENT_BOUNDING_CLIENT_RECT')) {
        viewportOffset = function viewportOffset(element) {
          element = $(element);
          var valueT = 0, valueL = 0;
          if (!Element.isFragment(element)) {
            // IE window's upper-left is at 2,2 (pixels) with respect
            // to the true client when not in quirks mode.
            var doc = getDocument(element),
             rect   = element.getBoundingClientRect(),
             root   = doc[Fuse._info.root.property];

            valueT = Math.round(rect.top)  - (root.clientTop  || 0);
            valueL = Math.round(rect.left) - (root.clientLeft || 0);
          }
          return Element._returnOffset(valueL, valueT);
        };
      }
      return viewportOffset;
    })();

    // prevent JScript bug with named function expressions
    var makeAbsolute =        null,
     clonePosition =          null,
     cumulativeScrollOffset = null,
     makeClipping =           null,
     makePositioned =         null,
     positionedOffset =       null,
     undoAbsolute =           null,
     undoClipping =           null,
     undoPositioned =         null;
  }).call(Element.Methods);

  /*---------------------------- ELEMENT: HELPERS ----------------------------*/

  (function() {
    this._returnOffset = function _returnOffset(left, top) {
      var result  = Fuse.List(Fuse.Number(left), Fuse.Number(top));
      result.left = result[0];
      result.top  = result[1];
      return result;
    };

    this._getCssDimensions = function _getCssDimensions(element) {
      return { 'width': Element._getCssWidth(element), 'height': Element._getCssHeight(element) };
    };

    this._hasLayout = function _hasLayout(element) {
      var currentStyle = element.currentStyle;
      return element.style.zoom && element.style.zoom !== 'normal' ||
        currentStyle && currentStyle.hasLayout;
    };

    this._ensureLayout = function _ensureLayout(element) {
      element = $(element);
      if (Element.getStyle(element, 'position') == 'static' &&
        !Element._hasLayout(element)) element.style.zoom = 1;
      return element;
    };

    this._getContentFromAnonymousElement = (function() {
      function getCache(ownerDoc) {
        if (ownerDoc === Fuse._doc)
          return getCache.cache[0];
        // TODO: This is a perfect example of when Element#getUniqueID could be used
        var id = Event.getEventID(getWindow(ownerDoc).frameElement);
        getCache.cache[id] = getCache.cache[id] || {
          'node':     ownerDoc.createElement('div'),
          'fragment': ownerDoc.createDocumentFragment()
        };
        return getCache.cache[id];
      }
      getCache.cache = { };
      getCache.cache[0] = { 'node': Fuse._div, 'fragment': Fuse._doc.createDocumentFragment() };

      var ELEMENT_TABLE_INNERHTML_INSERTS_TBODY =
        Bug('ELEMENT_TABLE_INNERHTML_INSERTS_TBODY');

      var getContentAsFragment = (function() {
        if (Feature('ELEMENT_REMOVE_NODE')) {
          return function(cache, container) {
            // removeNode: removes the parent but keeps the children
            cache.fragment.appendChild(container).removeNode();
            return cache.fragment;
          };
        }
        if (Feature('DOCUMENT_RANGE')) {
          return function(cache, container) {
            cache.range = cache.range || cache.node.ownerDocument.createRange();
            cache.range.selectNodeContents(container);
            var extracted = cache.range.extractContents();
            extracted && cache.fragment.appendChild(extracted);
            return cache.fragment;
          };
        }
        return function(cache, container) {
          var length = container.childNodes.length;
          while (length--)
            cache.fragment.insertBefore(container.childNodes[length], cache.fragment.firstChild);
          return cache.fragment;
        };
      })();

      function _getContentFromAnonymousElement(ownerDoc, nodeName, html) {
        var cache = getCache(ownerDoc), node = cache.node,
         t = Element._insertionTranslations.tags[nodeName];
        if (t) {
          var times = t[2];
          node.innerHTML= t[0] + html + t[1];
          while (times--) node = node.firstChild;
        } else node.innerHTML = html;

        // skip auto-inserted tbody
        if (ELEMENT_TABLE_INNERHTML_INSERTS_TBODY &&
            nodeName === 'TABLE' && /[^<]*<tr/i.test(html)) {
          node = node.firstChild;
        }
        return getContentAsFragment(cache, node);
      }
      
      return _getContentFromAnonymousElement;
    })();

    // prevent JScript bug with named function expressions
    var _ensureLayout =  null,
     _getCssDimensions = null,
     _hasLayout =        null, 
     _returnOffset =     null;
  }).call(Element);

  // define Element._getCssHeight(), Element._getCssWidth(),
  // Element._getBorderHeight(), Element._getBorderWidth(),
  // Element._getPaddingHeight(), Element._getPaddingWidth()
  (function() {
    function getAsNumber(element, style) {
      return parseFloat(Element.getStyle(element, style)) || 0;
    }

    Fuse.Util.$w('width height')._each(function(d) {
      var D = Fuse.String(d).capitalize(),
       pos = d == 'width' ? ['Left', 'Right'] : ['Top', 'Bottom'];

      Element['_getBorder' + D] = (function() {
        var a = 'border' + pos[0] + 'Width', b = 'border' + pos[1] + 'Width';
        return function(element) {
          return getAsNumber(element, a) + getAsNumber(element, b);
        };
      })();

      Element['_getPadding' + D] = (function() {
        var a = 'padding' + pos[0], b = 'padding' + pos[1];
        return function(element) {
          return getAsNumber(element, a) + getAsNumber(element, b);
        };
      })();

      Element['_getCss' + D] = (function() {
        var a = 'get' + D, b = '_getBorder' + D, c = '_getPadding' + D;
        return function(element) {
          return Math.max(0, Element[a](element) -
            Element[b](element) - Element[c](element));
        };
      })();
    });
  })();

  /*---------------------------------- FORM ----------------------------------*/

  Fuse.addNS('Util');

  Fuse.Util.$F = (function() {
    function $F(element) {
      element = $(element);
      var s = Field.Serializers,
       method = element && element.nodeName.toLowerCase();
      return s[method] ? s[method](element) : null;
    }
    return $F;
  })();

  /*--------------------------------------------------------------------------*/

  global.Form = { };

  (function() {
    this.reset = function reset(form) {
      form = $(form);
      form.reset();
      return form;
    };

    this.serializeElements = function serializeElements(elements, options) {
      if (typeof options !== 'object')
        options = { 'hash': !!options };
      else if (typeof options.hash === 'undefined')
        options.hash = true;

      var element, key, value, type,
       isImageType, isSubmitButton, submitSerialized,
       i = 0, result = Fuse.Object(), submit = options.submit;

      /* http://www.w3.org/TR/html401/interact/forms.html#successful-controls */
      while (element = elements[i++]) {
        element = $(element);
        key     = element.name;
        value   = Field.getValue(element);
        type    = element.type;

        isImageType = type === 'image';
        isSubmitButton = type === 'submit' || isImageType;

        // reduce array value
        if (Fuse.List.isArray(value) && value.length < 2)
          value = value[0];

        if (value == null    || // controls with null/undefined values are unsuccessful
            element.disabled || // disabled elements are unsuccessful
            type === 'file'  || // skip file inputs; 
            type === 'reset' || // reset buttons are unsuccessful
            (isSubmitButton &&  // non-active submit buttons are unsuccessful
            (submit === false || submitSerialized ||
            (submit && !(key === submit || element === submit))))) {
          continue;
        }
        if (isSubmitButton) {
          submitSerialized = true;
          if (isImageType) {
            var prefix = key ? key + '.' : '',
             x = options.x || 0, y = options.y || 0;
            result[prefix + 'x'] = x;
            result[prefix + 'y'] = y;
          }
        }
        if (!key) continue;

        // property exists and and belongs to result
        if (Fuse.Object.hasKey(result, key)) {
          // a key is already present; construct an array of values
          if (!Fuse.List.isArray(result[key])) result[key] = [result[key]];
          result[key].push(value);
        }
        else result[key] = value;
      }

      return options.hash
        ? result
        : Fuse.Object.toQueryString(result);
    };

    // prevent JScript bug with named function expressions
    var reset = null, serializeElements = null;
  }).call(Form);

  /*--------------------------------------------------------------------------*/

  Form.Methods = { };

  (function() {
    this.disable = function disable(form) {
      form = $(form);
      var node, nodes = Form.getElements(form), i = 0;
      while (node = nodes[i++]) Field.disable(node);
      return form;
    };

    this.enable = function enable(form) {
      form = $(form);
      var node, nodes = Form.getElements(form), i = 0;
      while (node = nodes[i++]) Field.enable(node);
      return form;
    };

    this.findFirstElement = function findFirstElement(form) {
      var firstByIndex, firstNode, node,
       nodes = $(form).getElements(), minTabIndex = Infinity, i = 0;

      while (node = nodes[i++]) {
        if (node.type !== 'hidden' && !node.disabled && !firstNode)
          firstNode = node;
        if (Element.hasAttribute(node, 'tabIndex') && node.tabIndex > -1 &&
            node.tabIndex < minTabIndex) {
          minTabIndex = node.tabIndex;
          firstByIndex = node;
        }
      }
      return firstByIndex || firstNode;
    };

    this.focusFirstElement = function focusFirstElement(form) {
      form = $(form); 
      var element = Form.findFirstElement(form); 
      element && Field.focus(element); 
      return form;
    };

    this.getElements = function getElements(form) {
      form = $(form);
      var node, results = Fuse.List(), i = 0,
       nodes = $(form).getElementsByTagName('*');
      while (node = nodes[i++])
        if (node.nodeType === 1 &&
            Field.Serializers[node.nodeName.toLowerCase()])
          results.push(Element.extend(node));
      return results;
    };

    this.getInputs = function getInputs(form, typeName, name) {
      form = $(form);
      typeName = String(typeName || '');
      name = String(typeName || '');

      var input, inputs = form.getElementsByTagName('input'),
       results = Fuse.List(), i = 0;

      if (!typeName && !name) {
        while (input = inputs[i]) results[i++] = Element.extend(input);
      }
      else if (typeName && !name) {
        while (input = inputs[i++])
          if (typeName === input.type) results.push(Element.extend(input));
      }
      else {
        while (input = inputs[i++])
          if ((!typeName || typeName === input.type) && (!name || name === input.name))
            results.push(Element.extend(input));
      }
      return results;
    };

    this.request = function request(form, options) {
      form = $(form), options = Fuse.Object.clone(options);

      var params = options.parameters, submit = options.submit,
       action = Element.readAttribute(form, 'action') || Fuse.String('');
      delete options.submit;

      if (action.blank()) action = global.location.href;
      options.parameters = Form.serialize(form, { 'submit':submit, 'hash':true });

      if (params) {
        if (Fuse.Object.isString(params)) params = Fuse.String(params).toQueryParams();
        Fuse.Object.extend(options.parameters, params);
      }

      if (Element.hasAttribute(form, 'method') && !options.method)
        options.method = form.method;

      return new Fuse.Ajax.Request(action, options);
    };

    this.serialize = function serialize(form, options) {
      return Form.serializeElements(Form.getElements(form), options);
    };

    // prevent JScript bug with named function expressions
    var disable =        null,
     enable =            null,
     findFirstElement =  null,
     focusFirstElement = null,
     getElements =       null,
     getInputs =         null,
     request =           null,
     serialize =         null;
  }).call(Form.Methods);

  /*--------------------------------------------------------------------------*/

  global.Field = Form.Element = { };

  (function() {
    this.focus = function focus(element) {
      element = $(element);
      // avoid IE errors when element
      // or ancestors are not visible
      try { element.focus() } catch(e) { }
      return element;
    };

    this.select = function select(element) {
      (element = $(element)).select();
      return element;
    };

    // prevent JScript bug with named function expressions
    var focus = null, select = null;
  }).call(Field);

  /*--------------------------------------------------------------------------*/

  Field.Methods = { };

  (function() {
    this.disable = function disable(element) {
      element = $(element);
      element.disabled = true;
      return element;
    };

    this.enable = function enable(element) {
      element = $(element);
      element.disabled = false;
      return element;
    };

    this.present = function present(element) {
      return !!$(element).value;
    };

    this.serialize = function serialize(element) {
      element = $(element);
      if (!element.disabled && element.name) {
        var value = Field.getValue(element);
        if (Fuse.List.isArray(value) && value.length < 2)
          value = value[0];
        if (value != null) {
          var pair = { };
          pair[element.name] = value;
          return Fuse.Object.toQueryString(pair);
        }
      }
      return '';
    };

    this.getValue = function getValue(element) {
      element = $(element);
      var method = element.nodeName.toLowerCase();
      return Field.Serializers[method](element);
    };

    this.setValue = function setValue(element, value) {
      element = $(element);
      var method = element.nodeName.toLowerCase();
      Field.Serializers[method](element, value || null);
      return element;
    };

    // prevent JScript bug with named function expressions
    var disable = null,
     enable =     null,
     getValue =   null,
     present =    null,
     serialize =  null,
     setValue =   null;
  }).call(Field.Methods);

  (function() {
    var matchInputButtons = /^(button|image|reset|submit)$/;

    this.activate = function activate(element) {
      element = $(element);
      try { element.focus() } catch(e) { }
      if (element.select && getNodeName(element) !== 'BUTTON' &&
          !matchInputButtons.test(element.type))
        element.select();
      return element;
    };

    this.clear = function clear(element) {
      element = $(element);
      if (getNodeName(element) !== 'BUTTON' &&
          !matchInputButtons.test(element.type))
        Field.setValue(element, null);
      return element;
    };

    // prevent JScript bug with named function expressions
    var activate = null, clear = null;
  }).call(Field.Methods);

  /*--------------------------------------------------------------------------*/

  Field.Serializers = { };

  (function() {
    this.button = function button(element, value){
      if (typeof value === 'undefined')
        return Element.readAttribute(element, 'value');
      else Element.writeAttribute(element, 'value', value);
    };

    this.input = function input(element, value) {
      var type = element.type.toLowerCase(),
       method = (type === 'checkbox' || type === 'radio')
        ? 'inputSelector' : 'textarea';
      return Field.Serializers[method](element, value);
    };

    this.inputSelector = function inputSelector(element, value) {
      if (typeof value === 'undefined')
        return element.checked ? element.value : null;
      else element.checked = !!value;
    };

    this.optionValue = function optionValue(element) {
      return element[Element.hasAttribute(element, 'value') ? 'value' : 'text'];
    };

    this.select = function select(element, value) {
      if (typeof value === 'undefined')
        return this[element.type === 'select-one' ? 
          'selectOne' : 'selectMany'](element);

      if (value === null)
        return element.selectedIndex = -1;

      var node, i = 0;
      if (Fuse.List.isArray(value)) {
        value = expando + value.join(expando) + expando;
        while (node = element.options[i++])
          node.selected = value.indexOf(expando + this.optionValue(node) +expando) > -1;
      } else {
        while (node = element.options[i++])
          if (this.optionValue(node) === value)
            return (node.selected = true);
      }
    };

    this.selectOne = function selectOne(element) {
      var index = element.selectedIndex;
      return index > -1 ? this.optionValue(element.options[index]) : null;
    };

    this.selectMany = function selectMany(element) {
      var node, results = Fuse.List(), i = 0;
      if (!element.options.length) return null;

      while (node = element.options[i++])
        if (node.selected) results.push(this.optionValue(node));
      return results;
    };

    this.textarea = function textarea(element, value) {
      if (typeof value === 'undefined') return element.value;
      else element.value = value || '';
    };

    // prevent JScript bug with named function expressions
    var button =     null,
     input =         null,
     inputSelector = null,
     optionValue =   null,
     select =        null,
     selectOne =     null,
     selectMany =    null,
     textarea =      null;
  }).call(Field.Serializers);

  /*-------------------------- FORM: TIMED OBSERVER --------------------------*/

  (function() {
    var BaseTimedObserver = Fuse.Class(Fuse.Timer, {
      'constructor': (function() {
        function BaseTimedObserver(element, callback, interval, options) {
          // this._super() equivalent
          Fuse.Timer.call(this, callback, interval, options);

          this.element = $(element);
          this.lastValue = this.getValue();
          this.start();
        }
        return BaseTimedObserver;
      })(),

      'execute': (function() {
        function execute() {
          var value = this.getValue();
          if (String(this.lastValue) != String(value)) {
            this.callback(this.element, value);
            this.lastValue = value;
          }
        }
        return execute;
      })()
    });

    /*------------------------------------------------------------------------*/

    Field.Observer = 
    Field.TimedObserver = Fuse.Class(BaseTimedObserver, {
      'constructor': (function() {
        function FieldTimedObserver(element, callback, interval, options) {
          if (!(this instanceof FieldTimedObserver))
            return new FieldTimedObserver(element, callback, interval, options);
          BaseTimedObserver.call(this, element, callback, interval, options);
        }
        return FieldTimedObserver;
      })(),

      'getValue': (function() {
        function getValue() { return Form.Element.getValue(this.element) }
        return getValue;
      })()
    });

    Form.Observer = 
    Form.TimedObserver = Fuse.Class(BaseTimedObserver, {
      'constructor': (function() {
        function FormTimedObserver(element, callback, interval, options) {
          if (!(this instanceof FormTimedObserver))
            return new FormTimedObserver(element, callback, interval, options);
          BaseTimedObserver.call(this, element, callback, interval, options);
        }
        return FormTimedObserver;
      })(),

      'getValue': (function() {
        function getValue() { return Form.serialize(this.element) }
        return getValue;
      })()
    });
  })();
  /*-------------------------- FORM: EVENT OBSERVER --------------------------*/

  (function() {
    var BaseEventObserver = Fuse.Class({
      'constructor': (function() {
        function BaseEventObserver(element, callback) {
          this.element = $(element);
          this.onElementEvent = Fuse.Function.bind(this.onElementEvent, this);

          if (getNodeName(this.element) === 'FORM')
            return this.registerFormCallbacks();

          var member, name = element.name, i = 0;
          this.group = (name && Selector.select(element.nodeName +
            '[name="' + name + '"]', getDocument(this.element))) || [element];

          this.callback = callback;
          this.lastValue = this.getValue();

          while (member = this.group[i++])
            this.registerCallback(member);
        }
        var Selector = Fuse.Dom.Selector;
        return BaseEventObserver;
      })()
    });

    (function() {
      this.onElementEvent = function onElementEvent() {
        var value = this.getValue();
        if (this.lastValue === value) return;
        this.callback(this.element, value);
        this.lastValue = value;
      };

      this.registerCallback = function registerCallback(element) {
        if (!element.type) return;
        var eventName = 'change', type = element.type;
        if (type === 'checkbox' || type === 'radio')
          eventName = 'click';
        Event.observe(element, eventName, this.onElementEvent);
      };

      this.registerFormCallbacks = function registerFormCallbacks() {
        var element, elements = Form.getElements(this.element), i= 0;
        while (element = elements[i++]) this.registerCallback(element);
      };

      // prevent JScript bug with named function expressions
      var onElementEvent = null, registerCallback = null, registerFormCallbacks = null;
    }).call(BaseEventObserver.Plugin);

  /*--------------------------------------------------------------------------*/

    Field.EventObserver = Fuse.Class(BaseEventObserver, {
      'constructor': (function() {
        function FieldEventObserver(element, callback) {
          if (!(this instanceof FieldEventObserver))
            return new FieldEventObserver(element, callback);
          BaseEventObserver.call(this, element, callback);
        }
        return FieldEventObserver;
      })(),

      'getValue': (function() {
        function getValue() {
          if (this.group.length === 1)
            return Form.Element.getValue(this.element);
          var member, value, i = 0;
          while (member = this.group[i++])
            if (value = Form.Element.getValue(member))
              return value;
        }
        return getValue;
      })()
    });

    Form.EventObserver = Fuse.Class(BaseEventObserver, {
      'constructor': (function() {
        function FormEventObserver(element, callback) {
          if (!(this instanceof FormEventObserver))
            return new FormEventObserver(element, callback);
          BaseEventObserver.call(this, element, callback);
        }
        return FormEventObserver;
      })(),

      'getValue': (function() {
        function getValue() { return Form.serialize(this.element) }
        return getValue;
      })()
    });
  })();

  /*---------------------------------- NODE ----------------------------------*/

  if (!global.Node) global.Node = { };

  if (typeof Node.ELEMENT_NODE !== 'number') {
    // DOM level 2 ECMAScript Language Binding
    Fuse.Object._extend(Node, {
      'ELEMENT_NODE':                1,
      'ATTRIBUTE_NODE':              2,
      'TEXT_NODE':                   3,
      'CDATA_SECTION_NODE':          4,
      'ENTITY_REFERENCE_NODE':       5,
      'ENTITY_NODE':                 6,
      'PROCESSING_INSTRUCTION_NODE': 7,
      'COMMENT_NODE':                8,
      'DOCUMENT_NODE':               9,
      'DOCUMENT_TYPE_NODE':          10,
      'DOCUMENT_FRAGMENT_NODE':      11,
      'NOTATION_NODE':               12
    });
  }

  /*--------------------------- DOCUMENT VIEWPORT ----------------------------*/

  Fuse._doc.viewport = { };

  (function() {
    this.getDimensions = function getDimensions() {
      return { 'width': this.getWidth(), 'height': this.getHeight() };
    };

    this.getScrollOffsets = function getScrollOffsets() {
      // lazy define
      return (this.getScrollOffsets =
        typeof global.pageXOffset === 'number' ?
          function() {
            return Element._returnOffset(global.pageXOffset, global.pageYOffset);
          } :
          function() {
            return Element._returnOffset(Fuse._scrollEl.scrollLeft, Fuse._scrollEl.scrollTop);
          }
      )();
    };

    // prevent JScript bug with named function expressions
    var getDimensions = null, getScrollOffsets = null;
  }).call(Fuse._doc.viewport);

  // lazy define document.viewport.getWidth() and document.viewport.getHeight()
  (function(v) {
    var node;
    function define(D) {
      // Safari < 3 -> doc
      // Opera  < 9.5, Quirks mode -> body
      // Others -> docEl
      node = node || ('clientWidth' in Fuse._doc ? Fuse._doc : Fuse._root);
      v['get' + D] = function() { return Fuse.Number(node['client' + D]) };
      return v['get' + D]();
    }
    v.getHeight = Fuse.Function.curry(define, 'Height');
    v.getWidth  = Fuse.Function.curry(define, 'Width');
  })(Fuse._doc.viewport);

  /*--------------------------- SELECTOR: NWMATCHER --------------------------*/

  (function() {
    this.match = function match(element, selector) {
      function match(element, selector) {
        return NWMatcher.match(element, String(selector || ''));
      }
      var NWMatcher = NW.Dom;
      return (this.match = match)(element, selector);
    };

    this.select = function select(selector, context) {
      var select = function select(selector, context) {
        return NWMatcher.select(String(selector || ''), context || Fuse._doc, Fuse.List())
          .map(Element.extend);
      };

      if (Feature('ELEMENT_EXTENSIONS'))
        select = function select(selector, context) {
          return NWMatcher.select(String(selector || ''), context || Fuse._doc, Fuse.List());
        };

      var NWMatcher = NW.Dom;
      return (this.select = select)(selector, context);
    };

    // prevent JScript bug with named function expressions
    var match = null, select = null;
  }).call(Fuse.Dom.Selector);

  /*---------------------------------- EVENT ---------------------------------*/

  if (!global.Event) global.Event = { };

  Event.cache = { };

  Event.CUSTOM_EVENT_NAME =
    Feature('ELEMENT_ADD_EVENT_LISTENER') ? 'dataavailable' :
    Feature('ELEMENT_ATTACH_EVENT') ? 'beforeupdate' : 'keyup';

  // temporarily hold these helpers to avoid creating several closures.
  Fuse.Object._extend(Event.Temp = { }, (function() {

    function addCache(element, eventName, handler) {
      var id = getCacheID(element),
       ec = getOrCreateCache(id, element, eventName);

      // bail if handler is already exists
      if (Fuse.List.Plugin.indexOf.call(ec.handlers, handler) != -1)
        return false;

      ec.handlers.unshift(handler);
      if (ec.dispatcher) return false;
      return (ec.dispatcher = createDispatcher(id, eventName));
    }

    function getCacheID(element) {
      var win = getWindow(element);
      // keep a loose match because frame object !== document.parentWindow
      if (element == win) {
        if (element == global) return 1;
        return getCacheID(win.frameElement) + '-1';
      }
      else if (element.nodeType === 9) {
        if (element === Fuse._doc) return 2;
        return getCacheID(win.frameElement) + '-2';
      }
      return element.getEventID();
    }
    getCacheID.id = 3;

    function getDOMEventName(eventName) {
      if (eventName && eventName.indexOf(':') != -1)
        return Event.CUSTOM_EVENT_NAME;
      return eventName;
    }

    function getNewCacheID(element) {
      var id = getCacheID.id++;
      element._prototypeEventID = [id]; // backwards compatibility
      return id;
    }

    function getOrCreateCache(id, element, eventName) {
      var c = Event.cache[id] = Event.cache[id] || { 'events': { } };
      c.element = c.element || element;
      return (c.events[eventName] = c.events[eventName] ||
        { 'handlers': [], 'dispatcher': false });
    }

    function removeCacheAtIndex(id, eventName, index) {
      // remove responders and handlers at the given index
      var c = Event.cache[id], ec = c.events[eventName];
      ec.handlers.splice(index, 1);

      // if no more handlers/responders then
      // remove the eventName cache
      if (!ec.handlers.length) delete c.events[eventName];

      // if no more events cached remove the
      // cache for the element
      if (!Fuse.Object.isEmpty(c.events)) return;
      delete Event.cache[id];
    }

    var addObserver =
      // DOM Level 2
      Feature('ELEMENT_ADD_EVENT_LISTENER') ?
        function(element, eventName, handler) {
          element.addEventListener(getDOMEventName(eventName), handler, false);
        } :
      Feature('ELEMENT_ATTACH_EVENT') ?
        function(element, eventName, handler) {
          element.attachEvent('on' + getDOMEventName(eventName), handler);
        } :
      // DOM Level 0
      function(element, eventName, handler) {
        var attrName = 'on' + getDOMEventName(eventName),
         oldHandler = element[attrName];
        if (oldHandler) {
          if (oldHandler.isDispatcher) return false;
          addCache(element, eventName, element[attrName]);
        }
        element[attrName] = Event.cache[getCacheID(element)]
          .events[eventName].dispatcher;
      },

    removeObserver =
      // DOM Level 2
      Feature('ELEMENT_ADD_EVENT_LISTENER') ?
        function(element, eventName, handler) {
          element.removeEventListener(getDOMEventName(eventName), handler, false);
        } :
      Feature('ELEMENT_ATTACH_EVENT') ?
        function(element, eventName, handler) {
          element.detachEvent('on' + getDOMEventName(eventName), handler);
        } :
      // DOM Level 0
      function(element, eventName, handler) {
        var attrName = 'on' + getDOMEventName(eventName);
        if (!eventName.indexOf(':') != -1 && element[attrName] === handler)
          element[attrName] = null;
      },

    // Event dispatchers manage several handlers and ensure
    // FIFO execution order. They are attached as the event
    // listener and execute all the handlers they manage.
    createDispatcher =
      // DOM Level 2
      Feature('ELEMENT_ADD_EVENT_LISTENER') || Feature('ELEMENT_ATTACH_EVENT') ?
        function(id, eventName) {
          return function(event) {
            // Prevent a Firefox bug from throwing errors on page
            // load/unload (#5393, #9421). When firing a custom event all the
            // CUSTOM_EVENT_NAME observers for that element will fire. Before
            // executing, make sure the event.eventName matches the eventName.
            if (!Event || !Event.extend || (event.eventName &&
                event.eventName !== eventName)) return false;
            // shallow copy handlers to avoid issues with nested
            // observe/stopObserving
            var c = Event.cache[id], ec = c.events[eventName],
             handlers = slice.call(ec.handlers, 0), length = handlers.length;
            event = Event.extend(event || getWindow(c.element).event, c.element);
            while (length--) handlers[length].call(c.element, event);
          };
        } :
        // DOM Level 0
        function(id, eventName) {
          var dispatcher = function(event) {
            if (!Event || !Event.extend) return false;
            event = Event.extend(event || getWindow(this).event, this);
            var c = Event.cache[id], ec = c && c.events &&
              c.events[event.eventName || eventName];
            if (!ec) return false;
            var handlers = slice.call(ec.handlers, 0), length = handlers.length;
            while (length--) handlers[length].call(this, event);
          };
          dispatcher.isDispatcher = true;
          return dispatcher;
        },

    createEvent =
      Feature('DOCUMENT_CREATE_EVENT') ?
        function(context, eventType) {
          var event = getDocument(context).createEvent('HTMLEvents');
          eventType && event.initEvent(eventType, true, true);
          return event;
        } :
      Feature('DOCUMENT_CREATE_EVENT_OBJECT') ?
        function(context, eventType) {
          var event = getDocument(context).createEventObject();
          eventType && (event.eventType = 'on' + eventType);
          return event;
        } :
      function() { return false },

    fireEvent =
      Feature('ELEMENT_DISPATCH_EVENT') ?
        function(element, event) {
          // In the W3C system, all calls to document.fire should treat
          // document.documentElement as the target
          if (element.nodeType === 9)
            element = element.documentElement;
          element.dispatchEvent(event);
        } :
      Feature('ELEMENT_FIRE_EVENT') ?
        function(element, event) {
          element.fireEvent(event.eventType, event);
        } :
      function() { return false };

    return {
      'addCache':           addCache,
      'addObserver':        addObserver,
      'createDispatcher':   createDispatcher,
      'createEvent':        createEvent,
      'fireEvent':          fireEvent,
      'getCacheID':         getCacheID,
      'getDOMEventName':    getDOMEventName,
      'getNewCacheID':      getNewCacheID,
      'getOrCreateCache':   getOrCreateCache,
      'removeCacheAtIndex': removeCacheAtIndex,
      'removeObserver':     removeObserver

    };
  })());

  /*--------------------------------------------------------------------------*/

  Event.Methods = { };

  (function() {
    this.element = function element(event) {
      event = Event.extend(event);
      var node = event.target, type = event.type,
       currentTarget = event.currentTarget;

      // Firefox screws up the "click" event when moving between radio buttons
      // via arrow keys. It also screws up the "load" and "error" events on images,
      // reporting the document as the target instead of the original image.

      // Note: Fired events don't have a currentTarget
      if (currentTarget && (/^(load|error)$/.test(type) ||
         (getNodeName(currentTarget) === 'INPUT' &&
          currentTarget.type === 'radio' && type === 'click'))) {
        node = currentTarget;
      }
      // Fix a Safari bug where a text node gets passed as the target of an
      // anchor click rather than the anchor itself.
      return node && Element.extend(node.nodeType === 3 ?
        node.parentNode : node);
    };

    this.findElement = function findElement(event, selector) {
      var element = Event.element(event);
      if (!selector || selector == null) return element;
      return Fuse.Dom.Selector.match(element, selector) ? element :
        Element.up(element, selector);
    };

    this.pointer = function pointer(event) {
      return { 'x': Event.pointerX(event), 'y': Event.pointerY(event) };
    };

    this.stop = function stop(event) {
      // Set a "stopped" property so that a custom event can be inspected
      // after the fact to determine whether or not it was stopped.
      event = Event.extend(event);
      event.stopped = true;
      event.preventDefault();
      event.stopPropagation();
    };

    // prevent JScript bug with named function expressions
    var element = null, findElement = null, pointer = null, stop = null;
  }).call(Event.Methods);

  (function() {
    var _isButton = function(event, mouseButton) {
      var property = (typeof event.which === 'number')
       ? 'which' : (typeof event.button === 'number')
         ? 'button' : false;

      var buttonMap = (property === 'button')
        ? { 'left': 1, 'middle': 4, 'right': 2 }
        : { 'left': 1, 'middle': 2, 'right': 3 };

      return (_isButton = (property === false)
        ? function() { return false }
        : function(event, mouseButton) {
            return event[property] === buttonMap[mouseButton];
          }
      )(event, mouseButton);
    };

    this.isLeftClick = function isLeftClick(event) {
      return _isButton(event, 'left');
    };

    this.isMiddleClick = function isMiddleClick(event) {
      return _isButton(event, 'middle');
    };

    this.isRightClick = function isRightClick(event) {
      return _isButton(event, 'right');
    };

    // prevent JScript bug with named function expressions
    var isLeftClick = null, isMiddleClick = null, isRightClick = null;
  }).call(Event.Methods);

  // lazy define Event.pointerX() and Event.pointerY()
  (function(m) {
    function define(methodName, event) {
      if (!Fuse._body) return 0;
      if (typeof event.pageX === 'number') {
        Event.pointerX = m.pointerX = function(event) { return event.pageX };
        Event.pointerY = m.pointerY = function(event) { return event.pageY };
      }
      else {
        Event.pointerX = m.pointerX = function(event) {
          var info = Fuse._info, doc = getDocument(event.srcElement || global),
           result = event.clientX + doc[info.scrollEl.property].scrollLeft -
             doc[info.root.property].clientLeft;
          return result > -1 ? result : 0;
        };

        Event.pointerY = m.pointerY = function(event) {
          var info = Fuse._info, doc = getDocument(event.srcElement || global),
           result = event.clientY + doc[info.scrollEl.property].scrollTop -
             doc[info.root.property].clientTop;
           return result > -1 ? result : 0;
        };
      }
      return m[methodName](event);
    }
    m.pointerX = Fuse.Function.curry(define, 'pointerX');
    m.pointerY = Fuse.Function.curry(define, 'pointerY');
  })(Event.Methods);

  /*--------------------------------------------------------------------------*/

  (function() {
    var Methods;

    function _addLevel2Methods(event) {
      event.inspect         = inspect;
      event.preventDefault  = preventDefault;
      event.stopPropagation = stopPropagation;

      // avoid memory leak
      event.pointer  = _createPointerMethod();
      event.pointerX = _createPointerMethod('x');
      event.pointerY = _createPointerMethod('y');

      var length = Methods.length;
      while (length--) {
        pair = Methods[length];
        if (!(pair[0] in event))
          event[pair[0]] = pair[1];
      }
      return event;
    }

    function _addLevel2Properties(event, element) {
      event.pageX = Event.pointerX(event);
      event.pageY = Event.pointerY(event);

      event._extendedByFuse = Fuse.emptyFunction;
      event.currentTarget   = element;
      event.target          = event.srcElement || element;
      event.relatedTarget   = _relatedTarget(event);
      return event;
    }

    function _createPointerMethod(xOrY) {
      switch (xOrY) {
        case 'x': return function() { return this.pageX };
        case 'y': return function() { return this.pageY };
        default : return function() { return { 'x': this.pageX, 'y': this.pageY } };
      }
    }

    function _relatedTarget(event) {
      switch (event.type) {
        case 'mouseover': return Element.extend(event.fromElement);
        case 'mouseout':  return Element.extend(event.toElement);
        default:          return null;
      }
    }

    function addMethods(methods) {
      var name; Methods = [];
      methods && Fuse.Object.extend(Event.Methods, methods);
      Fuse.Object._each(Event.Methods, Event.prototype
        ? function(value, key, object) {
            Event.prototype[key] = Fuse.Function.methodize([key, object]);
          }
        : function(value, key, object) {
            if (key.indexOf('pointer') != 0)
              Methods.push([key, Fuse.Function.methodize([key, object])]);
          }
      );
    }

    function extend(event, element) {
      return (event && !event._extendedByFuse)
        ? _addLevel2Properties(_addLevel2Methods(event), element)
        : event;
    }

    function inspect() {
      return '[object Event]';
    }

    function preventDefault() {
      this.returnValue = false;
    }

    function stopPropagation() {
      this.cancelBubble = true;
    }

    Event.addMethods = addMethods;

    if (Event.prototype || Feature('OBJECT__PROTO__')) {
      // Safari 2 support
      if (!Event.prototype)
        Event.prototype = Event.Temp.createEvent(Fuse._doc)['__proto__'];

      // IE8 supports Event.prototype but still needs
      // DOM Level 2 event methods and properties.
      if (Fuse.Object.hasKey(Event.prototype, 'cancelBubble') &&
          Fuse.Object.hasKey(Event.prototype, 'returnValue') &&
         !Fuse.Object.hasKey(Event.prototype, 'stopPropagation') &&
         !Fuse.Object.hasKey(Event.prototype, 'preventDefault') &&
         !Fuse.Object.hasKey(Event.prototype, 'target') &&
         !Fuse.Object.hasKey(Event.prototype, 'currentTarget')) {

        // initially add methods
        Event.addMethods();
        _addLevel2Methods(Event.prototype);

        Event.extend = function(event, element) {
          return (event && !event._extendedByFuse)
            ? _addLevel2Properties(event, element)
            : event;
        };
      }
      else Event.extend = Fuse.K;
    }
    else Event.extend = extend;

    // add methods if haven't yet
    if (!Methods) Event.addMethods();
  })();

  /*--------------------------------------------------------------------------*/

  (function() {
    var _addCache        = this.Temp.addCache,
     _addObserver        = this.Temp.addObserver,
     _createEvent        = this.Temp.createEvent,
     _fireEvent          = this.Temp.fireEvent,
     _getCacheID         = this.Temp.getCacheID,
     _getNewCacheID      = this.Temp.getNewCacheID,
     _removeCacheAtIndex = this.Temp.removeCacheAtIndex,
     _removeObserver     = this.Temp.removeObserver;

    this.fire = function fire(element, eventName, memo) {
      element = $(element);
      var event = _createEvent(element, Event.CUSTOM_EVENT_NAME);
      if (!event) return false;
      event.eventName = eventName;
      event.memo = memo || { };
      _fireEvent(element, event);
      return Event.extend(event);
    };

    this.getEventID = function getEventID() {
      // handle calls from Event object
      if (this != global) {
        var element = arguments[0];
        return (Fuse.Object.isString(element) || element.nodeType === 1)
          ? _getCacheID($(element))
          : _getCacheID(element);
      }

      // private id variable
      var id = _getNewCacheID(arguments[0]);
      // overwrite element.getEventID and execute
      return (arguments[0].getEventID = function() {
        // if cache doesn't match, request a new id
        var c = Event.cache[id];
        if (c && c.element !== this)
          id = _getNewCacheID(this);
        return id;
      })();
    };

    this.observe = function observe(element, eventName, handler) {
      element = $(element);
      var dispatcher = _addCache(element, eventName, handler);
      if (!dispatcher) return element;
      _addObserver(element, eventName, dispatcher);
      return element;
    };

    this.stopObserving = function stopObserving(element, eventName, handler) {
      element = $(element);
      eventName = Fuse.Object.isString(eventName) ? eventName : null;
      var id = _getCacheID(element), c = Event.cache[id];

      if (!c || !c.events) return element;
      var ec = c.events[eventName];

      if (ec && handler == null) {
        // If an event name is passed without a handler,
        // we stop observing all handlers of that type.
        var length = ec.handlers.length;
        if (!length) Event.stopObserving(element, eventName, 0);
        else while (length--) Event.stopObserving(element, eventName, length);      
        return element;
      }
      else if (!eventName || eventName == '') {
        // If both the event name and the handler are omitted,
        // we stop observing _all_ handlers on the element.
        for (eventName in c.events)
          Event.stopObserving(element, eventName);
        return element;
      }

      var dispatcher = ec.dispatcher, foundAt = Fuse.Object.isNumber(handler) ?
        handler : Fuse.List.Plugin.indexOf.call(ec.handlers, handler);

      if (foundAt == -1) return element;
      _removeCacheAtIndex(id, eventName, foundAt);

      if (!Event.cache[id] || !Event.cache[id].events[eventName])
      _removeObserver(element, eventName, dispatcher);

      return element;
    };

    // prevent JScript bug with named function expressions
    var fire = null, getEventID = null, observe = null, stopObserving = null;
  }).call(Event);

  /*--------------------------------------------------------------------------*/

  (function() {
    var _addObserver    = Event.Temp.addObserver,
     _createDispatcher  = Event.Temp.createDispatcher,
     _getOrCreateCache  = Event.Temp.getOrCreateCache,
     _domLoadDispatcher = _createDispatcher(2, 'dom:loaded'),
     _winLoadDispatcher = _createDispatcher(1, 'load');

    // Ensure that the dom:loaded event has finished
    // executing its observers before allowing the
    // window onload event to proceed.
    function _domLoadWrapper(event) {
      if (!Fuse._doc.loaded) {
        event = event || global.event;
        event.eventName = 'dom:loaded';

        // define pseudo private body and root properties
        Fuse._body     = Element.extend(Fuse._doc.body);
        Fuse._root     = Element.extend(Fuse._docEl);
        Fuse._scrollEl = Fuse._body;

        if (Bug('BODY_ACTING_AS_ROOT')) {
          Fuse._root = Fuse._body;
          Fuse._info.root = Fuse._info.body;
        }
        if (Bug('BODY_SCROLL_COORDS_ON_DOCUMENT_ELEMENT')) {
          Fuse._scrollEl = Fuse._docEl;
          Fuse._info.scrollEl = Fuse._info.docEl;
        }

        Fuse._doc.loaded = true;
        _domLoadDispatcher(event);
        Event.stopObserving(Fuse._doc, 'dom:loaded');
      }
    }

    function _winLoadWrapper(event) {
      event = event || global.event;
      if (!Fuse._doc.loaded)
        _domLoadWrapper(event);
      else if (Event.cache['2'] && Event.cache['2'].events['dom:loaded'])
        return Fuse.Function.defer(_winLoadWrapper, event);

      event.eventName = null;
      _winLoadDispatcher(event);
      Event.stopObserving(global, 'load');
    }

    // avoid Function#wrap for better performance esp.
    // in _winLoadWrapper which could be called every 10ms
    _addObserver(Fuse._doc, 'dom:loaded',
      _getOrCreateCache(2, Fuse._doc, 'dom:loaded').dispatcher = _domLoadWrapper);

    _addObserver(global, 'load',
      _getOrCreateCache(1, global, 'load').dispatcher = _winLoadWrapper);
  })();

  /*--------------------------------------------------------------------------*/

  delete Event.Temp;

  Fuse.Object.extend(Event, Event.Methods);

  Fuse.Object._extend(Event, {
    'KEY_BACKSPACE': 8,
    'KEY_DELETE':    46,
    'KEY_DOWN':      40,
    'KEY_END':       35,
    'KEY_ESC':       27,
    'KEY_HOME':      36,
    'KEY_INSERT':    45,
    'KEY_LEFT':      37,
    'KEY_PAGEDOWN':  34,
    'KEY_PAGEUP':    33,
    'KEY_RETURN':    13,
    'KEY_RIGHT':     39,
    'KEY_TAB':       9,
    'KEY_UP':        38
  });

  Fuse.Object._extend(Element.Methods, {
    'fire':          Event.fire,
    'getEventID':    Event.getEventID,
    'observe':       Event.observe,
    'stopObserving': Event.stopObserving
  });

  Fuse.Object._extend(Fuse._doc, {
    'loaded':        false,
    'fire':          Fuse.Function.methodize(['fire', Event]),
    'observe':       Fuse.Function.methodize(['observe', Event]),
    'stopObserving': Fuse.Function.methodize(['stopObserving', Event])
  });

  /*--------------------------- EVENT: DOM-LOADED ----------------------------*/

  // Support for the "dom:loaded" event is based on work by Dan Webb, 
  // Matthias Miller, Dean Edwards, John Resig and Diego Perini.
  (function() {
    function Poller(method) {
      var poller = this, defer = Fuse.Function.defer;
      var callback = function() {
        if (!method() && poller.id != null)
          poller.id = defer(callback);
      };
      this.id = defer(callback);
    }

    Poller.prototype.clear = function() {
      this.id != null && (this.id = global.clearTimeout(this.id));
    };

    function cssDoneLoading() {
      return (isCssLoaded = function() { return true })();
    }

    function fireDomLoadedEvent() {
      readyStatePoller.clear();
      cssPoller && cssPoller.clear();

      if (Fuse._doc.loaded) return;
      return Event.fire(Fuse._doc, 'dom:loaded');
    }

    function checkCssAndFire() {
      if (Fuse._doc.loaded) return fireDomLoadedEvent();
      return !!(isCssLoaded() && fireDomLoadedEvent());
    }

    function getSheetElements() {
      var i = 0, link, links = Fuse._doc.getElementsByTagName('LINK'),
       results = Fuse.List.fromNodeList(Fuse._doc.getElementsByTagName('STYLE'));
      while (link = links[i++])
        if (link.rel.toLowerCase() === 'stylesheet')
          results.push(link);
      return results;
    }

    function getSheetObjects(elements) {
      for (var i = 0, results = [], element, sheet; element = elements[i++]; ) {
        sheet = getSheet(element);
        // bail when sheet is null/undefined on elements
        if (sheet == null) return false;
        if (Fuse.Object.isSameOrigin(sheet.href)) {
          results.push(sheet);
          if (!addImports(results, sheet))
            return false;
        }
      }
      return results;
    }

    var cssPoller, readyStatePoller,

    checkDomLoadedState = function(event) {
      // Not sure if readyState is ever `loaded` in Safari 2.x but
      // we check to be on the safe side
      if (Fuse._doc.loaded) readyStatePoller.clear();
      else if (event && event.type === 'DOMContentLoaded' ||
          /^(loaded|complete)$/.test(Fuse._doc.readyState)) {
        readyStatePoller.clear();
        Fuse._doc.stopObserving('readystatechange', checkDomLoadedState);
        if (!checkCssAndFire()) cssPoller = new Poller(checkCssAndFire);
      }
    },

    addImports = function(collection, sheet) {
      return (addImports = isHostObject(sheet, 'imports')
        ? function(collection, sheet) {
            var length = sheet.imports.length;
            while (length--) {
              if (Fuse.Object.isSameOrigin(sheet.imports[length].href)) {
                collection.push(sheet.imports[length]);
                addImports(collection, sheet.imports[length])
              }
            }
            return collection;
          }
        : function(collection, sheet) {
            // Catch errors on partially loaded elements. Firefox may also
            // error when accessing css rules of sources using the file:// protocol
            try {
              var ss, rules = getRules(sheet), length = rules.length;
            } catch(e) {
              return false;
            }
            while (length--) {
              // bail when sheet is null on rules
              ss = rules[length].styleSheet;
              if (ss === null) return false;
              if (ss && Fuse.Object.isSameOrigin(ss.href)) {
                collection.push(ss);
                if (!addImports(collection, ss))
                  return false;
              }
            }
            return collection;
          }
      )(collection, sheet);
    },

    getStyle = function(element, styleName) {
      return (getStyle = Feature('ELEMENT_COMPUTED_STYLE')
        ? function(element, styleName) {
            var style = element.ownerDocument.defaultView.getComputedStyle(element, null);
            return (style || element.style)[styleName];
          }
        : function(element, styleName) {
            return (element.currentStyle || element.style)[styleName];
          }
      )(element, styleName);
    },

    getSheet = function(element) {
      return (getSheet = isHostObject(element, 'styleSheet')
        ? function(element) { return element.styleSheet }
        : function(element) { return element.sheet }
      )(element);
    },

    getRules = function(sheet) {
      return (getRules = isHostObject(sheet, 'rules')
        ? function(sheet) { return sheet.rules }
        : function(sheet) { return sheet.cssRules }
      )(sheet);
    },

    addRule = function(sheet, selector, cssText) {
      return (addRule = isHostObject(sheet, 'addRule')
        ? function(sheet, selector, cssText) { return sheet.addRule(selector, cssText) }
        : function(sheet, selector, cssText) { return sheet.insertRule(selector +
            '{' + cssText + '}', getRules(sheet).length) }
      )(sheet, selector, cssText);
    },

    removeRule = function(sheet, index) {
      return (removeRule = isHostObject(sheet, 'removeRule')
        ? function(sheet, index) { return sheet.removeRule(index) }
        : function(sheet, index) { return sheet.deleteRule(index) }
      )(sheet, index);
    },

    isCssLoaded = function() {
      var sheetElements = getSheetElements();
      return !sheetElements.length
        ? cssDoneLoading()
        : (isCssLoaded = function() {
            var cache = [];
            return !(function() {
              var sheets = getSheetObjects(sheetElements);
              if (!sheets) return false;

              var className, length = sheets.length;
              while (length--) {
                className = 'fuse_css_loaded_' + cache.length;
                cache.push({ 'className': className, 'sheet': sheets[length] });
                addRule(sheets[length], '.' + className, 'margin-top: -1234px!important;');
              }
              return true;
            })()
              ? false
              : (isCssLoaded = function() {
                  var c, lastIndex, rules, length = cache.length, done = true;
                  while (length--) {
                    c = cache[length];
                    rules = getRules(c.sheet);
                    lastIndex = rules.length && rules.length - 1;

                    // if styleSheet was still loading when test rule
                    // was added it will have removed the rule.
                    if (rules[lastIndex].selectorText.indexOf(c.className) > -1) {
                      done = false;

                      // if the styleSheet has only the test rule then skip
                      if (rules.length === 1) continue;

                      if (!c.div) {
                        c.div = Fuse._doc.createElement('div');
                        c.div.className = c.className;
                        c.div.style.cssText = 'position:absolute;visibility:hidden;';
                      }

                      Fuse._doc.body.appendChild(c.div);

                      // when loaded clear cache entry
                      if (getStyle(c.div, 'marginTop') === '-1234px')
                        cache.splice(length, 1);

                      // cleanup
                      removeRule(c.sheet, lastIndex);
                      Fuse._doc.body.removeChild(c.div);
                    }
                  }

                  if (done) {
                    cache = null;
                    return cssDoneLoading();
                  }
                  return done;
                })();
          })();
    };

    /*------------------------------------------------------------------------*/

    // Ensure the document is not in a frame because
    // doScroll() will not throw an error when the document
    // is framed. Fallback on document readyState.
    if (!Feature('ELEMENT_ADD_EVENT_LISTENER') && Feature('ELEMENT_DO_SCROLL')) {

      // Avoid a potential browser hang when checking global.top (thanks Rich Dougherty)
      // Checking global.frameElement could throw if not accessible.
      var isFramed = true;
      try { isFramed = global.frameElement != null } catch(e) { }

      // Derived with permission from Diego Perini's IEContentLoaded
      // http://javascript.nwbox.com/IEContentLoaded/
      if (!isFramed)
        checkDomLoadedState = function() {
          if (Fuse._doc.loaded) readyStatePoller.clear();
          else {
            if (Fuse._doc.readyState === 'complete')
              fireDomLoadedEvent();
            else {
              try { Fuse._div.doScroll() } catch(e) { return }
              fireDomLoadedEvent();
            }
          }
        };
    }
    else if (Feature('ELEMENT_ADD_EVENT_LISTENER'))
      Fuse._doc.observe('DOMContentLoaded', checkDomLoadedState);

    // readystate and poller are used (first one to complete wins)
    Fuse._doc.observe('readystatechange', checkDomLoadedState);
    readyStatePoller = new Poller(checkDomLoadedState);
  })();

  Fuse.updateGenerics();
  Element.addMethods();
})(this);
